/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/



#ifndef APPTYPES_H
#define APPTYPES_H

#ifdef __cplusplus
extern "C" {
#endif
  
#define APPTYPES_COMMAND_TARGET_SIZE (5)
  
  
  
  typedef enum
  {
    apptypesNoAction,
    apptypesConfirm,
    apptypesReject,
    apptypesSetAddress,
    apptypesSetLRQResolvedRAS,
    apptypesSetLRQResolvedCallSignal,
    appTypesTransfer,
    apptypesHunt
  } apptypesAction;
  
  typedef enum
  {
    apptypesNoTransaction,
    apptypesGRQ,
    apptypesRRQ,
    apptypesURQ,
    apptypesLRQ,
    apptypesCall
  } apptypesTransaction;
  
  typedef enum
  {
    apptypesAddressTypeNone,
    apptypesAddressTypeName,
    apptypesAddressTypePhone,
    apptypesAddressTypeURL,          /* version 2 */
    apptypesAddressTypeEmail,        /* version 2 */
    apptypesAddressTypeTransName,    /* version 2 */
    apptypesAddressTypePartyNumber,  /* version 2 */
    apptypesAddressTypeIpPort
  } apptypesAddressType;
  
  typedef struct
  {
    INT32 ip;
    UINT16 port;
  } apptypesIpPort;
  
  typedef struct
  {
    apptypesAddressType type;
    gkaddName_t  name;
    gkaddPhone_t phone;
    gkaddPhone_t url;   /* version 2 */
    gkaddPhone_t email; /* version 2 */
    gkaddPN_t    partyNumber; /* version 2 */
    apptypesIpPort transname; /* version 2 */
    apptypesIpPort ipPort;
  } apptypesAddress;
  
  typedef struct
  {
    apptypesTransaction transaction;
    apptypesAddress id;
    apptypesAction action;
    apptypesAddress target[APPTYPES_COMMAND_TARGET_SIZE];
  } apptypesCommand;
  
  typedef struct
  {
    apptypesCommand* commands;
    int size;/*the size of the commandsArray that commands points to*/
  } apptypesCommandTable;
  
  
  
  
  /*------------------------------------------------------*/
  /*                  PUBLIC  FUNCTIONS                   */
/*------------------------------------------------------*/
  
  
  void apptypesIpPortReset(
			   OUT apptypesIpPort* ipPort);
  
  BOOL apptypesIpPortEmpty(
			   IN apptypesIpPort* ipPort);
  
  int apptypesIpPortCmp(/*returns 0 iff the ipPort1 equals ipPort2. Equality is defined as follows :
			  a : if both ip and port are identical
			  b : ip's are identical and either one or both ports are 0.
			  NOTE: nonZero value returned if one or both addresses are empty*/
			IN apptypesIpPort* ipPort1,
			IN apptypesIpPort* ipPort2);
  
  void apptypesCommandReset(
			    OUT apptypesCommand* command);
  
  BOOL apptypesCommandEmpty(
			    IN apptypesCommand* c);
  
  void apptypesAddressReset(
			    OUT apptypesAddress* address);
  
  BOOL apptypesAddressEmpty(
			    IN apptypesAddress* a);
  
  int apptypesAddressCmp(/*returns 0 iff addressess are equal.
			   NOTE: nonZero value returned if one or both addresses are empty.
			   NOTE that the equality of the ipPort field is defined by the apptypesIpPortCmp function*/
			 IN apptypesAddress* a1,
			 IN apptypesAddress* a2);
  
  void apptypesAddressSetIpPort(/*resets a and sets it with the proper value*/
				IN OUT apptypesAddress* a,
				IN apptypesIpPort* ipPort);
 

  void apptypesAddressSetName(/*sets it with the proper value*/
			      IN OUT apptypesAddress* a,
			      IN gkaddName_t name);
  
  void apptypesAddressSetPhone(/*sets it with the proper value*/
			       IN OUT apptypesAddress* a,
			       IN gkaddPhone_t phone);
  
  void apptypesAddressSetAlias(/*sets it with the proper value*/
			       IN OUT apptypesAddress* a,
			       IN gkaddAlias_t* alias);

  void apptypesAddressSetRgi(/*reset appAddress and sets it with the value of the gkrgi type address specified by gkAddress and p*/
			     IN OUT apptypesAddress* appAddress,
			     IN void* gkAddress,/*should be a pointer gkaddTrans_t gkaddName_t or gkaddPhone_th*/
			     IN gkrgiParameter_t p);

void apptypesAddressSetLrqi(/*reset appAddress and sets it with the value of the gklrqgi type address specified by gkAddress and p*/
			   IN OUT apptypesAddress* appAddress,
			   IN void* gkAddress,/*should be a pointer gkaddTrans_t gkaddName_t or gkaddPhone_th*/
			   IN gklrqiParameter_t p);

void apptypesAddressSetCci(/*reset appAddress and sets it with the value of the gkcci type address specified by gkAddress and p*/
			   IN OUT apptypesAddress* appAddress,
			   IN void* gkAddress,/*should be a pointer gkaddTrans_t gkaddName_t or gkaddPhone_th*/
			   IN gkcciParameter_t p);
       
  BOOL apptypesCommandTableAllocate(/*deletes the previously allocated command array and allocates and resets a new one.
				      FALSE returned if memoryAllocation fails*/
				    OUT apptypesCommandTable* ct,
				    IN int size);
  
  void apptypesCommandPrint(
			    IN apptypesCommand* c);
  
  void apptypesCommandTablePrint(
				 IN apptypesCommandTable* ct);
  
  const char* apptypesTransactionString(
					IN apptypesTransaction t);
  
  const char* apptypesActionString(
				   IN apptypesAction a);
  
  
  const char* apptypesAddressString(
				    IN apptypesAddress* a);
  
  const char* apptypesIpPortString(
				   IN apptypesIpPort* ipPort);
  
  int apptypesCommandTableFind(/*returns the the command in commandTable whose id field matches 'commandId'.
				 return value: the index of the matching command, or -1 if none found*/
			       IN apptypesCommandTable* ct,
			       IN apptypesAddress* id);

  BOOL apptypesAddressToAlias(/*if address is of neither name or phone types, it is reset and FALSE returned*/
			      OUT gkaddAlias_t* alias,
			      IN apptypesAddress* address);

  void apptypesIpPort2Trans(
			    OUT gkaddTransport_t* trans,
			    IN apptypesIpPort* ipPort);
  
  void apptypesTrans2IpPort(
			    OUT apptypesIpPort* ipPort,
			    IN gkaddTransport_t* trans);
  
  BOOL apptypesCommandAddTarget(/*if unable to add (target array is full), FALSE is returned*/
				IN OUT apptypesCommand* c,
				IN apptypesAddress* a);
  
  BOOL apptypesCommandTableAddCommand(/*if unable to add (commandTable is full), FALSE is returned*/ 
				      IN OUT apptypesCommandTable* ct,
				      IN apptypesCommand* command);
  
  
  int apptypesIdentifyRgiCommand(/*uses the parameter p to identify the matching command. If found, its index is returned.
				   otherwise -1 is returned*/
				 IN apptypesCommandTable* ct,
				 IN gkHREG hReg,
				 IN gkrgiParameter_t p,
				 IN gkrgiMessage_t m);
  
  int apptypesIdentifyLrqiCommand(/*uses the parameter p to identify the matching command. If found, its index is returned.
				    otherwise -1 is returned*/
				  IN apptypesCommandTable* ct,
				  IN gkHLRQ hLRQ,
				  IN gklrqiParameter_t p,
				  IN gklrqiMessage_t m);
  
  int apptypesIdentifyCciCommand(/*uses the parameter p to identify the matching command. If found, its index is returned.
				   otherwise -1 is returned*/
				 IN apptypesCommandTable* ct,
				 IN gkHCALL hCall,
				 IN gkcciParameter_t p,
				 IN gkcciSide_t s);
  
  BOOL apptypesEmptyName(gkaddName_t name);
  
  BOOL apptypesEmptyPhone(gkaddPhone_t phone);
  
  BOOL apptypesEmptyTransport(gkaddTransport_t* trans);
  
  BOOL apptypesEmptyAlias(/*TRUE if variable alias is empty*/
			  IN gkaddAlias_t *alias);
  
  void apptypesResetTransport(/*reset the transport address to the empty address */
			      IN gkaddTransport_t *trans);
  
  void apptypesResetName(/*reset the name alias to the empty alias */
			 IN gkaddName_t name);
  
  void apptypesResetPhone(/*reset the phone alias to the empty alias */
			  gkaddPhone_t phone);
  
  BOOL apptypesEmptyAlias(/*TRUE if variable alias is empty*/
			  IN gkaddAlias_t *alias);
  
  void apptypesResetAlias(/*reset the alias to the empty alias */
			  IN gkaddAlias_t  *alias);
  
#ifdef __cplusplus
	   }
#endif

#endif

