/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/


/**********************************************************
		   PUBLIC  DECLARATIONS
*=========================================================
* TITLE: configutl.c
*---------------------------------------------------------
* PROJECT: GK CONFIGURATION FOR PC
*---------------------------------------------------------
* $Workfile$
*---------------------------------------------------------
* $Author$ NATHANIEL LEIBOWITZ
*---------------------------------------------------------
* $Date$ 3/6/98
*---------------------------------------------------------
* $Revision$ This file was cloned and adapted from the
*            sample application of the UNIX platform
**********************************************************/
#include "windowsx.h"
#include "windows.h"
#include "commctrl.h"

#include "rvcommon.h"
#include "gkadd.h"
#include "cfgapi.h"
#include "gkrgi.h"
#include "gklrqi.h"
#include "gkTypes.h"
#include "gkcci.h"
#include "gkmain.h"

#include "apptypes.h"
#include "configutl.h"

#include "ctype.h"

/********************************************************/
/*                     LOCAL DECLARATIONS               */
/********************************************************/

/*-----------------*/
/* LOCAL CONSTANTS */
/*-----------------*/

/*-----------------*/
/*  LOCAL MACROS   */
/*-----------------*/


/*-----------------*/
/*   LOCAL TYPES   */
/*-----------------*/


/*-----------------*/
/* LOCAL VARIABLES */
/*-----------------*/



extern HWND cfghWnd; //SARAH5

/*------------------------------------------------------*/
/*            PROTOTYPES OF LOCAL FUNCTIONS             */
/*------------------------------------------------------*/

/* removes 'isspace' characters from begining and end of line. 
   If line becomes empty FAlSE is returned*/
BOOL cleanTheLine(IN OUT char* line);

/*------------------------------------------------------*/
/*                  PUBLIC  FUNCTIONS                   */
/*------------------------------------------------------*/

BOOL utilErrorHandler(char *errmsg)
{
  MessageBox(cfghWnd, errmsg, "error", MB_OK | MB_ICONEXCLAMATION);
  return FALSE;
}

BOOL utilSearchForKeyword(/* search for the string passed as the second
                         argument at the file passed as the first one. 
                         The string should be at the start of a new line, 
                         and have nothing afterward except blanks etc.*/
                         IN FILE *fd, 
                         IN char *str)
{
  char buf[1000];
 
  fseek(fd, 0L, SEEK_SET);

  while(fgets(buf,999,fd))
  {
    if(strncmp(str,buf,strlen(str)) == 0 )
      return TRUE;
  }
  return FALSE;
}

BOOL utilReadNextLine(/* reads the next line. ignore lines that start with white space,
                     lines that only contain new-line, or lines that start with
                     the comment sign (#) at the first column. If the EOF is reached or
                     another section ("[something]") encountered, return FALSE */
                     IN FILE *fd, 
                     IN char *str,
                     IN int  size)
{
  char* p;
  while(fgets(str,size-1,fd))
  {
    if((*str == '\n') || (*str == 0))
     continue;
    if ((p = strstr(str,"#")))
      *p = 0;
    if (*str == 0)
      continue;
    if(*str == '[')
      return FALSE;
    if (!cleanTheLine(str))
      continue;
    return TRUE;
  }
  return FALSE;
}

BOOL utilSplit(/*the head of str up to delimitor (not-inclusive) is copied to head and the tail of
		 the string from the delimitor (not inclusive) is copied to tail. Both head and tail
		 are returned null terminated. FALSE returned if delimitor not found.
		 NOTE: delimitor must be null terminated string. Also
		 the delimitor "," refers to commaSeparators i.e. a comma that is not preceded by backslash.
		 Such commas denote a separator between fields. Commas that are part of strings will
		 appear as "\," and will not be regarded as a separator*/
	       IN char* str,
	       IN char* delimitor,
	       OUT char* head,
	       OUT char* tail)
{
  int headSz;
  char *tailAddress,*delAddress,*dummy;
  if (strcmp(delimitor,",") == 0)
    delAddress = utilFirstCommaSeparator(str,&dummy);
  else
    delAddress = strstr(str,delimitor);
  if (!delAddress)
    return FALSE;/*utilErrorHandler("delimitor not found in utilSplit");*/
  headSz = delAddress - str;
  tailAddress = delAddress+strlen(delimitor);
  memcpy(head,str,headSz);
  head[headSz] = 0; 
  strcpy(tail,tailAddress);
  return TRUE;
}
  
BOOL utilReadINT32(/*reads from str the first int and returns T/F on success*/
		   OUT INT32* val,
		   IN char* str)
{
  
  if (sscanf(str,"%i",val) != 1)
    return utilErrorHandler("utilReadInt");
  else
    return TRUE;
}

BOOL utilReadUINT16(/*reads from str the first UINT16 and returns T/F on success*/
		 OUT UINT16* val,
		 IN char* str)
{
  UINT32 tmp;
  if (sscanf(str,"%u",&tmp) != 1)
    return utilErrorHandler("utilReadUINT16");
  else
    *val = (UINT16)tmp;
  return TRUE;
}


BOOL utilIpNum(/*converts a string with ip in format ddd.ddd.ddd.ddd to its numerical value
		 FALSE returned if it is not a valid ip*/
	       OUT UINT32* ip,
	       IN char* ipString)
{
  unsigned char * ipAdd = (unsigned char*)ip;
  int i0,i1,i2,i3;
  if (!ipString || strlen(ipString) == 0) {
    utilErrorHandler("utilReadIp with empyt or null ipString");
    return FALSE;
  }
  sscanf(ipString,"%d.%d.%d.%d",&i0,&i1,&i2,&i3);
  if (!utilValidIpSegment(i0) || !utilValidIpSegment(i1) || !utilValidIpSegment(i2) || !utilValidIpSegment(i3)) {
    utilErrorHandler("utilReadIp with illegal ip segment");
    return FALSE;
  }
  ipAdd[0] = i0;
  ipAdd[1] = i1;
  ipAdd[2] = i2;
  ipAdd[3] = i3;
  return TRUE;
}

BOOL utilReadIpAndPort(/*parses the string into ip part and port part.
			 str must be in the format ddd.ddd.ddd.ddd:pppp
			 where each of the fields conforms to the usual
			 ip and port definitions. FALSE returned on error*/
		       IN char* str,
		       OUT UINT32* ip,
		       OUT UINT16* port)
{
  UINT16 tmp;
  char ipBuf[100],portBuf[50];
  if (!utilSplit(str,":",ipBuf,portBuf))
    return utilErrorHandler("when splitting in utilReadIpAndPort");
  if (!utilIpNum(ip,ipBuf) || !utilReadUINT16(&tmp,portBuf))
    return utilErrorHandler("utilReadIpAndPort could not parse the str");
  *port = tmp;
  return TRUE;
}

BOOL utilReadIp(/*parses the string into ip part.
		  str must be in the format ddd.ddd.ddd.ddd
		  where each of the fields conforms to the usual
		  ip definitions. FALSE returned on error*/
		IN char* str,
		OUT UINT32* ip)
{
  if (!utilIpNum(ip,str))
    return utilErrorHandler("utilReadIp could not parse the str");
  return TRUE;
}

BOOL utilValidIpSegment(/*Positive if 'ddd' is a valid value in the ip format 'ddd.ddd.ddd.ddd'.
			  Otherwise negative value is returned*/
			IN INT32 ddd)
{
  return ( (ddd>=0) && (ddd<256) );
}

char* utilParseNextEquation(/*finds the first '"' in the string, and sets lhsBuf with
			      the string till the =(non- inclusive), and sets the rhsBuf with
			      the remaining string until the equation terminates. The equation
			      terminates either at end of sting or when a comma is encountered.
			      RETURN VAL: if the string's end was reached or no '=' is found
			      NULL is returned. Otherwise
			      pointer to the char beyond the comma is returned or NULL if the string 
			      ends with the comma*/
			    IN char* str,
			    OUT char* lhsBuf,
			    OUT char* rhsBuf)
{
  char* returnVal;
  char tmp[1000];
  char dummy[1000];

  utilFirstCommaSeparator(str,&returnVal);
  if (!utilSplit(str,"=",lhsBuf,tmp))
    return NULL;
  if (!utilSplit(tmp,",",rhsBuf,dummy))
    strcpy(rhsBuf,tmp);
  utilRemoveLeadingBackslashes(rhsBuf); 
  return returnVal;
}

BOOL utilParseStatusPrefixedAddress(/*reads address of the format {status}ADDRESS. 'address'
				      is returned as null terminated string.
				      NOTE: commas and backslashes are taken as part 
				      of the address. Caller should first 'clean' the prefixedAddress
				      string for example by calling utilParseNextEquation or
				      utilRemoveLeadingBackslashes
				      False returned if no formatted address was found,
				      or strlen(prefixedAddress) > 1000*/
				    IN char* prefixedAddress,/*limited to 500 chars*/
				    OUT gkrgiParameterStatus_t* status,
				    OUT char* address)
{
  char head[1000];
  
  if (strlen(prefixedAddress) > 1000)
    return utilErrorHandler("!! utilParseStatusPrefixedAddress called with prefixedAddress longer than 500 chars");
  if (utilSplit(prefixedAddress,"{s}",head,address))
    *status = gkrgiSttsStatic;
  else {
    if (utilSplit(prefixedAddress,"{d}",head,address))
      *status = gkrgiSttsDynamic;
    else
      return FALSE;
  }
  return TRUE;			
}	      

BOOL utilParseTypePrefixedAddress(/*reads address of the format {type}ADDRESS. 'address'
				    is returned as null terminated string.
				    NOTE: commas and backslashes are taken as part 
				    of the address. Caller should first 'clean' the prefixedAddress
				    string for example by calling utilParseNextEquation or
				    utilRemoveLeadingBackslashes
				    False returned if no formatted address was found,
				    or strlen(prefixedAddress) > 1000*/
				  IN char* prefixedAddress,/*limited to 500 chars*/
				  OUT apptypesAddress* address)
{
  char head[1000],tail[1000];
  
  if (strlen(prefixedAddress) > 1000)
    return utilErrorHandler("!! utilParseTypePrefixedAddress called with prefixedAddress longer than 500 chars");
  if (utilSplit(prefixedAddress,"{ras}",head,tail) || utilSplit(prefixedAddress,"{cs}",head,tail)) {
    address->type = apptypesAddressTypeIpPort;
    if (strstr(tail,":")) {
      if (!utilReadIpAndPort(tail,&address->ipPort.ip,&address->ipPort.port))
	return utilErrorHandler("error in utilParseTypePrefixedAddress with ras or cs");
    }
    else {
      if (!utilReadIp(tail,&address->ipPort.ip))
	return utilErrorHandler("error in utilParseTypePrefixedAddress with ras or cs");
      address->ipPort.port = 0;
    }
  }
  else if(utilSplit(prefixedAddress,"{name}",head,tail)) {
    address->type = apptypesAddressTypeName;
    strcpy(address->name,tail);
  } 
  else if(utilSplit(prefixedAddress,"{phone}",head,tail)) {
    address->type = apptypesAddressTypePhone;
    strcpy(address->phone,tail);
  }
  else 
    return FALSE;
  return TRUE;
}

char* utilFirstCommaSeparator(/*finds the first comma that is not preceeded by a single backslash.
				a pointer to it is returned or null if not found. Also, 
				nextChar will point to the place after the comma, or will be 
				NULL if the comma is at the end of the string*/
			      IN char* str,
			      OUT char** nextChar)
{
  char* comma = str;
  char* end = &(str[strlen(str)-1]);
  if (str[0] == ',') {
    if (strlen(str) > 1)
      *nextChar = &(str[1]);
    else
      *nextChar = NULL;
    return str;
  }
  if (str[1] == ',' && str[0] != '\\') {
    if (strlen(str) > 2)
      *nextChar = &(str[2]);
    else
      *nextChar = NULL;
    return &str[1];
  }
  else {
    if (strlen(str) > 2)
      comma = &(str[2]);
    else {
      *nextChar = NULL;
      return NULL;
    }
  }
  while ((comma = strstr(comma,",")) != NULL && (*(comma-1) == '\\' && *(comma-2) != '\\') && comma != end)
    comma++;
  if (comma != NULL && *comma == ',' && !(*(comma-1) == '\\' && *(comma-2) != '\\')) {
    if (comma == end)
      *nextChar = NULL;
    else
      *nextChar = (comma+1);    
  }
  else {
    comma = NULL;
    *nextChar = NULL;
  }
  return comma;
}
  
//  BOOL utilSetTransport(/*friendly interface for the gkrgiAddAddress for transport addresses only*/
/*		      IN gkHREG hReg,
		      IN gkrgiParameter_t param,
		      IN gkrgiParameterStatus_t status,
		      IN char* ipStr)
{
  INT32 result;
  INT32 ip;
  UINT16 port;
  gkaddTransport_t trans;
  apptypesResetTransport(&trans);
  if (!utilReadIpAndPort(ipStr,&ip,&port))
    return utilErrorHandler("error in utilSetTransport");
  trans.ip = (UINT32)ip;
  trans.port = port;
  if ((result = gkrgiAddAddress(hReg,param,status,&trans)) < 0)
      return utilErrorHandler("error in gkrgiAddAddress(gkrgiParamRAS) called by utilSetTransport");
  return TRUE;
}*/

BOOL utilStr2EndpType(/*returns the gkaddEndpointType corresponding to the str.
			FALSE returned if no correspondence found*/
		      OUT gkaddEndpointType* endpType,
		      IN char* str)
{
  if (strcmp(str,"TERMINAL") == 0) {
    *endpType = gkaddEndpointTypeTerminal;
      return TRUE;
  }
  if (strcmp(str,"GW") == 0) {
    *endpType = gkaddEndpointTypeGateway ;
      return TRUE;
  }
  if (strcmp(str,"MCU") == 0) {
    *endpType = gkaddEndpointTypeMCU;
      return TRUE;
  }
  if (strcmp(str,"GK") == 0) {
    *endpType = gkaddEndpointTypeGK;
      return TRUE;
  }
  if (strcmp(str,"UNDEFINED") == 0) {
    *endpType = gkaddEndpointTypeUndefined;
      return TRUE;
  }
  return utilErrorHandler("undefined string in utilStr2EndpType");
}

//BOOL utilIndexOfServicePrefix (/*returns the index of the service that has the specified prefix.
/*				if none found, FALSE is returned*/
/*			      OUT INT32* index,
			      IN char* prefix)
{
  int i,maxIndex;
  gkcfgiService_t service;
 
  maxIndex = gkcfgiSrvcMaxNumGet();
  for (i = 1;i<=maxIndex;i++) {
    if (gkcfgiSrvcGet(i,&service) == GKRGI_OK)
      if (strcmp(service.prefix,prefix) == 0) {
	*index = i;
	return TRUE;
      }
  }
  return FALSE;
}*/

void utilRemoveLeadingBackslashes(/*removes leading backslashes from string. i.e 
				    all single backslashes are deleted one backslash from each pair
				    is removed.Behaviour when a triplet
				    or more backslashes is encountered is undefined.*/
				  IN OUT char* str)
{
  int i,sz,lag=0;
  if (!str)
    return;
  sz = strlen(str);
  for (i = 0;i < sz;i++) {
    if (str[i] == '\\' && str[i+1] != '\\')
      lag++;
    else
      str[i-lag] = str[i];
  }
  if (str[sz-1] == '\\')
    lag++;
  else
    str[sz-lag] = str[sz];
  str[sz-lag] = 0;
}


/*------------------------------------------------------*/
/*                 LOCAL  FUNCTIONS                   */
/*------------------------------------------------------*/



BOOL cleanTheLine(/*removes 'isspace' characters from begining and end of line. If line becomes empty
		    FAlSE is returned*/
		  IN OUT char* line)
{
  int sz,spaces=0;
  int i = strlen(line)-1;
  if ( i == -1)
    return FALSE;
  while (isspace(line[i])/*(line[i] == '\n' || line[i] == ' ' || line[i] == '\t')*/ && (i >= 0))
    line[i--] = 0;
  
  sz = strlen(line);
  while (spaces < sz && isspace(line[spaces]))
    spaces++;
  if (spaces) {
    for (i = spaces;i < sz;i++) 
      line[i-spaces] = line[i];
    line[sz-spaces] = 0;
  }
  return strlen(line)>0; 
}
