/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/



#ifndef SAMAPP_UTILITIES_H
#define SAMAPP_UTILITIES_H


#ifdef __cplusplus
extern "C" {
#endif
  
#define UTIL_GATEKEEPER_CONFIG_FILE "gkmem.ini"

	/*yonatan 30.9: taken from apptypes.h*/  


  
  /*------------------------------------------------------*/
  /*                  PUBLIC  FUNCTIONS                   */
  /*------------------------------------------------------*/
  
  
  BOOL utilSearchForKeyword(/* search for the string passed as the second
			       argument at the file. The string should be
			       at the start of a new line, and have nothing afterward
			       except blanks etc.*/
			    IN FILE *fd, 
			    IN char *str);
  
  BOOL utilReadNextLine(/* reads the next line. ignore empty lines, or lines that start with
			   the comment sign # at the first column. If the EOF is reached or
			   another section ("[something]") encountered, return FALSE */
			IN FILE *fd, 
			IN char *str,
			IN int  size);
  
  BOOL utilErrorHandler(/*prints the error message and returns FALSE*/
			IN char* errorMessage);
  
  INT32 utilApiErrorHandler(/*prints the error value and error message and returns the requested returnVal*/
			    IN INT32 errorVal,
			    IN char* errorMessage,
			    IN INT32 returnVal);

  BOOL utilSplit(/*the head of str up to delimitor (not-inclusive) is copied to head and the tail of
		   the string from the delimitor (not inclusive) is copied to tail. Both head and tail
		   are returned null terminated. FALSE returned if delimitor not found.
		   NOTE: delimitor must be null terminated string. Also
		   the delimitor "," refers to commaSeparators i.e. a comma that is not preceded by backslash.
		   Such commas denote a separator between fields. Commas that are part of strings will
		   appear as "\," and will not be regarded as a separator*/
		 IN char* str,
		 IN char* delimitor,
		 OUT char* head,
		 OUT char* tail);

  char* utilFirstCommaSeparator(/*finds the first comma that is not preceeded by a single backslash.
				  a pointer to it is returned or null if not found. Also, 
				  nextChar will point to the place after the comma, or will be 
				  NULL if the comma is at the end of the string*/
				IN char* str,
				OUT char** nextChar);
  
  char* utilParseNextEquation(/*finds the first '"' in the string, and sets lhsBuf with
				the string till the =(non- inclusive), and sets the rhsBuf with
				the remaining string until the equation terminates. The equation
				terminates either at end of sting or when a comma is encountered.
				RETURN VAL: if the string's end was reached or no '=' is found
				NULL is returned. Otherwise
				pointer to the char beyond the comma is returned or NULL if the string 
				ends with the comma*/
			      IN char* str,
			      OUT char* lhsBuf,
			      OUT char* rhsBuf);

  BOOL utilParseStatusPrefixedAddress(/*reads address of the format {status}ADDRESS. 'address'
					is returned as null terminated string.
					NOTE: commas and backslashes are taken as part 
					of the address. Caller should first 'clean' the prefixedAddress
					string for example by calling utilParseNextEquation or
					utilRemoveLeadingBackslashes
					False returned if no formatted address was found,
					or strlen(prefixedAddress) > 1000*/
				    IN char* prefixedAddress,/*limited to 500 chars*/
				    OUT gkrgiParameterStatus_t* status,
				    OUT char* address);
  
/*  BOOL utilParseTypePrefixedAddress(/*reads address of the format {type}ADDRESS. 'address'
				      is returned as null terminated string.
				      NOTE: commas and backslashes are taken as part 
				      of the address. Caller should first 'clean' the prefixedAddress
				      string for example by calling utilParseNextEquation or
				      utilRemoveLeadingBackslashes
				      False returned if no formatted address was found,
				      or strlen(prefixedAddress) > 1000*/
/*				    IN char* prefixedAddress,/*limited to 500 chars*/
//				    OUT apptypesAddress* address);
  
  BOOL utilReadINT32(/*reads from str the first int and returns T/F on success*/
		     OUT INT32* val,
		     IN char* str);
  
  BOOL utilReadUINT16(/*reads from str the first UINT16 and returns T/F on success*/
		      OUT UINT16* val,
		      IN char* str);

  BOOL utilIpNum(/*converts a string with ip in format ddd.ddd.ddd.ddd to its numerical value
		   FALSE returned if it is not a valid ip*/
		 OUT UINT32* ip,
		 IN char* ipString);
  
  BOOL utilReadIpAndPort(/*parses the string into ip part and port part.
			   str must be in the format "ddd.ddd.ddd.ddd:pppp"
			   where each of the fields conforms to the usual
			   ip and port definitions. FALSE returned on error*/
			 IN char* str,
			 OUT UINT32* ip,
			 OUT UINT16* port);

  BOOL utilReadIp(/*parses the string into ip part.
		    str must be in the format ddd.ddd.ddd.ddd
		    where each of the fields conforms to the usual
		    ip definitions. FALSE returned on error*/
		  IN char* str,
		  OUT UINT32* ip);
  
  BOOL utilValidIpSegment(/*Positive if 'ddd' is a valid value in the ip format 'ddd.ddd.ddd.ddd'.
			    Otherwise negative value is returned*/
			  IN INT32 ddd);
  
  BOOL utilSetTransport(/*friendly interface for the gkrgiAddAddress for transport addresses only*/
			IN gkHREG hReg,
			IN gkrgiParameter_t param,
			IN gkrgiParameterStatus_t status,
			IN char* ipStr);		     
  
  
  BOOL utilSetTransport(/*friendly interface for the gkrgiAddAddress for transport addresses only*/
			IN gkHREG hReg,
			IN gkrgiParameter_t param,
			IN gkrgiParameterStatus_t status,
			IN char* ipStr);
  
  BOOL utilStr2EndpType(/*returns the gkaddEndpointType corresponding to the str.
			  FALSE returned if no correspondence found*/
			OUT gkaddEndpointType* endpType,
			IN char* str);
  
  BOOL utilIndexOfServicePrefix(/*returns the index of the service that has the specified prefix.
				  if none found, FALSE is returned*/
				OUT INT32* index,
				IN char* prefix);

  void utilRemoveLeadingBackslashes(/*removes leading backslashes from string. i.e 
				      all single backslashes are deleted one backslash from each pair
				      is removed.Behaviour when a triplet
				      or more backslashes is encountered is undefined.*/
				    IN OUT char* str);
       
#ifdef __cplusplus
	   }
#endif


#endif




