/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/


/**********************************************************
		   PUBLIC  DECLARATIONS
*=========================================================
* TITLE: mcLogOptions.c
*---------------------------------------------------------
* PROJECT: MC CONFIGURATION FOR PC
*---------------------------------------------------------
* $Workfile$
*---------------------------------------------------------
* $Author$ YONATAN BECKER
*---------------------------------------------------------
* $Date$ 21/10/99
*---------------------------------------------------------
* $Revision$
**********************************************************/

/********************************************************/
/*                     INTERFACE DECLARATIONS           */
/********************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN32
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#endif

#include <cm.h>

#include "resource.h"

#include "rvcommon.h"
#include "mci_types.h"
#include "mci_report.h"
#include "mci_general.h"

#include "gkadd.h"
#include "gkrgi.h"
#include "gkcci.h"

#include "mci_chan.h"
#include "mci_call.h"
#include "mci_conf.h"
#include "mci_mc.h"
#include "mci_call_caps.h"

#include "gkmc.h"
#include "mcgki.h"
#include "mcconfig.h"
#include "mcmain.h"
#include "mcrefresh.h"

/*-----------------*/
/* LOCAL VARIABLES */
/*-----------------*/

static UINT32 logClasses=0;
static logLevel_t logLevel=0;


void initDebugLevelCB(HWND hWnd)
{
	SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,CB_ADDSTRING,0,(LPARAM)"Exceptions");
	SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,CB_ADDSTRING,0,(LPARAM)"BasicFlow");
	SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,CB_ADDSTRING,0,(LPARAM)"States");
	SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,CB_ADDSTRING,0,(LPARAM)"DataBase");
	SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,CB_ADDSTRING,0,(LPARAM)"API");
	SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,CB_ADDSTRING,0,(LPARAM)"Protocol");
	SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,CB_ADDSTRING,0,(LPARAM)"Parameters");

	switch(logLevel)
	{
	case logLevelExceptions:
		SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,WM_SETTEXT,0,(LPARAM)"Exceptions");
		break;
	case logLevelBasicFlow:
		SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,WM_SETTEXT,0,(LPARAM)"BasicFlow");
		break;
	case logLevelStates:
		SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,WM_SETTEXT,0,(LPARAM)"States");
		break;
	case logLevelDataBase:
		SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,WM_SETTEXT,0,(LPARAM)"DataBase");
		break;
	case logLevelAPI:
		SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,WM_SETTEXT,0,(LPARAM)"API");
		break;
	case logLevelProtocol:
		SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,WM_SETTEXT,0,(LPARAM)"Protocol");
		break;
	case logLevelParameters:
		SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,WM_SETTEXT,0,(LPARAM)"Parameters");
		break;
	}
}

void initLogLevelCBs(HWND hWnd)
{
	if(LOG_CHANNEL==(logClasses&LOG_CHANNEL))
		CheckDlgButton(hWnd,IDC_CHAN,1);

	if(LOG_CALL==(logClasses&LOG_CALL))
		CheckDlgButton(hWnd,IDC_CALL,1);

	if(LOG_CONFERENCE==(logClasses&LOG_CONFERENCE))
		CheckDlgButton(hWnd,IDC_CONF,1);

	if(LOG_MC==(logClasses&LOG_MC))
		CheckDlgButton(hWnd,IDC_MC,1);

	if(LOG_STACK==(logClasses&LOG_STACK))
		CheckDlgButton(hWnd,IDC_STACK,1);

	if(LOG_API==(logClasses&LOG_API))
		CheckDlgButton(hWnd,IDC_API,1);

	if(LOG_EXCEPTION==(logClasses&LOG_EXCEPTION))
		CheckDlgButton(hWnd,IDC_EXCEPTION,1);

	if(LOG_H245MESSAGE==(logClasses&LOG_H245MESSAGE))
		CheckDlgButton(hWnd,IDC_H245MESSAGE,1);

	if(LOG_APPLICATION==(logClasses&LOG_APPLICATION))
		CheckDlgButton(hWnd,IDC_APPLICATION,1);

	if(LOG_WARNING==(logClasses&LOG_WARNING))
		CheckDlgButton(hWnd,IDC_WARNING,1);


}


 
void getLogClass(HWND hWnd,UINT32 * logClass)
{
	UINT32 retVal=0;
	if(IsDlgButtonChecked(hWnd,IDC_CHAN))
		retVal=retVal|LOG_CHANNEL;
	if(IsDlgButtonChecked(hWnd,IDC_CALL))
		retVal=retVal|LOG_CALL;
	if(IsDlgButtonChecked(hWnd,IDC_CONF))
		retVal=retVal|LOG_CONFERENCE;
	if(IsDlgButtonChecked(hWnd,IDC_MC))
		retVal=retVal|LOG_MC;
	if(IsDlgButtonChecked(hWnd,IDC_STACK))
		retVal=retVal|LOG_STACK;
/*	if(IsDlgButtonChecked(hWnd,IDC_DB))
		retVal=retVal|LOG_DB;
*/	if(IsDlgButtonChecked(hWnd,IDC_API))
		retVal=retVal|LOG_API;
	if(IsDlgButtonChecked(hWnd,IDC_EXCEPTION))
		retVal=retVal|LOG_EXCEPTION;
	if(IsDlgButtonChecked(hWnd,IDC_PROTOCOL))
		retVal=retVal|LOG_H245MESSAGE;
	if(IsDlgButtonChecked(hWnd,IDC_USER))
		retVal=retVal|LOG_APPLICATION;
	if(IsDlgButtonChecked(hWnd,IDC_WARNING))
		retVal=retVal|LOG_WARNING;
	*logClass=retVal;

}

void readlogOptionsDialogValues(HWND hWnd)
{
	char logLevelName[20];
	SendDlgItemMessage(hWnd,IDC_DEBUG_LEVEL,WM_GETTEXT,sizeof(logLevelName),(LPARAM)logLevelName);
/*	logLevel=atoi(buff);*/
	mciLogLevelByName(logLevelName,&logLevel);
	getLogClass(hWnd,&logClasses);
	mciReportSetDebugOptions(logClasses,logLevel);
}

BOOL CALLBACK logOptionsWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{	
	switch (message) 
	{
	case WM_INITDIALOG:
		mciReportGetDebugOptions(&logClasses,&logLevel);
		initDebugLevelCB(hWnd);
		initLogLevelCBs(hWnd);
		break;

	case WM_CLOSE:
		EndDialog(hWnd,0);
		break;
	case WM_COMMAND:
		switch(GET_WM_COMMAND_ID(wParam,lParam))
		{
		case IDC_LOGOPTIONS_OK:
			readlogOptionsDialogValues(hWnd);
			EndDialog(hWnd,0);
			return TRUE;
		}

	default: return FALSE;
	} 
	return TRUE;
}

