/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN32
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#endif

#include <cm.h>

#include "resource.h"

#include "rvcommon.h"
#include "mci_types.h"
#include "mci_report.h"
#include "mci_general.h"

#include "gkadd.h"
#include "gkrgi.h"
#include "gkcci.h"

#include "mci_chan.h"
#include "mci_call.h"
#include "mci_conf.h"
#include "mci_mc.h"
#include "mci_call_caps.h"

#include "gkmc.h"
#include "mcgki.h"
#include "mcconfig.h"
#include "mcmain.h"

#include "mccall_details.h"
#include "mcrefresh.h"

//3.11
static mciConfHandleType CURRENT_CONF_HANDLE=0;
static mciCallHandleType CURRENT_CALL_HANDLE=0;
static mciChanHandleType CURRENT_CHAN_HANDLE=0;

static HWND CONF_DETAIL_HANDLE;
static HWND CALL_DETAIL_HANDLE;
static HWND CHAN_CONTROL_HANDLE;
//

static int CURRENT_DIALOG;
static int CURRENT_HANDLE; //to a conf,a call or a channel
static HWND GLOBALHWND;


RVAPI 
MciRV CALLCONV mciCallCapName(IN  mciCapType cap,
								OUT const char** capName) 
{
	
	switch(cap) 
	{
   case  mciCapTypeNonStandard:
		*capName = "NonStandard";
		return MCI_RV_OK;
   case  mciCapTypeReceiveVideo:
		*capName = "ReceiveVideo";
		return MCI_RV_OK;
	case  mciCapTypeTransmitVideo:
		*capName = "TransmitVideo";
		return MCI_RV_OK;
	case  mciCapTypeReceiveAndTransmitVideo:
		*capName = "ReceiveAndTransmitVideo";
		return MCI_RV_OK;
	case  mciCapTypeReceiveAudio:
		*capName = "ReceiveAudio";
		return MCI_RV_OK;
 	case  mciCapTypeTransmitAudio:
		*capName = "TransmitAudio";
		return MCI_RV_OK;
    case  mciCapTypeReceiveAndTransmitAudio:
		*capName = "ReceiveAndTransmitAudio";
		return MCI_RV_OK;
    case  mciCapTypeReceiveData:
		*capName = "ReceiveData";
		return MCI_RV_OK;
	case  mciCapTypeTransmitData:
		*capName = "TransmitData";
		return MCI_RV_OK;
	case  mciCapTypeReceiveAndTransmitData:
		*capName = "ReceiveAndTransmitData";
		return MCI_RV_OK;
	case  mciCapTypeH233EncryptionTransmit:
		*capName = "H233EncryptionTransmit";
		return MCI_RV_OK;
	case  mciCapTypeConference:
		*capName = "Conference";
		return MCI_RV_OK;
	case  mciCapTypeH235Security:
		*capName = "H235Security";
		return MCI_RV_OK;
    case  mciCapTypeMaxPendingReplacementFor:
		*capName = "MaxPendingReplacementFor";
		return MCI_RV_OK;
	case  mciCapTypeReceiveUserInput:
		*capName = "ReceiveUserInput";
		return MCI_RV_OK;
	case  mciCapTypeTransmitUserInput:
		*capName = "TransmitUserInput";
		return MCI_RV_OK;
	case  mciCapTypeReceiveAndTransmitUserInput:
		*capName = "ReceiveAndTransmitUserInput";
		return MCI_RV_OK;
	case  mciCapTypeUndefined:
		*capName = "Undefined";
		return MCI_RV_OK;
	default:
		mciReport(LOG_EXCEPTION, logLevelExceptions,"Unknown capability");
		return MCI_RV_UNKNOWN_STATE;
	}
}





/******************************************************/
/*getting parameter from string*/
RVAPI 
MciRV CALLCONV mciChanMediaByName(IN const char * mediaName,
								OUT mciMediaType * media) 
{
	if(!strcmp(mediaName,"nonStandard")){
		*media=mciMediaNonStandard;
		return MCI_RV_OK;
	}
	else if(!strcmp(mediaName,"audioData")){
		*media=mciMediaAudioData;
		return MCI_RV_OK;
	}
	else if(!strcmp(mediaName,"videoData")){
		*media=mciMediaVideoData;
		return MCI_RV_OK;
	}
	else if(!strcmp(mediaName,"data")){
		*media=mciMediaData;
		return MCI_RV_OK;
	}
	else if(!strcmp(mediaName,"nullData")){
		*media=mciMediaNullData;
		return MCI_RV_OK;
	}
	else if(!strcmp(mediaName,"encryptionData")){
		*media=mciMediaEncryptionData;
		return MCI_RV_OK;
	}
	else if(!strcmp(mediaName,"h235Control")){
		*media=mciMediaH235Control;
		return MCI_RV_OK;
	}
	else if(!strcmp(mediaName,"h235Media")){
		*media=mciMediaH235Media;
		return MCI_RV_OK;
	}
	else if(!strcmp(mediaName,"undefined")){
		*media=mciMediaUndefined;
		return MCI_RV_OK;
	}
	mciReport(LOG_EXCEPTION, logLevelExceptions,"Unknown media");
	return MCI_RV_UNKNOWN_STATE;

}

RVAPI 
MciRV CALLCONV mciChanProtocolByName(IN const char * protocolName,
								OUT mciMediaProtocolType * protocol) 
{
	//non standard
	if(!strcmp(protocolName,"object")){
		*protocol=object;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"h221NonStandard")){
		*protocol=h221NonStandard;
		return MCI_RV_OK;
	}
    //audio
	if(!strcmp(protocolName,"audioNonStandard")){
		*protocol=audioNonStandard;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g711Alaw64k")){
		*protocol=g711Alaw64k;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g711Alaw56k")){
		*protocol=g711Alaw56k;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g711Ulaw64k")){
		*protocol=g711Ulaw64k;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g711Ulaw56k")){
		*protocol=g711Ulaw56k;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g722_64k")){
		*protocol=g722_64k;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g722_56k")){
		*protocol=g722_56k;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g722_48k")){
		*protocol=g722_48k;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g7231")){
		*protocol=g7231;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g728")){
		*protocol=g728;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g729")){
		*protocol=g729;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g729AnnexA")){
		*protocol=g729AnnexA;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g729wAnnexB")){
		*protocol=g729wAnnexB;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g729AnnexAwAnnexB")){
		*protocol=g729AnnexAwAnnexB;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g729AnnexAwAnnexB")){
		*protocol=g729AnnexAwAnnexB;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"g7231AnnexCCapability")){
		*protocol=g7231AnnexCCapability;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"gsmFullRate")){
		*protocol=gsmFullRate;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"gsmHalfRate")){
		*protocol=gsmHalfRate;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"gsmEnhancedFullRate")){
		*protocol=gsmEnhancedFullRate;
		return MCI_RV_OK;
	}
   //video
	else if(!strcmp(protocolName,"videoNonStatndard")){
		*protocol=videoNonStatndard;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"H261")){
		*protocol=H261;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"H262")){
		*protocol=H262;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"H263")){
		*protocol=H263;
		return MCI_RV_OK;
	}
//data
	else if(!strcmp(protocolName,"dataNonStandard")){
		*protocol=dataNonStandard;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"t120")){
		*protocol=t120;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"dsm_cc")){
		*protocol=dsm_cc;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"userData")){
		*protocol=userData;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"t84")){
		*protocol=t84;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"t434")){
		*protocol=t434;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"h224")){
		*protocol=h224;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"dlpid")){
		*protocol=dlpid;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"h222DataPartitioning")){
		*protocol=h222DataPartitioning;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"t30fax")){
		*protocol=t30fax;
		return MCI_RV_OK;
	}
	else if(!strcmp(protocolName,"t140")){
		*protocol=t140;
		return MCI_RV_OK;
	}
	mciReport(LOG_EXCEPTION, logLevelExceptions,"Unknown protocol");
	return MCI_RV_UNKNOWN_STATE;

}


RVAPI 
MciRV CALLCONV mciLogLevelByName(IN const char * logLevelName,
								OUT logLevel_t * logLevel) 
{
	
	if(!strcmp(logLevelName,"Exceptions")){
		*logLevel=logLevelExceptions;
		return MCI_RV_OK;
	}
	else if(!strcmp(logLevelName,"BasicFlow")){
		*logLevel=logLevelBasicFlow;
		return MCI_RV_OK;
	}
	else if(!strcmp(logLevelName,"States")){
		*logLevel=logLevelStates;
		return MCI_RV_OK;
	}
	else if(!strcmp(logLevelName,"DataBase")){
		*logLevel=logLevelDataBase;
		return MCI_RV_OK;
	}
	else if(!strcmp(logLevelName,"API")){
		*logLevel=logLevelAPI;
		return MCI_RV_OK;
	}
	else if(!strcmp(logLevelName,"Protocol")){
		*logLevel=logLevelProtocol;
		return MCI_RV_OK;
	}
	else if(!strcmp(logLevelName,"Parameters")){
		*logLevel=logLevelParameters;
		return MCI_RV_OK;
	}
	return MCI_RV_UNKNOWN_PRM;

}

void getErrorName(MciRV retVal,char ** errorName)
{
	switch (retVal)
	{
	case MCI_RV_NO_MEM:
		*errorName="no mem";
	case MCI_RV_CANNOT_OPEN_FILE:
		*errorName="cannot open file";
	case MCI_RV_CANNOT_CLOSE_FILE:
		*errorName="cannot close file";
	case MCI_RV_CANNOT_ALLOC:
		*errorName="cannot alloc";
	case MCI_RV_WRONG_HANDLE:
		*errorName="wrong handle";
	case MCI_RV_VACANT_HANDLE:
		*errorName="vacant handle";
	case MCI_RV_NULL_HANDLE:
		*errorName="null handle";
	case MCI_RV_ARRAY_TOO_SHORT:
		*errorName="array too short";
	case MCI_RV_NULL_PTR:
		*errorName="null prt";
	case MCI_RV_NO_PLACE_IN_DB:
		*errorName="no place id db";
	case MCI_RV_UNKNOWN_PRM:
		*errorName="unknown prm";
	case MCI_RV_ILLEGAL_PRM:
		*errorName="illegal prm";
	case MCI_RV_ILLEGAL_VALUE:
		*errorName="illegal value";
	case MCI_RV_NUM_TOO_SMALL:
		*errorName="num too small";
	case MCI_RV_NUM_TOO_LARGE:
		*errorName="num too large";
	case MCI_RV_UNKNOWN_STATE:
		*errorName="unknown state";
	case MCI_RV_ILLEGAL_STATE:
		*errorName="illegal state";
	case MCI_RV_ILLEGAL_MULTIPOINT:
		*errorName="illegal multipoint";
	case MCI_RV_INCOMPATIBLE_TYPES:
		*errorName="incompatible types";
	case MCI_RV_STACK_ERR:
		*errorName="stack err";
	case MCI_RV_STACK_INIT_ERR:
		*errorName="stack init err";
	case MCI_RV_ILLEGAL_TRANSITION:
		*errorName="illegal transition";
	case MCI_RV_UNRECOGNIZED_GK_NOTIFICATION:
		*errorName="unrecognized gk notification";
	case MCI_RV_NO_ADDRESS:
		*errorName="no address";
	case MCI_RV_ERROR:            
		*errorName="error";
	default:
		*errorName="unknown error";
	}
}




#ifdef __cplusplus
}
#endif

