/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/


/**********************************************************
		   PUBLIC  DECLARATIONS
*=========================================================
* TITLE: mccall details.c
*---------------------------------------------------------
* PROJECT: MC CONFIGURATION FOR PC
*---------------------------------------------------------
* $Workfile$
*---------------------------------------------------------
* $Author$ YONATAN BECKER
*---------------------------------------------------------
* $Date$ 23/9/99
*---------------------------------------------------------
* $Revision$
**********************************************************/

/********************************************************/
/*                     INTERFACE DECLARATIONS           */
/********************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN32
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#endif

#include <cm.h>

#include "resource.h"

#include "rvcommon.h"
#include "mci_types.h"
#include "mci_report.h"
#include "mci_general.h"

#include "gkadd.h"
#include "gkrgi.h"
#include "gkcci.h"

#include "mci_chan.h"
#include "mci_call.h"
#include "mci_conf.h"
#include "mci_mc.h"
#include "mci_call_caps.h"

#include "gkmc.h"
#include "mcgki.h"
#include "mcconfig.h"
#include "mcmain.h"

#include "mccall_details.h"
#include "mcrefresh.h"
#include "mcchannels_control.h"
#include "mccapabilities.h"
#include "mcdescriptors.h"

/*-----------------*/
/* LOCAL CONSTANTS */
/*-----------------*/
#define APPCALL_NULL				    0
#define NO_COLUMN					   (-1)
#define CHANNEL_HANDLE_COLUMN			(0)
#define APPLICATION_HANDLE_COLUMN		(1)
#define STATE_COLUMN					(2)
#define FORWARD_MEDIA_COLUMN	        (3)
#define REVERSE_MEDIA_COLUMN	        (4)

static void channels2Window(HWND hWnd);
BOOL CALLBACK channelsWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK channelDetailsWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);


/*-----------------*/
/* LOCAL VARIABLES */
/*-----------------*/
static mciCallHandleType hCall;
static HWND channelsHWnd = NULL;
static void initChannelsColumns(HWND hWnd)
{
      LV_COLUMN lvc;

      //set colunms

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText ="Channel Handle";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CHANNELS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CHANNELS_LIST, LVM_INSERTCOLUMN,CHANNEL_HANDLE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText ="Application Handle";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CHANNELS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CHANNELS_LIST, LVM_INSERTCOLUMN,APPLICATION_HANDLE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText ="State";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CHANNELS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CHANNELS_LIST, LVM_INSERTCOLUMN,STATE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Forward Media";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CHANNELS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"999.999.999.999:9999") + 10;
      SendDlgItemMessage(hWnd, IDC_CHANNELS_LIST, LVM_INSERTCOLUMN,FORWARD_MEDIA_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Reverse Media";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CHANNELS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"999.999.999.999:9999") + 10;
      SendDlgItemMessage(hWnd, IDC_CHANNELS_LIST, LVM_INSERTCOLUMN,REVERSE_MEDIA_COLUMN, (LPARAM)&lvc);

}

void initCallDetails(HWND hWnd,mciCallHandleType hCall)
{
    int channelsNum,EpTerminal;
	char *name,str[100],*p;
    mciCallStateType CallState=0;
	mciMsdStateType MsdState=0;
	mciCallMsdStatusType MsdStatus=0;
	mciCEStateType CEState=0;

	sprintf(str,"%x",hCall);
	SendDlgItemMessage(hWnd,IDC_CALL_HANDLE,WM_SETTEXT,0,(LPARAM)str);
	SendDlgItemMessage(hWnd,IDC_CALL_HANDLE,EM_SETREADONLY,TRUE,0);

	mciCallGetPrm(hCall,mciCallPrmEpTerminalType,(void *)&EpTerminal);
	sprintf(str,"%d",EpTerminal);
	SendDlgItemMessage(hWnd,IDC_EP_TERMINAL,WM_SETTEXT,0,(LPARAM)str);

  	mciCallGetState(hCall,&CallState);
	if(mciCallStateName(CallState,&name)==MCI_RV_OK)
		SendDlgItemMessage(hWnd,IDC_STATE,WM_SETTEXT,0,(LPARAM)name);
	else
		SendDlgItemMessage(hWnd,IDC_STATE,WM_SETTEXT,0,(LPARAM)"unavailable");
	SendDlgItemMessage(hWnd,IDC_STATE,EM_SETREADONLY,TRUE,0);

	if(mciCallGetChanNum(hCall,&channelsNum)==MCI_RV_OK)
	{
		sprintf(str,"%d Channels",channelsNum);
		SendDlgItemMessage(hWnd,IDC_CHANNELS_NUM,WM_SETTEXT,0,(LPARAM)str);
	}
	else
		SendDlgItemMessage(hWnd,IDC_CHANNELS_NUM,WM_SETTEXT,0,(LPARAM)"0 Channels");
	SendDlgItemMessage(hWnd,IDC_CHANNELS_NUM,EM_SETREADONLY,TRUE,0);

	mciCallGetPrm(hCall,mciCallPrmMsdState,(void *)&MsdState);
	if(mciCallMsdStateName(MsdState,&name)==MCI_RV_OK)
		SendDlgItemMessage(hWnd,IDC_MSD_STATE,WM_SETTEXT,0,(LPARAM)name);
	else
		SendDlgItemMessage(hWnd,IDC_MSD_STATE,WM_SETTEXT,0,(LPARAM)"unavailable");
	SendDlgItemMessage(hWnd,IDC_MSD_STATE,EM_SETREADONLY,TRUE,0);

	mciCallGetPrm(hCall,mciCallPrmMsdStatus,(void *)&MsdStatus);
	if(mciCallMsdStatusName(MsdStatus,&p)==MCI_RV_OK)
		SendDlgItemMessage(hWnd,IDC_MSD_STATUS,WM_SETTEXT,0,(LPARAM)p);
	else
		SendDlgItemMessage(hWnd,IDC_MSD_STATUS,WM_SETTEXT,0,(LPARAM)"unavailable");
	SendDlgItemMessage(hWnd,IDC_MSD_STATUS,EM_SETREADONLY,TRUE,0);

	mciCallGetPrm(hCall,mciCallPrmRemoteCEState,(void *)&CEState);
	if(mciCallCEStateName(CEState,&name)==MCI_RV_OK)
		SendDlgItemMessage(hWnd,IDC_REMOTE_CE_STATE,WM_SETTEXT,0,(LPARAM)name);
	else
		SendDlgItemMessage(hWnd,IDC_REMOTE_CE_STATE,WM_SETTEXT,0,(LPARAM)"unavailable");
	SendDlgItemMessage(hWnd,IDC_REMOTE_CE_STATE,EM_SETREADONLY,TRUE,0);

	mciCallGetPrm(hCall,mciCallPrmLocalCEState,(void *)&CEState);
	if(mciCallCEStateName(CEState,&name)==MCI_RV_OK)
		SendDlgItemMessage(hWnd,IDC_LOCAL_CE_STATE,WM_SETTEXT,0,(LPARAM)name);
	else
		SendDlgItemMessage(hWnd,IDC_LOCAL_CE_STATE,WM_SETTEXT,0,(LPARAM)"unavailable");
	SendDlgItemMessage(hWnd,IDC_LOCAL_CE_STATE,EM_SETREADONLY,TRUE,0);


}

static void mcChannels2Window(HWND hWnd)
{
	  DWORD  D;
	  BOOL isDuplex;
      HWND listID;
      LV_ITEM lvi;
      int itemNum,i;
	  char str[100];
	  char *p;
	  mciChanHandleType hChan=0;
	  mciChanStateType state=0;
	  char valBuff[100];
	  UINT32 chanNum=0; 
	  mciCallGetChanNum(hCall,&chanNum);
	  mciCallGetFirstChan(hCall,&hChan);
      listID = GetDlgItem(hWnd,IDC_CHANNELS_LIST);
      itemNum = ListView_GetItemCount(listID);
	  //while(hChan)
	  while(chanNum--)
	  {

		/* CHANNEL_HANDLE_COLUMN */
		lvi.mask=LVIF_TEXT|LVIF_PARAM;
		lvi.iItem=itemNum;
		sprintf(str,"%x",hChan);
		lvi.pszText=str;
		lvi.iSubItem=CHANNEL_HANDLE_COLUMN;
        lvi.lParam=(LPARAM)hChan;
		i=ListView_InsertItem(listID,&lvi);

		/* APPLICATION_HANDLE_COLUMN */
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;
		mciChanGetPrm(hChan,mciChanPrmAppHandle,mciChanGeneral,(void *)valBuff);
		if (valBuff)
			sprintf(str,"%d",*(int *)&valBuff);
		else 
			sprintf(str,"unavailable");
		lvi.pszText=str;
		lvi.iSubItem=APPLICATION_HANDLE_COLUMN;
		i=ListView_SetItem(listID,&lvi);
		D=GetLastError();
 
		/* STATE_COLUMN */
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;
		mciChanGetState(hChan,&state);
		mciChanStateName(state,&p);
		lvi.pszText=p;
		lvi.iSubItem=STATE_COLUMN;
		i=ListView_SetItem(listID,&lvi);


		/* MEDIA_TYPE_COLUMN */
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;

		isDuplex=mciChanGetPrm(hChan,mciChanPrmIsDuplex,mciChanGeneral,(void *)valBuff);

		mciChanGetPrm(hChan,mciChanPrmMediaType,mciChanForward,(void *)valBuff);
		mciChanMediaName((mciMediaType)*valBuff,&p);
		if(p) 
			strcpy(str,p);
		else 
			sprintf(str,"unavailable");
		lvi.pszText=str;
		lvi.iSubItem=FORWARD_MEDIA_COLUMN;
		i=ListView_SetItem(listID,&lvi);

		itemNum++;
		mciCallGetNextChan(hChan,&hChan);

	  }
}


BOOL apputilHitTestItem(/*performs the hit_Test. If successful return value is true, and 
                     lvItem->iItem will contatin the item that was hit.otherwise FALSE is returned*/
                     IN HWND hList,
                     OUT LV_ITEM *lvItem)
{
  DWORD mouse_pt;
  LV_HITTESTINFO hti;

  mouse_pt = GetMessagePos();

  hti.pt.x = LOWORD(mouse_pt);
  hti.pt.y = HIWORD(mouse_pt);
  ScreenToClient(hList, &hti.pt);
  ListView_HitTest(hList, &hti);
  if (hti.iItem >= 0)
  {
    lvItem->iItem = hti.iItem;
    return TRUE;
  }
  else
    return FALSE;
}


static LRESULT notifyChannelsHandler(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	NM_LISTVIEW *pNm=(NM_LISTVIEW *)lParam;
    HWND hList;
	LV_ITEM lvItem;
    BOOL b;
	int retVal;

	if(wParam!=IDC_CHANNELS_LIST)
		return 0L;

	hList=GetDlgItem(hWnd,wParam);

	switch (pNm->hdr.code)
	{
	case NM_DBLCLK :
		if (apputilHitTestItem(hList,&lvItem))
		{
			lvItem.mask=LVIF_PARAM;
			lvItem.iSubItem=0;
			b=ListView_GetItem(hList, &lvItem);
			if (wParam==IDC_CHANNELS_LIST){
				retVal=DialogBoxParam(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_CHANNEL_CONTROL),hWnd,channelControlWndProc,lvItem.lParam);
			}
		}
		break;
	}
  return TRUE;
}

void readCallDetailsDialogParams(HWND hWnd)
{
	char str[100];
	mciConfHandleType hCallParent;
	long success;
	int EpTerminal=0;
	success=SendDlgItemMessage(hWnd,IDC_EP_TERMINAL,WM_GETTEXT,sizeof(str),(LPARAM)str);
	if(success)
	{
		EpTerminal=atoi(str);
		mciCallSetPrm(hCall,mciCallPrmEpTerminalType,(void *)&EpTerminal);
	}
	mciCallGetConf(hCall,&hCallParent);
	RefreshConfDialog(hCallParent,VALID_HANDLE);


}


void checkManualModes(HWND hWnd)
{
	mciMsdStateType MsdState=0;
	mciCEStateType CEState=0;
	Button_Enable(GetDlgItem(hWnd,IDC_REMOTE_CE_APPROVE),FALSE);
	Button_Enable(GetDlgItem(hWnd,IDC_REMOTE_CE_RELAY),FALSE);
	Button_Enable(GetDlgItem(hWnd,IDC_LOCAL_CE_RELAY),FALSE);
	Button_Enable(GetDlgItem(hWnd,IDC_LOCAL_CE_REJECT),FALSE);
    Button_Enable(GetDlgItem(hWnd,IDC_MSD_APPROVE),FALSE);


/*manual mode for CE*/
	if(!(IsDlgButtonChecked(hWnd,IDC_CHECK_CE)))
	{
		/*remote*/
		mciCallGetPrm(hCall,mciCallPrmRemoteCEState,(void *)&CEState);
		if(CEState==mciCESttCapSetRecvd)
		{/*Relay,Approve*/
			Button_Enable(GetDlgItem(hWnd,IDC_REMOTE_CE_APPROVE),TRUE);
			Button_Enable(GetDlgItem(hWnd,IDC_REMOTE_CE_RELAY),TRUE);
			//return;
		}
		/*local*/
		mciCallGetPrm(hCall,mciCallPrmLocalCEState,(void *)&CEState);
		{
		if(CEState==mciCESttCapSetReady)
			{/*Approve,Reject*/
		    Button_Enable(GetDlgItem(hWnd,IDC_LOCAL_CE_RELAY),TRUE);
		    Button_Enable(GetDlgItem(hWnd,IDC_LOCAL_CE_REJECT),TRUE);
			//return;
			}
		}
	}
	
		
/*manual mode for MSD*/
	if(!(IsDlgButtonChecked(hWnd,IDC_CHECK_MSD)))
	{
		
		mciCallHandleType hCoupledCall;
		mciMsdStateType MsdCoupledState=0;
		
		mciCallGetPrm(hCall,mciCallPrmCoupledHandle,(void *)&hCoupledCall);			
		if(hCoupledCall == NULL) {
           Button_Enable(GetDlgItem(hWnd,IDC_MSD_APPROVE),FALSE);
		   return;
        }
 		
		mciCallGetPrm(hCall,mciCallPrmMsdState,(void *)&MsdState);
		mciCallGetPrm(hCoupledCall,mciCallPrmMsdState,(void *)&MsdCoupledState);
		

		if(((MsdState==mciMsdSttMsdRecvd)||(MsdState == mciMsdSttMsdRelayed)) && 
		   (MsdCoupledState != mciMsdSttAckSent) && (MsdCoupledState != mciMsdSttAckRecvd)
		   ||(MsdState==mciMsdSttAckRecvd))
		    Button_Enable(GetDlgItem(hWnd,IDC_MSD_APPROVE),TRUE);
        
	    else 
	        Button_Enable(GetDlgItem(hWnd,IDC_MSD_APPROVE),FALSE);
	}
}


void refreshCallDetails(HWND hWnd)
{
	HWND listID;
	setCALL_DETAIL_HANDLE(hWnd);
	setCURRENT_CALL_HANDLE(hCall);

	initCallDetails(hWnd,hCall);

	listID=GetDlgItem(hWnd,IDC_INCOMING_CAPABILITIES);
	ListView_DeleteAllItems(listID);		
	Capapbilities2Window(hWnd,hCall,IDC_INCOMING_CAPABILITIES,mciDirTypeIncoming);

	listID=GetDlgItem(hWnd,IDC_OUTGOING_CAPABILITIES);
	ListView_DeleteAllItems(listID);		
	Capapbilities2Window(hWnd,hCall,IDC_OUTGOING_CAPABILITIES,mciDirTypeOutgoing);

	listID=GetDlgItem(hWnd,IDC_INCOMING_DESCRIPTORS);
	ListView_DeleteAllItems(listID);		
	Descriptors2Window(hWnd,hCall,IDC_INCOMING_DESCRIPTORS,mciDirTypeIncoming);

	listID=GetDlgItem(hWnd,IDC_OUTGOING_DESCRIPTORS);
	ListView_DeleteAllItems(listID);		
	Descriptors2Window(hWnd,hCall,IDC_OUTGOING_DESCRIPTORS,mciDirTypeOutgoing);

    listID=GetDlgItem(hWnd,IDC_CHANNELS_LIST);
	ListView_DeleteAllItems(listID);		
	mcChannels2Window(hWnd);
	checkManualModes(hWnd);
}
/**************************************************************************************************/
/*deleting objects*/

LPARAM getSelectedObject(/*returns value of the lParam field of the selected item in the endpoints list.
                              If none selected, -1 is returned*/
                              IN HWND hList)
{
  int item;
  LV_ITEM lvItem;
  item = ListView_GetNextItem(hList,-1,LVNI_FOCUSED/*LVNI_SELECTED*/);
  if (item == -1)
  return item;
  lvItem.iItem = item;
  lvItem.mask = LVIF_PARAM;
  lvItem.iSubItem =0;
  ListView_GetItem(hList, &lvItem);
  return lvItem.lParam;
}

int	EndCallDetailsDialog(HWND hWnd,int nResult)
{
	setCURRENT_CALL_HANDLE(0);
	setCALL_DETAIL_HANDLE(0);
	return EndDialog(hWnd,nResult);
}


BOOL CALLBACK callDetailsWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HWND hList;
	LPARAM object=0;
	mciCallHandleType coupledCallHandle;
	mciConfHandleType coupledCallConf; //used to test if the coupled call is valid

	switch (message) 
	{
	case WM_INITDIALOG:
	{
		hCall=(mciCallHandleType)lParam;
		initCapabilities(hWnd,hCall,IDC_INCOMING_CAPABILITIES);
		initCapabilities(hWnd,hCall,IDC_OUTGOING_CAPABILITIES);
		initDescriptors(hWnd,IDC_INCOMING_DESCRIPTORS);
		initDescriptors(hWnd,IDC_OUTGOING_DESCRIPTORS);
		initChannelsColumns(hWnd);
		refreshCallDetails(hWnd);
	}

	
	case WM_REFRESH_DIALOG:
		if(lParam)
			hCall=(mciCallHandleType)lParam;
		refreshCallDetails(hWnd);
		return TRUE;
    case WM_NOTIFY :
      return notifyChannelsHandler(hWnd,message,wParam,lParam);
    break;

	case WM_CLOSE:
		EndCallDetailsDialog(hWnd,0);
		return TRUE;

	case WM_COMMAND:
	{
		switch(GET_WM_COMMAND_ID(wParam,lParam))
		{

		case IDC_CALL_DEATILS_OK:
			readCallDetailsDialogParams(hWnd);
			EndCallDetailsDialog(hWnd,0);
			return TRUE;
/*remote CE*/
		case IDC_REMOTE_CE_APPROVE:
			readCallDetailsDialogParams(hWnd);
			return mciCallPTPCEReply(hCall,mciDirTypeIncoming,mciCEReplyApprove,mciCEReplyReasonUnspecified);

		case IDC_REMOTE_CE_RELAY:
			readCallDetailsDialogParams(hWnd);
			return mciCallPTPCEReply(hCall,mciDirTypeIncoming,mciCEReplyPTPRelay,mciCEReplyReasonUnspecified);

/*local CE*/
		case IDC_LOCAL_CE_REJECT:
			readCallDetailsDialogParams(hWnd);
		     return mciCallPTPCEReply(hCall,mciDirTypeOutgoing,mciCEReplyReject,mciCEReplyReasonUnspecified);
			
		case IDC_LOCAL_CE_RELAY:
			readCallDetailsDialogParams(hWnd);
			
			return mciCallPTPCEReply(hCall,mciDirTypeOutgoing,mciCEReplyPTPRelay,mciCEReplyReasonUnspecified);

/*MSD*/
		case IDC_MSD_APPROVE:
			//readCallDetailsDialogParams(hWnd);			
			{ int rv;
			rv =  mciCallPTPMsdReply(hCall,mciMsdReplyPTPRelay,mciMsdReplyReasonUnspecified);
            readCallDetailsDialogParams(hWnd);		
            refreshCallDetails(hWnd);
			return rv;
			}
		/*deleting a capability*/
		case IDC_DELETE_INCOMING_CAPABILITIES:
			hList=GetDlgItem(hWnd,IDC_INCOMING_CAPABILITIES);
			object=getSelectedObject(hList);
			if(object>=0)
			{
				mciCapRemoveCapability(hCall,mciDirTypeIncoming,object);
				refreshCallDetails(hWnd);
			}
			break;
		case IDC_DELETE_OUTGOING_CAPABILITIES:
			hList=GetDlgItem(hWnd,IDC_OUTGOING_CAPABILITIES);
			object=getSelectedObject(hList);
			if(object>=0)
			{
				mciCapRemoveCapability(hCall,mciDirTypeOutgoing,object);
				refreshCallDetails(hWnd);
			}
			break;
/*deleting a descriptor*/
		case IDC_DELETE_INCOMING_DESCRIPTORS:
			hList=GetDlgItem(hWnd,IDC_INCOMING_DESCRIPTORS);
			object=getSelectedObject(hList);
			if(object>=0)
			{
				mciCapRemoveDescriptor(hCall,mciDirTypeIncoming,object);
				refreshCallDetails(hWnd);
			}
			break;
		case IDC_DELETE_OUTGOING_DESCRIPTORS:
			hList=GetDlgItem(hWnd,IDC_OUTGOING_DESCRIPTORS);
			object=getSelectedObject(hList);
			if(object>=0)
			{
				mciCapRemoveDescriptor(hCall,mciDirTypeOutgoing,object);
				refreshCallDetails(hWnd);
			}
			break;
		case IDC_COUPLED_CALL:
			mciCallGetPrm(hCall,mciCallPrmCoupledHandle,(void *)&coupledCallHandle);			
			if(mciCallGetConf(hCall,&coupledCallConf)!=MCI_RV_OK)
			{
				MessageBox(0,"The Coupled Call doesn't exist","Alarm",MB_OK|MB_ICONEXCLAMATION);
				return FALSE;
			}
			else
			{
				SendMessage(hWnd,WM_REFRESH_DIALOG,0,(LPARAM)coupledCallHandle);
/*				EndCallDetailsDialog(hWnd,0); 
				DialogBoxParam(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_CALL_DETAILS),hWnd,callDetailsWndProc,(LPARAM)coupledCallHandle);		
*/			}			

		}
	}

  default: return FALSE;
  }
  return FALSE;
}

#ifdef __cplusplus
}
#endif
