/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/


/**********************************************************
		   PUBLIC  DECLARATIONS
*=========================================================
* TITLE: mccapabilites.c
*---------------------------------------------------------
* PROJECT: MC CONFIGURATION FOR PC
*---------------------------------------------------------
* $Workfile$
*---------------------------------------------------------
* $Author$ YONATAN BECKER
*---------------------------------------------------------
* $Date$ 30/10/99
*---------------------------------------------------------
* $Revision$
**********************************************************/

/********************************************************/
/*                     INTERFACE DECLARATIONS           */
/********************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN32
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#endif

#include <cm.h>

#include "resource.h"

#include "rvcommon.h"
#include "mci_types.h"
#include "mci_report.h"
#include "mci_general.h"

#include "gkadd.h"
#include "gkrgi.h"
#include "gkcci.h"

#include "mci_chan.h"
#include "mci_call.h"
#include "mci_conf.h"
#include "mci_mc.h"
#include "mci_call_caps.h"

#include "gkmc.h"
#include "mcgki.h"
#include "mcconfig.h"
#include "mcmain.h"

#include "mccall_details.h"
#include "mcrefresh.h"
#include "mcchannels_control.h"

/*-----------------*/
/* LOCAL CONSTANTS */
/*-----------------*/
#define ENTRY_NUM_COLUMN			            (0)
#define CAPABILITY_TYPE_COLUMN		            (1)
#define MEDIA_NAME_COLUMN					    (2)
#define MEDIA_PROTOCOL_COLUMN	                (3)
#define CIF_COLUMN	                            (4)
#define QCIF_COLUMN	                            (5)
#define MAX_BIT_RATE_COLUMN					    (6)
#define CAPABILITY_HANDLE_COLUMN			    (7)

/*-----------------*/
/* LOCAL VARIABLES */
/*-----------------*/


void initCapabilities(HWND hWnd,mciCallHandleType hCall,int listControlID)
{
      LV_COLUMN lvc;

      //set colunms

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText ="Entry Number";
      lvc.cx = SendDlgItemMessage(hWnd, listControlID, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, listControlID, LVM_INSERTCOLUMN,ENTRY_NUM_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText ="Capability Type";
      lvc.cx = SendDlgItemMessage(hWnd, listControlID, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, listControlID, LVM_INSERTCOLUMN,CAPABILITY_TYPE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText ="Media Name";
      lvc.cx = SendDlgItemMessage(hWnd, listControlID, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, listControlID, LVM_INSERTCOLUMN,MEDIA_NAME_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Media Protocol";
      lvc.cx = SendDlgItemMessage(hWnd, listControlID, LVM_GETSTRINGWIDTH, 0, (LPARAM)"999.999.999.999:9999") + 10;
      SendDlgItemMessage(hWnd, listControlID, LVM_INSERTCOLUMN,MEDIA_PROTOCOL_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "CIF";
      lvc.cx = SendDlgItemMessage(hWnd, listControlID, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxx") + 10;
      SendDlgItemMessage(hWnd, listControlID, LVM_INSERTCOLUMN,CIF_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "QCIF";
      lvc.cx = SendDlgItemMessage(hWnd, listControlID, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxx") + 10;
      SendDlgItemMessage(hWnd, listControlID, LVM_INSERTCOLUMN,QCIF_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Max Bit Rate";
      lvc.cx = SendDlgItemMessage(hWnd, listControlID, LVM_GETSTRINGWIDTH, 0, (LPARAM)"999.999.999.999:9999") + 10;
      SendDlgItemMessage(hWnd, listControlID, LVM_INSERTCOLUMN,MAX_BIT_RATE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Capability Handle";
      lvc.cx = SendDlgItemMessage(hWnd, listControlID, LVM_GETSTRINGWIDTH, 0, (LPARAM)"999.999.999.999:9999") + 10;
      SendDlgItemMessage(hWnd, listControlID, LVM_INSERTCOLUMN,CAPABILITY_HANDLE_COLUMN, (LPARAM)&lvc);
}

void Capapbilities2Window(HWND hWnd,mciCallHandleType hCall,int listControlID,mciDirectionType direction)
{
    HWND listID;
    LV_ITEM lvi;
    int itemNum,i,numOfCaps=0,capIdx;
	char str[100];
	char *p;
	mciCapabilityType capability;	
    listID=GetDlgItem(hWnd,listControlID);
    itemNum=ListView_GetItemCount(listID);
	capability.capabilityHandle=0;
	capability.capType=0;
	capability.cif=0;
	capability.entryNum=0;
	capability.maxBitRate=0;
	capability.mediaProtocol=0;
	capability.mediaType=0;
	capability.qcif=0;



	if(mciCapGetNumOfCapabilities(hCall,direction,&numOfCaps)== MCI_RV_OK) {
		
		for(capIdx=1;capIdx<=numOfCaps;capIdx++)
		{
			mciCapGetCapability(hCall,direction,capIdx,&capability) ;
			
			/* ENTRY_NUM_COLUMN */
			lvi.mask=LVIF_TEXT|LVIF_PARAM;
			lvi.iItem=itemNum;
			sprintf(str,"%d",capability.entryNum);
			lvi.pszText=str;
			lvi.iSubItem=ENTRY_NUM_COLUMN;
			lvi.lParam=(LPARAM)capIdx;
			i=ListView_InsertItem(listID,&lvi);
			
			/* CAPABILITY_TYPE_COLUMN */
			lvi.mask=LVIF_TEXT;
			lvi.iItem=itemNum;
			mciCallCapName(capability.capType,&p);
			lvi.pszText=p;
			lvi.iSubItem=CAPABILITY_TYPE_COLUMN;
			i=ListView_SetItem(listID,&lvi);
			
			/* MEDIA_NAME_COLUMN */
			lvi.mask=LVIF_TEXT;
			lvi.iItem=itemNum;
			if(capability.mediaType != mciMediaUndefined) 
			{
				mciChanMediaName(capability.mediaType,&p);
				lvi.pszText=p;
				lvi.iSubItem=MEDIA_NAME_COLUMN;
				i=ListView_SetItem(listID,&lvi);
			}
			
			/* MEDIA_PROTOCOL_COLUMN */
			lvi.mask=LVIF_TEXT;
			lvi.iItem=itemNum;
			if(capability.mediaProtocol != undefinedProtocol)
			{
				mciChanProtocolName(capability.mediaProtocol,&p);
				lvi.pszText=p;
				lvi.iSubItem=MEDIA_PROTOCOL_COLUMN;
				i=ListView_SetItem(listID,&lvi);
			}
			/* CIF_COLUMN */
			lvi.mask=LVIF_TEXT;
			lvi.iItem=itemNum;
			if(capability.cif!=-1)
			{
				sprintf(str,"%d",capability.cif);
				lvi.pszText=str;
				lvi.iSubItem=CIF_COLUMN;
				i=ListView_SetItem(listID,&lvi);
			}
			
			/* QCIF_COLUMN */
			lvi.mask=LVIF_TEXT;
			lvi.iItem=itemNum;
			if(capability.qcif!=-1)
			{
				sprintf(str,"%d",capability.qcif);
				lvi.pszText=str;
				lvi.iSubItem=QCIF_COLUMN;
				i=ListView_SetItem(listID,&lvi);
			}
			
			/*MAX_BIT_RATE_COLUMN*/
			lvi.mask=LVIF_TEXT;
			lvi.iItem=itemNum;
			if(capability.maxBitRate != -1) 
			{
				sprintf(str,"%d",capability.maxBitRate);
				lvi.pszText=str;
				lvi.iSubItem=MAX_BIT_RATE_COLUMN;
				i=ListView_SetItem(listID,&lvi);
			}
			/* CAPABILITY_HANDLE_COLUMN */
			lvi.mask=LVIF_TEXT;
			lvi.iItem=itemNum;
			sprintf(str,"%d",capability.capabilityHandle);
			lvi.pszText=str;
			lvi.iSubItem=CAPABILITY_HANDLE_COLUMN;
			i=ListView_SetItem(listID,&lvi);
			
			itemNum++;
		}
	}

}


#ifdef __cplusplus
}
#endif
