/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/


/**********************************************************
		   PUBLIC  DECLARATIONS
*=========================================================
* TITLE: mcconference details.c
*---------------------------------------------------------
* PROJECT: MC CONFIGURATION FOR PC
*---------------------------------------------------------
* $Workfile$
*---------------------------------------------------------
* $Author$ YONATAN BECKER
*---------------------------------------------------------
* $Date$ 22/9/99
*---------------------------------------------------------
* $Revision$
**********************************************************/

/********************************************************/
/*                     INTERFACE DECLARATIONS           */
/********************************************************/

#ifdef WIN32
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#endif

#include <cm.h>

#include "resource.h"

#include "rvcommon.h"
#include "mci_types.h"
#include "mci_report.h"
#include "mci_general.h"

#include "gkadd.h"
#include "gkrgi.h"
#include "gkcci.h"

#include "mci_chan.h"
#include "mci_call.h"
#include "mci_conf.h"
#include "mci_mc.h"
#include "mci_call_caps.h"

#include "gkmc.h"
#include "mcgki.h"
#include "mcconfig.h"
#include "mcmain.h"

#include "mccall_details.h"
#include "mcpending.h"
#include "mcrefresh.h"

/*-----------------*/
/* LOCAL CONSTANTS */
/*-----------------*/
#define APPCALL_NULL				    0
#define NO_COLUMN					   (-1)
#define CALL_HANDLE_COLUMN				(0)
#define CHANNELS_NUM_COLUMN				(1)
#define EP_TERMINAL_TYPE_COLUMN		    (2)
#define STATE_COLUMN					(3) 
#define MSD_STATE_COLUMN				(4)
#define MSD_STATUS_COLUMN				(5)
#define INCOMING_CE_STATE_COLUMN		(6) 
#define OUTGOING_CE_STATE_COLUMN		(7) 
#define PENDING_CHANNELS            	(8) 

mciConfHandleType hConf;

static void mcCalls2Window(HWND hWnd);
static void call2Window(HWND hWnd);


/*-----------------*/
/* LOCAL VARIABLES */
/*-----------------*/
static HWND callsHWnd = NULL;


static void initCallsColumns(HWND hWnd)
{
      LV_COLUMN lvc;

      //set colunms

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText ="Call Handle";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_INSERTCOLUMN,CALL_HANDLE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText ="Channels Num";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_INSERTCOLUMN,CHANNELS_NUM_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "EP Terminal";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_INSERTCOLUMN, EP_TERMINAL_TYPE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "State";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_INSERTCOLUMN, STATE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "MSD State";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_INSERTCOLUMN, MSD_STATE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "MSD Status At Proxy";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_INSERTCOLUMN, MSD_STATUS_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Incoming CE state";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_INSERTCOLUMN, INCOMING_CE_STATE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Outgoing CE State";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_INSERTCOLUMN, OUTGOING_CE_STATE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Pending Channels";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 10;
      SendDlgItemMessage(hWnd, IDC_CALLS_LIST, LVM_INSERTCOLUMN, PENDING_CHANNELS, (LPARAM)&lvc);

}

void initConfDetails(HWND hWnd,mciConfHandleType hConf)
{
    int callsNum=0,terminal=0;
	char *name,str[100];
    mciConfStateType confState=0;

	sprintf(str,"%x",hConf);
	SendDlgItemMessage(hWnd,IDC_CONFERENCE_HANDLE,WM_SETTEXT,0,(LPARAM)str);
	SendDlgItemMessage(hWnd,IDC_CONFERENCE_HANDLE,EM_SETREADONLY,TRUE,0);

  	mciConfGetState(hConf,&confState);
	mciConfStateName(confState,&name);
	SendDlgItemMessage(hWnd,IDC_CONF_STATE,WM_SETTEXT,0,(LPARAM)name);
	SendDlgItemMessage(hWnd,IDC_CONF_STATE,EM_SETREADONLY,TRUE,0);

	mciConfGetCallNum(hConf,&callsNum);
	sprintf(str,"%d",callsNum);
	SendDlgItemMessage(hWnd,IDC_CALLS_NUM,WM_SETTEXT,0,(LPARAM)str);
	SendDlgItemMessage(hWnd,IDC_CALLS_NUM,EM_SETREADONLY,TRUE,0);

	mciConfGetPrm(hConf,mciConfPrmTerminalType,(void *)&terminal);
	sprintf(str,"%d",terminal);
	SendDlgItemMessage(hWnd,IDC_TERMINAL_TYPE,WM_SETTEXT,0,(LPARAM)str);

}

static void Calls2Window(HWND hWnd)
{
  HWND hList;
  hList= GetDlgItem(hWnd,IDC_CALLS_LIST);
//  ListView_DeleteAllItems(hList);
  mcCalls2Window(hWnd);
}


static void mcCalls2Window(HWND hWnd)

{
      HWND listID;
      LV_ITEM lvi;
      int itemNum,channelsNum=0,EpTerminal=0,i;
	  char *name,str[100],*p;
	  mciCallStateType callState=0;
	  mciCallHandleType hCall=0;
	  mciMsdStateType MsdState=0;
      mciCallMsdStatusType MsdStatus=0;
	  mciCEStateType CEState=0; 
	  UINT32 callNum=0;
  	  mciConfGetCallNum(hConf,&callNum);

	  mciConfGetFirstCall(hConf,&hCall);
      listID = GetDlgItem(hWnd,IDC_CALLS_LIST);
//	  while(hCall)
	  while(callNum--)
	  {
		itemNum = ListView_GetItemCount(listID);

		/* CALL_HANDLE_COLUMN */
		lvi.mask=LVIF_TEXT| LVIF_PARAM;
		lvi.iItem=itemNum;
		sprintf(str,"%x",hCall);
		lvi.pszText=str;
		lvi.iSubItem=CALL_HANDLE_COLUMN;
        lvi.lParam=(LPARAM)hCall;
		i=ListView_InsertItem(listID,&lvi);

		/* CHANNELS_NUM_COLUMN */
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;
		mciCallGetChanNum(hCall,&channelsNum);
		sprintf(str,"%d",channelsNum);
		lvi.pszText=str;
		lvi.iSubItem=CHANNELS_NUM_COLUMN;
		i=ListView_SetItem(listID,&lvi);

		/* EP_TERMINAL_TYPE_COLUMN */
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;
		mciCallGetPrm(hCall,mciCallPrmEpTerminalType,(void *)&EpTerminal); 
		sprintf(str,"%d",EpTerminal);
		lvi.pszText=str;
		lvi.iSubItem=EP_TERMINAL_TYPE_COLUMN;
		ListView_SetItem(listID,&lvi);
      
		/*STATE_COLUMN*/
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;
		mciCallGetState(hCall,&callState);
		mciCallStateName(callState,&name);
		lvi.pszText=name;
		lvi.iSubItem=STATE_COLUMN;
		i=ListView_SetItem(listID,&lvi);


		/*MSD_STATE_COLUMN*/
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;
		mciCallGetPrm(hCall,mciCallPrmMsdState,(void *)&MsdState);
		mciCallMsdStateName(MsdState,&name);
		lvi.pszText=name;
		lvi.iSubItem=MSD_STATE_COLUMN;
		i=ListView_SetItem(listID,&lvi);

		/*MSD_STATUS_COLUMN*/
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;
		mciCallGetPrm(hCall,mciCallPrmMsdStatus,(void *)&MsdStatus);
		mciCallMsdStatusName(MsdStatus,&p);
		lvi.pszText=p;
		lvi.iSubItem=MSD_STATUS_COLUMN;
		i=ListView_SetItem(listID,&lvi);

		/*REMOTE_CE_STATE_COLUMN*/
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;
		mciCallGetPrm(hCall,mciCallPrmRemoteCEState,(void *)&CEState);
		mciCallCEStateName(CEState,&name);
		lvi.pszText=name;
		lvi.iSubItem=INCOMING_CE_STATE_COLUMN;
		i=ListView_SetItem(listID,&lvi);

		/*LOCAL_CE_STATE_COLUMN*/
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;
		mciCallGetPrm(hCall,mciCallPrmLocalCEState,(void *)&CEState);
		mciCallCEStateName(CEState,&name);
		lvi.pszText=name;
		lvi.iSubItem=OUTGOING_CE_STATE_COLUMN;
		i=ListView_SetItem(listID,&lvi);

		/*PENDING_CHANNELS_COLUMN*/
		if(pendingChannelsInCall(hCall))
		{
			lvi.mask=LVIF_TEXT;
			lvi.iItem=itemNum;
			lvi.pszText="!";
			lvi.iSubItem=PENDING_CHANNELS;
			i=ListView_SetItem(listID,&lvi);
		}
		itemNum++;
		mciConfGetNextCall(hCall,&hCall);

	  }
}



static LRESULT notifyCallsHandler(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	NM_LISTVIEW *pNm=(NM_LISTVIEW *)lParam;
	HWND hList;
	BOOL b;

	if (wParam!=IDC_CALLS_LIST)
		return 0L;

	hList = GetDlgItem(hWnd,IDC_CALLS_LIST);

	switch (pNm->hdr.code)
	{
    case NM_DBLCLK :
		{
		LV_ITEM lvItem;
		if (apputilHitTestItem(hList,&lvItem))
			{
			lvItem.mask = LVIF_PARAM;
			lvItem.iSubItem = 0;
			b=ListView_GetItem(hList, &lvItem);
			DialogBoxParam(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_CALL_DETAILS),hWnd,callDetailsWndProc,/*(LPARAM)*/lvItem.lParam);
			//refreshConferenceDetails(hWnd);
			}
		}
    break;
	}
	return TRUE;
}



LPARAM getSelectedCall(/*returns value of the lParam field of the selected item in the endpoints list.
                              If none selected, -1 is returned*/
                              IN HWND hList)
{
  int item;
  LV_ITEM lvItem;
  item = ListView_GetNextItem(hList,-1,LVNI_SELECTED);
  if (item == -1)
  return item;
  lvItem.iItem = item;
  lvItem.mask = LVIF_PARAM;
  lvItem.iSubItem =CALL_HANDLE_COLUMN;
  ListView_GetItem(hList, &lvItem);
  return lvItem.lParam;
}

void readConferenceDetailsDialogParams(HWND hWnd)
{
	char str[100];
	long success;
	int terminal;
	success=SendDlgItemMessage(hWnd,IDC_TERMINAL_TYPE,WM_GETTEXT,sizeof(str),(LPARAM)str);
	if(success)
	{
		terminal=atoi(str);
		mciConfSetPrm(hConf,mciConfPrmTerminalType,(void *)&terminal);
	}

}

void refreshConferenceDetails(HWND hWnd)
{
    HWND listID=GetDlgItem(hWnd,IDC_CALLS_LIST);
	setCONF_DETAIL_HANDLE(hWnd);
	setCURRENT_CONF_HANDLE(hConf);

	ListView_DeleteAllItems(listID);		
	initConfDetails(hWnd,hConf);
	Calls2Window(hWnd);
}

int	EndConfDetailsDialog(HWND hWnd,int nResult)
{
	setCURRENT_CONF_HANDLE(0);
	setCONF_DETAIL_HANDLE(0);
	return EndDialog(hWnd,nResult);
}


BOOL CALLBACK callsWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int retVal;
	switch (message) 
	{
	case WM_INITDIALOG:
	{
		hConf=(mciConfHandleType)lParam;
		initCallsColumns(hWnd);
		refreshConferenceDetails(hWnd);
	}
		break;

	case WM_REFRESH_DIALOG:
		if(lParam)
			hConf=(mciConfHandleType)lParam;
		refreshConferenceDetails(hWnd);
		return TRUE;

    case WM_NOTIFY :
      return notifyCallsHandler(hWnd,message,wParam,lParam);
    break;

	case WM_CLOSE:
		EndConfDetailsDialog(hWnd,0);
		break;

	case WM_COMMAND:
	{
		switch(GET_WM_COMMAND_ID(wParam,lParam))
		{
		case IDC_GET_CALL_DETAILS:
			{
			HWND hList=GetDlgItem(hWnd,IDC_CALLS_LIST);
			LPARAM hCall=getSelectedCall(hList);
			mciCallHandleType callBuff=(mciCallHandleType)hCall;
			if(hCall<0)
				{
				MessageBox(0,"Please select a call","Alarm",MB_OK|MB_ICONEXCLAMATION);
				return FALSE;
				}
			retVal=DialogBoxParam(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_CALL_DETAILS),hWnd,callDetailsWndProc,hCall);
			//refreshConferenceDetails(hWnd);
			
			break;
			}
		case IDC_CONFERENCE_DETAILS_OK:
			{
			readConferenceDetailsDialogParams(hWnd);
			EndConfDetailsDialog(hWnd,0);
			break;
			}

		}
	}

  default: return FALSE;
  }
  return TRUE;
}