/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/


/**********************************************************
		   PUBLIC  DECLARATIONS
*=========================================================
* TITLE: conferences.c
*---------------------------------------------------------
* PROJECT: MC CONFIGURATION FOR PC
*---------------------------------------------------------
* $Workfile$
*---------------------------------------------------------
* $Author$ YONATAN BECKER
*---------------------------------------------------------
* $Date$ 7/9/99
*---------------------------------------------------------
* $Revision$
**********************************************************/

/********************************************************/
/*                     INTERFACE DECLARATIONS           */
/********************************************************/

#ifdef WIN32
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#endif

#include <cm.h>

#include "resource.h"

#include "rvcommon.h"
#include "mci_types.h"
#include "mci_report.h"
#include "mci_general.h"

#include "gkadd.h"
#include "gkrgi.h"
#include "gkcci.h"

#include "mci_chan.h"
#include "mci_call.h"
#include "mci_conf.h"
#include "mci_mc.h"
#include "mci_call_caps.h"

#include "gkmc.h"
#include "mcgki.h"
#include "mcconfig.h"
#include "mcmain.h"
#include "mccall_details.h"
#include "mcconferences.h"
#include "mcpending.h"
#include "mcrefresh.h"

/*-----------------*/
/* LOCAL CONSTANTS */
/*-----------------*/
#define APPCALL_NULL				0
#define NO_COLUMN					(-1)
#define CONF_HANDLE_COLUMN	        (0)
#define CONF_STATE_COLUMN			(1)
#define CALLS_NUM_COLUMN			(2)
#define PENDING_CHANNELS_COLUMN     (3) 
#define PENDING_CE_COLUMN			(4) 
#define PENDING_MSD_COLUMN			(5) 


static mciMcHandleType hMc;

static void call2Window(HWND hWnd);
BOOL CALLBACK callsWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);


/*-----------------*/
/* LOCAL VARIABLES */
/*-----------------*/
static HWND conferencesHWnd = NULL;

static void initConferencesColumns(HWND hWnd)
{
      LV_COLUMN lvc;

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Conference Handle";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"999.999.999.999:9999")/* + 10*/;
      SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_INSERTCOLUMN, CONF_HANDLE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Conf State";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"CALLID")/* + 10*/;
      SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_INSERTCOLUMN, CONF_STATE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Calls Num";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"CALLID")/* + 10*/;
      SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_INSERTCOLUMN, CALLS_NUM_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "Chans";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"CALLID") + 10;
      SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_INSERTCOLUMN, PENDING_CHANNELS_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "CE";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"CALLID") + 10;
      SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_INSERTCOLUMN, PENDING_CE_COLUMN, (LPARAM)&lvc);

      lvc.mask = LVCF_TEXT | LVCF_WIDTH;
      lvc.pszText = "MSD";
      lvc.cx = SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_GETSTRINGWIDTH, 0, (LPARAM)"CALLID") + 10;
      SendDlgItemMessage(hWnd, IDC_CONFERENCES_LIST, LVM_INSERTCOLUMN, PENDING_MSD_COLUMN, (LPARAM)&lvc);

}


static void conferences2Window(HWND hWnd)

{
      HWND listID;
      LV_ITEM lvi;
      int itemNum,callsNum=0,i;
	  char *name,str[100];
	  mciConfStateType confState=0;
	  mciConfHandleType hConf=0;
	  mciMcGetFirstConf(hMc,&hConf);
      listID = GetDlgItem(hWnd,IDC_CONFERENCES_LIST);
      itemNum = ListView_GetItemCount(listID);
	  while(hConf)
	  {

		/* CONF_HANDLE_COLUMN */
		lvi.mask=LVIF_TEXT| LVIF_PARAM;
		lvi.iItem=itemNum;
		sprintf(str,"%x",hConf);
		lvi.pszText=str;
		lvi.iSubItem=CONF_HANDLE_COLUMN;
        lvi.lParam = (LPARAM)hConf;
 		i=ListView_InsertItem(listID,&lvi);

		/* CONF_STATE_COLUMN */
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;
		mciConfGetState(hConf,&confState);
		mciConfStateName(confState,&name);
		lvi.pszText=name;
		lvi.iSubItem=CONF_STATE_COLUMN;
		i=ListView_SetItem(listID,&lvi);


		/*CALLS_NUM_COLUMN */
		lvi.mask=LVIF_TEXT;
		lvi.iItem=itemNum;
		mciConfGetCallNum(hConf,&callsNum);
		sprintf(str,"%d",callsNum);
		lvi.pszText=str;
		lvi.iSubItem=CALLS_NUM_COLUMN;
		i=ListView_SetItem(listID,&lvi);

		/*PENDING_CHANNELS_COLUMN*/
		if(pendingChannelsInConf(hConf))
		{
			lvi.mask=LVIF_TEXT;
			lvi.iItem=itemNum;
			lvi.pszText="!";
			lvi.iSubItem=PENDING_CHANNELS_COLUMN;
			i=ListView_SetItem(listID,&lvi);
		}

		/*PENDING_CE_COLUMN*/
		if(pendingCE(hConf))
		{
			lvi.mask=LVIF_TEXT;
			lvi.iItem=itemNum;
			lvi.pszText="!";
			lvi.iSubItem=PENDING_CE_COLUMN;
			i=ListView_SetItem(listID,&lvi);
		}

		/*PENDING_MSD_COLUMN*/
		if(pendingMSD(hConf))
		{
			lvi.mask=LVIF_TEXT;
			lvi.iItem=itemNum;
			lvi.pszText="!";
			lvi.iSubItem=PENDING_MSD_COLUMN;
			i=ListView_SetItem(listID,&lvi);
		}

		itemNum++;
		mciMcGetNextConf(hConf,&hConf);

	  }
}

static LRESULT notifyConferencesHandler(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  NM_LISTVIEW *pNm=(NM_LISTVIEW *)lParam;
  HWND hList;
  BOOL b;

  if (wParam!=IDC_CONFERENCES_LIST)
    return 0L;

  hList = GetDlgItem(hWnd,IDC_CONFERENCES_LIST);

  switch (pNm->hdr.code)
  {
    case NM_DBLCLK :
    {
      LV_ITEM lvItem;
      if (apputilHitTestItem(hList,&lvItem))
      {
        lvItem.mask = LVIF_PARAM;
        lvItem.iSubItem = 0;
        b=ListView_GetItem(hList, &lvItem);
		DialogBoxParam(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_CONFERENCE_DETAILS),hWnd,callsWndProc,lvItem.lParam);
		refreshConference(hWnd);
      }
    }
    break;
  }
  return TRUE;
}




/*returns value of the pszText field of the selected item in the conferences list.
                              If none selected, -1 is returned*/
LPARAM getSelectedConference(IN HWND hList)
{
  int item;
  LV_ITEM lvItem;
  item = ListView_GetNextItem(hList,-1,LVNI_SELECTED);
  if(item==-1)
	return item;
  lvItem.iItem=item;
  lvItem.mask=LVIF_PARAM ;
  lvItem.iSubItem=CONF_HANDLE_COLUMN;
  ListView_GetItem(hList,&lvItem);
  //ListView_GetItemText(hList,item,HCONF_COLUMN,buff,strlen(buff));
  return lvItem.lParam;
}

void refreshConference(HWND hWnd)
{
	HWND hList=GetDlgItem(hWnd,IDC_CONFERENCES_LIST);
	setCONFERENCES_HANDLE(hWnd);
	ListView_DeleteAllItems(hList);
	conferences2Window(hWnd);
}

int	EndConferencesDetailsDialog(HWND hWnd,int nResult)
{
	setCONFERENCES_HANDLE(0);
	return EndDialog(hWnd,nResult);
}


BOOL CALLBACK conferenceWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{	
	int i=WM_REFRESH_DIALOG;
	switch (message) 
	{
	case WM_INITDIALOG:
		hMc=(mciMcHandleType)lParam;

		initConferencesColumns(hWnd);
		refreshConference(hWnd);
		break;
	case WM_REFRESH_DIALOG:
		refreshConference(hWnd);
		return TRUE;

    case WM_NOTIFY :
      return notifyConferencesHandler(hWnd,message,wParam,lParam);
    break;

	case WM_CLOSE:
		EndConferencesDetailsDialog(hWnd,0);
		break;

	case WM_COMMAND:
	{
		switch(GET_WM_COMMAND_ID(wParam,lParam))
		{
		case IDC_GET_CONFERENCE_DETAILS:
			{	
			HWND hList=GetDlgItem(hWnd,IDC_CONFERENCES_LIST);
			LPARAM hConference=getSelectedConference(hList);
			mciConfHandleType confBuff=(mciConfHandleType)hConference;
			if(hConference<0)
				{
				MessageBox(0,"Please select a conference","Alarm",MB_OK|MB_ICONEXCLAMATION);
				return FALSE;
				}
			DialogBoxParam(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_CONFERENCE_DETAILS),hWnd,callsWndProc,hConference);
			break;
			}
		}
		
	}

  default: return FALSE;
  }
  return TRUE;
}