/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/


/**********************************************************
		   PUBLIC  DECLARATIONS
*=========================================================
* TITLE: config.c
*---------------------------------------------------------
* PROJECT: MC CONFIGURATION FOR PC
*---------------------------------------------------------
* $Workfile$
*---------------------------------------------------------
* $Author$ YONATAN BECKER
*---------------------------------------------------------
* $Date$ 26/9/99
*---------------------------------------------------------
* $Revision$
**********************************************************/

/********************************************************/
/*                     INTERFACE DECLARATIONS           */
/********************************************************/

#ifdef WIN32
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#endif


#include <cm.h>

#include "resource.h"

#include "rvcommon.h"
#include "mci_types.h"
#include "mci_report.h"
#include "mci_general.h"

#include "gkadd.h"
#include "gkrgi.h"
#include "gkcci.h"

#include "mci_chan.h"
#include "mci_call.h"
#include "mci_conf.h"
#include "mci_mc.h"
#include "mci_call_caps.h"

#include "gkmc.h"
#include "mcgki.h"
#include "mcconfig.h"
#include "mcmain.h"

#include "configutl.h"

/*-----------------*/
/*   LOCAL TYPES   */
/*-----------------*/


static configInput *configData;
//yonatan 28.12
static BOOL READ_ONLY=FALSE;
HWND	cfghWnd;
BOOL 	CONFIG_FILE_MISSING=FALSE;


 
/*------------------------------------------------------------------------------------------*/

static void mcWinReport (IN UINT32 logClass,
						 IN char* msgStr, IN const char* fileName, IN int line)
{
	static char reportStr[1024];
	char*		temp;

	if (temp = strrchr (fileName, '\\'))
		fileName = temp + 1;

	sprintf(reportStr,"%s  ('%s', line %d)", msgStr, fileName, line);
	mcOutputMsg (logClass,reportStr);
}

/*------------------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------*/
/* Configuration File Read Support                                   */
/*-------------------------------------------------------------------*/
/* This function open the init file, if such exist and read the configuration     */
/* data into global structures which be later on read to the configuration window */
/* if the file does not exist, this function returns FALSE and the config data is */
/* being read later on from the hard coded default values in the GK. If a parsing */ 
/* error is encountered the default values are read to the global structures      */
BOOL loadConfigData(void)
{
	BOOL success = TRUE;
	FILE * fd;
	if((fd = fopen(ALLOCATIONS_CONFIG_FILE,"r")) == NULL)
	{
		/*sprintf(msg,"!!Exception: Failure opening the file %s",ALLOCATIONS_CONFIG_FILE);
		mcWinReport (LOG_EXCEPTION, msg,__FILE__,__LINE__);
	*/	CONFIG_FILE_MISSING=TRUE;
		return FALSE;	
	}
	loadGeneralData(fd);
	fclose(fd);
	return success;
}

static BOOL loadGeneralData(FILE* fd)
{
	if(utilSearchForKeyword(fd,"[GENERAL]"))
	{
		readGeneralValues(fd);
		return TRUE;
	}
	return FALSE;
}

void readGeneralValues(IN FILE* fd)
{
	char buf[1000];
	char *line;
	char lhs[1000],rhs[1000];

	while(utilReadNextLine(fd,buf,1000)) {
		lhs[0] = rhs[0] = 0;
		line = utilParseNextEquation(buf,lhs,rhs);

		if(strcmp(lhs,"MAX_CONFS")==0)
			configData->maxConfs=atoi(rhs);

		if(strcmp(lhs,"MAX_CALLS")==0) 
			configData->maxCalls=atoi(rhs);

		if(strcmp(lhs,"MAX_CHANS")==0) 
			configData->maxChans=atoi(rhs);

		if(strcmp(lhs,"PORT")==0)
			configData->port=atoi(rhs);
	}
}

void configData2Dialog(HWND hWnd)
{
	char buff[100];
	sprintf(buff,"%d",configData->maxConfs);
	SetDlgItemText(hWnd,IDC_CONFS,buff);
	sprintf(buff,"%d",configData->maxCalls);
	SetDlgItemText(hWnd,IDC_CALLS,buff);
	sprintf(buff,"%d",configData->maxChans);
	SetDlgItemText(hWnd,IDC_CHANS,buff);
	sprintf(buff,"%d",configData->port);
	SetDlgItemText(hWnd,IDC_PORT,buff);
}


void readDialogValues(HWND hWnd)
{
	char buff[100];
    int recNum = 0;

	SendDlgItemMessage(hWnd,IDC_CONFS,WM_GETTEXT,sizeof(buff),(LPARAM)buff);
	recNum = atoi(buff);
	if(recNum < 0) 
       configData->maxConfs = 0;
	else 
	   configData->maxConfs = recNum;

	SendDlgItemMessage(hWnd,IDC_CALLS,WM_GETTEXT,sizeof(buff),(LPARAM)buff);
	recNum = atoi(buff);
	if(recNum < 0) 
		configData->maxCalls=0;
	else
	    configData->maxCalls = recNum;

	
	SendDlgItemMessage(hWnd,IDC_CHANS,WM_GETTEXT,sizeof(buff),(LPARAM)buff);
	recNum = atoi(buff);
	if(recNum < 0) 
		configData->maxChans=0;
	else
	    configData->maxChans = recNum;

		
	SendDlgItemMessage(hWnd,IDC_PORT,WM_GETTEXT,sizeof(buff),(LPARAM)buff);
	
	if(atoi(buff) < 0)
       configData->port = 0;
	else 
	   configData->port=atoi(buff);
		


}

BOOL saveConfigurationData()
{
	char buf[100];
	FILE * fd;
	int success;
	if(CONFIG_FILE_MISSING){
		if((fd=fopen(ALLOCATIONS_CONFIG_FILE,"w+"))==NULL){
			mcWinReport (LOG_EXCEPTION,"!!Exception: Failure saving the configuration data",__FILE__,__LINE__);
			return FALSE;
		}
		else;
	}
	else{
		if((fd=fopen(ALLOCATIONS_CONFIG_FILE,"r+"))==NULL){
			mcWinReport (LOG_EXCEPTION,"!!Exception: Failure saving the configuration data",__FILE__,__LINE__);
			return FALSE;
		}
	}
	success=fprintf(fd,"[GENERAL]\n");
	sprintf(buf,"MAX_CONFS=%d\n",configData->maxConfs);
 
	success=fprintf(fd,"%s",buf);
	sprintf(buf,"MAX_CALLS=%d\n",configData->maxCalls);
	success=fprintf(fd,"%s",buf);
	sprintf(buf,"MAX_CHANS=%d\n",configData->maxChans);
	success=fprintf(fd,"%s",buf);
	sprintf(buf,"PORT=%d\n",configData->port);
	success=fprintf(fd,"%s",buf);
	fclose(fd);
	return TRUE;
}

void setDefaultValues()
{
	configData->maxConfs = DEFAULT_MAX_CONF;
	configData->maxCalls = DEFAULT_MAX_CALLS;
	configData->maxChans = DEFAULT_MAX_CHANS;
	configData->port = DEFAULT_PORT;
}

BOOL CALLBACK configWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
	case WM_INITDIALOG:
		{
		configData = (configInput *)lParam;
		cfghWnd = hWnd; 
		if(READ_ONLY==FALSE) //the dialog is displayed for the first time
		{
			BOOL initFileFound;
			READ_ONLY=TRUE;
			initFileFound = loadConfigData();
			if(!initFileFound)
				setDefaultValues();
			
		}
		else //disable buttons and edit boxes
			{
			Button_Enable(GetDlgItem(hWnd,IDC_DEFAULT),FALSE);
			Button_Enable(GetDlgItem(hWnd,IDC_OK),FALSE);
			Button_Enable(GetDlgItem(hWnd,IDC_SAVE),FALSE);
			SendDlgItemMessage(hWnd,IDC_CONFS,EM_SETREADONLY,TRUE,0);
			SendDlgItemMessage(hWnd,IDC_CALLS,EM_SETREADONLY,TRUE,0);
			SendDlgItemMessage(hWnd,IDC_CHANS,EM_SETREADONLY,TRUE,0);
			SendDlgItemMessage(hWnd,IDC_PORT,EM_SETREADONLY,TRUE,0);
			}
		configData2Dialog(cfghWnd);
		}
		break;
	case WM_CLOSE:/*ends MC*/
		EndDialog(hWnd,END_MC);
		break;

	case WM_COMMAND:
	{
		switch(GET_WM_COMMAND_ID(wParam,lParam))
		{
		case IDC_DEFAULT:
			setDefaultValues(); 
			EndDialog(hWnd,0);
			break;
		case IDC_OK:
			readDialogValues(cfghWnd);
			EndDialog(hWnd,0);
			break;
		case IDC_SAVE:
			readDialogValues(cfghWnd);
			saveConfigurationData();
			EndDialog(hWnd,0);
			break;
		}
	}

  default: return FALSE;
  }
  return TRUE;
}
