/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/


/**********************************************************
		   PUBLIC  DECLARATIONS
*=========================================================
* TITLE: mcdescriptors.c
*---------------------------------------------------------
* PROJECT: MC CONFIGURATION FOR PC
*---------------------------------------------------------
* $Workfile$
*---------------------------------------------------------
* $Author$ YONATAN BECKER
*---------------------------------------------------------
* $Date$ 1/11/99
*---------------------------------------------------------
* $Revision$
**********************************************************/

/********************************************************/
/*                     INTERFACE DECLARATIONS           */
/********************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN32
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#endif

#include <cm.h>

#include "resource.h"

#include "rvcommon.h"
#include "mci_types.h"
#include "mci_report.h"
#include "mci_general.h"

#include "gkadd.h"
#include "gkrgi.h"
#include "gkcci.h"

#include "mci_chan.h"
#include "mci_call.h"
#include "mci_conf.h"
#include "mci_mc.h"
#include "mci_call_caps.h"

#include "gkmc.h"
#include "mcgki.h"
#include "mcconfig.h"
#include "mcmain.h"

#include "mccall_details.h"
#include "mcrefresh.h"
#include "mcchannels_control.h"

/*-----------------*/
/* LOCAL CONSTANTS */
/*-----------------*/
#define NUM_OF_ALTERNATIVE_SETS 10
#define SIZE_OF_ALTERNATIVE_SET 10

/*-----------------*/
/* LOCAL VARIABLES */
/*-----------------*/
	int descriptor[NUM_OF_ALTERNATIVE_SETS][SIZE_OF_ALTERNATIVE_SET];


/*gets a row in the bi-directional array
returns a buffer that displayes the descriptor*/	
void createSingleDescriptor(IN int rowIndex,OUT char * descriptorsBuff,OUT int * descriptorsBuffIndex)
{
	char str[100];
	int i=0,descriptorIndex=0;

 
	descriptorsBuff[*descriptorsBuffIndex]='{';
	*descriptorsBuffIndex=(*descriptorsBuffIndex)++;

	for(descriptorIndex=0;(descriptor[rowIndex][descriptorIndex]!=-1)&&(descriptorIndex<SIZE_OF_ALTERNATIVE_SET);descriptorIndex++)
	{
		if(descriptorIndex!=0)
		{
			descriptorsBuff[*descriptorsBuffIndex]=',';
			*descriptorsBuffIndex=(*descriptorsBuffIndex)++;
		}

		sprintf(str,"%d",descriptor[rowIndex][descriptorIndex]);
		strcat(descriptorsBuff,str);
		*descriptorsBuffIndex=(*descriptorsBuffIndex)+strlen(str);
		
	}

	descriptorsBuff[*descriptorsBuffIndex]='}';
	*descriptorsBuffIndex=(*descriptorsBuffIndex)++;
	if((descriptor[rowIndex+1][0]!=-1)&&(rowIndex<NUM_OF_ALTERNATIVE_SETS))
	{
		strcat(descriptorsBuff,", ");
		*descriptorsBuffIndex=(*descriptorsBuffIndex)+2;
	}

}

/*displays the descriptors displayed in a string*/
void createDescriptors(OUT char * descriptorsBuff)
{
	int rowIndex,descriptorsBuffIndex=0;
	if(descriptor[0][0]!=-1)
	{
		descriptorsBuff[0]='{';
		descriptorsBuffIndex++;
		for(rowIndex=0;(rowIndex<NUM_OF_ALTERNATIVE_SETS)&&(descriptor[rowIndex][0]!=-1);rowIndex++)
			createSingleDescriptor(rowIndex,descriptorsBuff,&descriptorsBuffIndex);
		descriptorsBuff[descriptorsBuffIndex]='}';
		descriptorsBuffIndex++;
	}

}

void Descriptors2Window(HWND hWnd,mciCallHandleType hCall,int listControlID,mciDirectionType direction)
{
    HWND listID;
    LV_ITEM lvi;

    int numOfDesc=0,descIdx,i,j,itemNum;
  	char descriptorsBuff[100];

    listID=GetDlgItem(hWnd,listControlID);
    itemNum=ListView_GetItemCount(listID);

	if(mciCapGetNumOfDescriptors(hCall,direction,&numOfDesc) == MCI_RV_OK) {
		for(i=0;i<100;i++)
			descriptorsBuff[i]=0;
		
		
		
		for(descIdx=1;descIdx<=numOfDesc;descIdx++)
		{
			mciCapGetDescriptor(hCall,direction,descIdx,NUM_OF_ALTERNATIVE_SETS,SIZE_OF_ALTERNATIVE_SET,(void *)descriptor) ;
			for(i=0;i<100;i++)
				descriptorsBuff[i]=0;
			createDescriptors(descriptorsBuff);
			lvi.mask=LVIF_TEXT|LVIF_PARAM;
			lvi.iItem=itemNum;
			lvi.pszText=descriptorsBuff;
			lvi.iSubItem=0;
			lvi.lParam=(LPARAM)descIdx;
			j=ListView_InsertItem(listID,&lvi);
			itemNum++;
			
			
		}
		
	}
}


void initDescriptors(HWND hWnd,int listControlID)
{
    LV_COLUMN lvc;
    lvc.mask = LVCF_TEXT | LVCF_WIDTH;
    lvc.pszText ="";
    lvc.cx = SendDlgItemMessage(hWnd,listControlID, LVM_GETSTRINGWIDTH, 0, (LPARAM)"xxxxxxxxxxxxxxx") + 200;
    SendDlgItemMessage(hWnd,listControlID,LVM_INSERTCOLUMN,0,(LPARAM)&lvc);

}

#ifdef __cplusplus
}
#endif
