/*
*********************************************************************************
*                                                                               *
* NOTICE:                                                                       *
* This document contains information that is confidential and proprietary to    *
* RADVision LTD.. No part of this publication may be reproduced in any form     *
* whatsoever without written prior approval by RADVision LTD..                  *
*                                                                               * 
* RADVision LTD. reserves the right to revise this publication and make changes *
* without obligation to notify any person of such revisions or changes.         *    
* Copyright RADVision 1996.                                                     *
* Last Revision: Jan. 2000                                                      *
*********************************************************************************
*/

/* File : mcmain.c */


#ifdef WIN32
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#endif

/*-------------------------------------------------------------------------
                         Include header files
 -------------------------------------------------------------------------*/

#include <cm.h>
#include "resource.h"
#include "rvcommon.h"

#include "gkadd.h"
#include "gkrgi.h"
#include "gkcci.h"

#include "mci_types.h"
#include "mci_report.h"
#include "mci_general.h"
#include "mci_chan.h"
#include "mci_call.h"
#include "mci_conf.h"
#include "mci_mc.h"
#include "mci_call_caps.h"
#include "gkmc.h"
#include "mcgki.h"

#include "mcconfig.h"
#include "mcmain.h"
#include "mcrefresh.h"
#include "mcconferences.h"

#include <stdio.h>
#include <stdarg.h>

/*----------------------------------------------------------------------
                               Definitions
 -----------------------------------------------------------------------*/

#define MAX_DEBUG_LINES (5000)  //the maximun lines in the main window.
 
 
 
 /*------------------------------------------------------------------------
                              Static variables
 -------------------------------------------------------------------------*/
static mciMcHandleType hMc;
static HWND mcMainWnd = NULL;
static HWND	mcExceptionWnd = NULL;
static HWND globalWnd;

HINSTANCE hInst;
static configInput		configData;

static	FILE*	logfile = NULL;


/*------------------------------------------------------------------------
                            Static functions prototypes
 -------------------------------------------------------------------------*/

static BOOL CALLBACK mainWndProc (HWND hWnd, 
								  UINT message, 
								  WPARAM wParam, 
								  LPARAM lParam);


static BOOL wmInitHandler (mciMcHandleType* hMc,
						   HWND hWnd);


static BOOL wmCommandHandler (HWND hWnd, 
							  mciMcHandleType hMc, 
							  WPARAM wParam, 
							  LPARAM lParam);

static BOOL wmCloseHandler	(mciMcHandleType hMc,
							 HWND hWnd);



static void mcWinReport (IN UINT32 logClass,
						 IN char* msgStr, 
						 IN const char* fileName, 
						 IN int line);


static MciRV CALLCONV setEventHandlers (IN  mciMcHandleType hMc);



/*-----------------------------------------------------------------------------*/

void getMcMainHwnd(HWND *hWnd) {
	*hWnd=globalWnd;
}

/*------------------------------------------------------------------------------*/

int APIENTRY WinMain (HINSTANCE hInstance, 
					  HINSTANCE hPrevInstance,
					  LPSTR lpCmdLine, int nCmdShow)  {
	InitCommonControls();
	hInst = hInstance;
	return DialogBox (hInstance, MAKEINTRESOURCE(IDD_MC), NULL, mainWndProc);
}

/*------------------------------------------------------------------------------------------*/

static BOOL CALLBACK mainWndProc (HWND hWnd, UINT message, 
								  WPARAM wParam, LPARAM lParam) {
	
	switch (message) {
	case WM_INITDIALOG:
		mcMainWnd = hWnd;   //save the window handle in global variable
		globalWnd=hWnd;     
        CheckDlgButton(hWnd,IDC_CHECK_CHANNEL,1);
        CheckDlgButton(hWnd,IDC_CHECK_MSD,1);
        CheckDlgButton(hWnd,IDC_CHECK_CE,1);
		return wmInitHandler (&hMc, hWnd);
	case WM_COMMAND:
		return wmCommandHandler (hWnd, hMc, wParam, lParam);
	case WM_CLOSE:
		return wmCloseHandler (hMc,hWnd);
	case WM_PARENTNOTIFY:
		return TRUE;
	default:
		return FALSE;
	}
}

/*------------------------------------------------------------------------------------------*/

MciRV CALLCONV reportInit()
{	
	if ((logfile = fopen("mc.log", "w")) == NULL)
		return MCI_RV_CANNOT_OPEN_FILE;

	return MCI_RV_OK;
}

/*------------------------------------------------------------------------------------------*/

MciRV CALLCONV reportClose()
{
	if (fclose(logfile) != 0)
		return MCI_RV_CANNOT_CLOSE_FILE;

	return MCI_RV_OK;
}

/*------------------------------------------------------------------------------------------*/

static BOOL wmInitHandler(mciMcHandleType* hMc, HWND hWnd)
{
	
	
	//definitions of configuration structures
	mciReportEvStructType	logHandlers;
	mciStackConfigType		stackConfig;
	mciMcCoreConfigType     coreConfig;
	mciTransportType        mcListen;

	
	static HAPP				hStack;
	char vbuf[64],version[64],stackVersion[64];
	mcMainWnd = GetDlgItem (hWnd, IDC_OUTPUT);
	mcExceptionWnd = GetDlgItem (hWnd, IDC_EXCEPTION);

    
    
	//reseting configuration structures
	memset(&(logHandlers), 0, sizeof (mciReportEvStructType));
    memset(&(stackConfig), 0, sizeof (mciStackConfigType));
    memset(&(coreConfig), 0, sizeof (mciMcCoreConfigType));
    memset(&(mcListen), 0, sizeof(mciTransportType));
	
	//initializing the configuration window 
	//the configuration data will be set in the configData struct passed to the function
	memset(&(configData), 0, sizeof (configInput));
	if(DialogBoxParam(hInst,MAKEINTRESOURCE(IDD_CONFIG),hWnd,configWndProc,(LPARAM)&configData)==END_MC)
		EndDialog(hWnd,0);

	
	//initializing configuration structures 
    logHandlers.mciEvReportInit = reportInit;    
    logHandlers.mciEvReportClose = reportClose;
	logHandlers.mciEvReport = mcOutputMsg;
  
   	coreConfig.maxMCs = 1;
	coreConfig.maxConfs = configData.maxConfs;
	coreConfig.maxCalls = configData.maxCalls;
	coreConfig.maxChans = configData.maxChans;

	stackConfig.maxCalls = coreConfig.maxCalls;
    if(coreConfig.maxChans == 0 || coreConfig.maxCalls == 0) 
	     stackConfig.maxChans = 0;
    else 
		 stackConfig.maxChans = (coreConfig.maxChans % coreConfig.maxCalls == 0) ?
		                         coreConfig.maxChans / coreConfig.maxCalls :
                                 coreConfig.maxChans / coreConfig.maxCalls + 1;  
                                 
	
	mcListen.ip = DEFAULT_IP;
	mcListen.port = configData.port;
	
	
	
	//initializing mcCore
	if (mciInitMcCore (&coreConfig, &logHandlers) != MCI_RV_OK) {
		mciReport(LOG_EXCEPTION,logLevelExceptions,"!!Exception: MC Core init failure");
		exit(1);
	}

	//initializing stack
	if (mciInitStack (&stackConfig, &hStack) != MCI_RV_OK) {
		mciReport(LOG_EXCEPTION,logLevelExceptions,"!!Exception: Stack init failure");
		exit(1);
	}
	
	//initializing an MC entity
	if (mciMcInit (NULL, hStack, hMc) != MCI_RV_OK) {
		mciReport(LOG_EXCEPTION,logLevelExceptions,"!!Exception: MC init failure");
		exit(1);
	}
   
	//setting EventHandlers
	if (setEventHandlers(*hMc) != MCI_RV_OK) 
      	mciReport(LOG_EXCEPTION,logLevelExceptions,"!!Exception: MC failed setting handlers");
       

    //initialize mcgk
	if (!mcgkiInitialize (*hMc, &mcListen))
		mciReport (LOG_EXCEPTION,logLevelExceptions, "!!Exception: MC->GK Connection Failure");

	
	/* Display the MC and Stack versions on the top of the window */
	strcpy(stackVersion,cmGetVersion());
	mciMcCoreVersion(version);
    sprintf(vbuf, "%s / %s",version,stackVersion);
    SendDlgItemMessage(hWnd,IDC_STATIC_VERSION,WM_SETTEXT,0,(long)vbuf);

	return TRUE;

}	
	

/*------------------------------------------------------------------------------------------*/

static BOOL wmCommandHandler (HWND hWnd, mciMcHandleType hMc, WPARAM wParam, LPARAM lParam)
{
	switch (GET_WM_COMMAND_ID(wParam,lParam)) {
	case IDC_CLEAR:
		SendDlgItemMessage (hWnd, IDC_EXCEPTION, LB_RESETCONTENT, 0, 0);
		SendDlgItemMessage (hWnd, IDC_OUTPUT, LB_RESETCONTENT, 0, 0);
		return FALSE;
	case IDC_EXCEPTION:
		{
			static char logMsg[1024];

			INT32 selection = SendDlgItemMessage (hWnd, IDC_EXCEPTION, LB_GETCURSEL,0,0);
			if (selection == LB_ERR)
				return TRUE;
			SendDlgItemMessage (hWnd, IDC_EXCEPTION, LB_GETTEXT, selection, (LPARAM)logMsg);
			selection = SendDlgItemMessage (hWnd, IDC_OUTPUT, LB_FINDSTRINGEXACT, 0, (LPARAM)logMsg);
			SendDlgItemMessage (hWnd, IDC_OUTPUT, LB_SETCURSEL,selection,0);
			selection -= 15;
			if (selection < 0)
				selection = 0;
			SendDlgItemMessage (hWnd, IDC_OUTPUT, LB_SETTOPINDEX,selection,0);
			
			return FALSE;
		}	
	case IDC_LOGOPTIONS:
		DialogBox(hInst,MAKEINTRESOURCE(IDD_LOGOPTIONS),NULL,logOptionsWndProc);
		return FALSE;

	case IDC_CONFIG:
		DialogBoxParam(hInst,MAKEINTRESOURCE(IDD_CONFIG),hWnd,configWndProc,(LPARAM)&configData);
		return FALSE;

	case IDC_CONFERENCES:
			DialogBoxParam(hInst,MAKEINTRESOURCE(IDD_CONFERENCES),hWnd,conferenceWndProc,(LPARAM)hMc);
			return FALSE;
	default:
		return TRUE;
	}
}

/*------------------------------------------------------------------------------------------*/

static BOOL wmCloseHandler (mciMcHandleType hMc,HWND hWnd)
{
	static BOOL handled = FALSE;
    HAPP          hStack;

	if (!handled) {
		/*get the stack handle*/
		mcgkiTerminate(NULL);
		mciMcGetStackHandle (hMc, &hStack);

		/*close the stack*/
		cmEnd(hStack);
		
//     	mciMcClose (hMc);  
		
		
		/*free the mc*/
		mciCloseMcCore ();
		
	
		handled = TRUE;
	}
	PostQuitMessage (0);

	return TRUE;
}

/*------------------------------------------------------------------------------------------*/

MciRV CALLCONV mcOutputMsg (IN UINT32 logClass, IN const char* msg)
{
	static UINT16 messageCount = 0;


	if (mcMainWnd) {
		if (SendMessage(mcMainWnd,LB_GETCOUNT,0,0) > MAX_DEBUG_LINES)
		  SendMessage(mcMainWnd,LB_DELETESTRING,0,0);
		SendMessage (mcMainWnd, LB_ADDSTRING, 0, (LPARAM)msg);
		//messageCount++;
	}

	if (logClass == LOG_EXCEPTION)
		if (mcExceptionWnd) {
			SendMessage (mcExceptionWnd, LB_ADDSTRING, 0, (LPARAM)msg);
			SendMessage(mcMainWnd,LB_SETCURSEL,SendMessage(mcMainWnd,LB_GETCOUNT,0,0)-1,0);
			messageCount++;
		}

	if(logfile == NULL)
	  return MCI_RV_ERROR;
	fprintf (logfile, "%s\n", msg);
	fflush (logfile);

	return MCI_RV_OK;
}

/*------------------------------------------------------------------------------------------*/

static void mcWinReport (IN UINT32 logClass,
						 IN char* msgStr, IN const char* fileName, IN int line)
{
	static char reportStr[1024];
	char*		temp;

	if (temp = strrchr (fileName, '\\'))
		fileName = temp + 1;

	sprintf(reportStr,"%s  ('%s', line %d)", msgStr, fileName, line);
	mcOutputMsg (logClass,reportStr);
}

/*------------------------------------------------------------------------------------------*/
//----------------------application event handlers------------------------

RVAPI
MciRV  CALLCONV chanEvStateChangeHandler (IN mciChanAppHandleType haChan,
		                                  IN mciChanHandleType hChan,
		                            	  IN mciCallHandleType  hCall, 
										  IN mciChanStateType   state,
 										  IN mciChanReasonType stateMode)
{
    
	MciRV rv;


	if (IsDlgButtonChecked(globalWnd,IDC_CHECK_CHANNEL) == 1)
	{ //auto
		//refreshing before destroying the handle in order to be able to get the
		//call from the handle
		if(state==mciChanStateIdle)
			RefreshChanDialog(hChan,INVALID_HANDLE);
		else
			RefreshChanDialog(hChan,VALID_HANDLE);
		
		rv=mciDefEvPTPChanStateHandler (haChan, hChan, hCall,state,stateMode);
		return rv;
	}
	else//manual
	{
		RefreshChanDialog(hChan,VALID_HANDLE);
		return FALSE;
	}
	
	return MCI_RV_OK;
}
/*---------------------------set application event handler ---------------------------------*/

static
MciRV  CALLCONV callEvPTPStateChangeHandler (IN  mciCallAppHandleType   haCall,
										  IN  mciCallHandleType      hCall,
										  IN  mciCallStateType       callState,
										  IN  mciCallStateReasonType callReason )
{
	MciRV		rv;

	if(callState == mciCallSttIdle)
		// refreshing before destroying the handle in order to be able to get the 	
		// conf from the handle
		RefreshCallDialog(hCall,INVALID_HANDLE);
	else
		RefreshCallDialog(hCall,VALID_HANDLE);

	rv = mciDefEvPTPCallStateHandler (haCall, hCall, callState, callReason);
	if (rv != MCI_RV_OK)
		return rv;
	
	rv = mcgkiPTPCallStateHandler (haCall, hCall, callState, callReason);
	if (rv != MCI_RV_OK)
		return rv;

	return MCI_RV_OK;
}

/*------------------------------------------------------------------------------------------*/
//yonatan 3.11:

static
MciRV  CALLCONV confEvStateChangeHandler (IN mciConfAppHandleType haConf,
									      IN mciConfHandleType	  hConf,
                                          IN mciConfStateType	  confState,
                                          IN mciConfReasonType    confReason)
{
	MciRV rv;


	if(confState==mciConfSttInactive)
		//refreshing before destroying the handle in order to be able to get the
		//call from the handle
		RefreshConfDialog(hConf,INVALID_HANDLE);
	else
		RefreshConfDialog(hConf,VALID_HANDLE);
	
	rv=mciDefEvPTPConfStateHandler(haConf,hConf,confState,confReason);

	return rv;
}

/*------------------------------------------------------------------------------------------*/

static
MciRV  CALLCONV callMsdEvPTPMsdStateHandler (IN mciCallAppHandleType	haCall,
											 IN mciCallHandleType		hCall,
											 IN mciMsdStateType			msdState,
                                             IN mciMsdReasonType		msdReason)
{
	MciRV rv;

	if(IsDlgButtonChecked(globalWnd,IDC_CHECK_MSD)) { //auto
		rv=mciDefEvPTPCallMsdStateHandler(haCall,hCall,msdState,msdReason);
		RefreshCallDialog(hCall,VALID_HANDLE);
		return rv;
	}
	else //manual
	{
		RefreshCallDialog(hCall,VALID_HANDLE);
		return MCI_RV_OK;
	}		
}

/*------------------------------------------------------------------------------------------*/

static
MciRV  CALLCONV callCeEvPTPCEStateHandler (IN mciCallAppHandleType 	haCall,
                                           IN mciCallHandleType     hCall,
                                           IN mciDirectionType      direction,
                                           IN mciCEStateType		ceEvent,
                                           IN mciCEReasonType		ceReason)
{
	MciRV rv;

	if(IsDlgButtonChecked(globalWnd,IDC_CHECK_CE)){ //auto
		rv=mciDefEvPTPCallCEStateHandler(haCall,hCall,direction,ceEvent,ceReason);
		RefreshCallDialog(hCall,VALID_HANDLE);
		return rv;
	}
	else //manual
	{
		RefreshCallDialog(hCall,VALID_HANDLE);
		return MCI_RV_OK;
	}
 
	
}

/*------------------------------------------------------------------------------------------*/

static MciRV CALLCONV setEventHandlers (IN  mciMcHandleType hMc)

{
	MciRV					rv1, rv2, rv3;
	//definition of event structures
	mciChanEvStructType		mcChanEvH;
	mciCallEvStructType		mcCallEvH;
	mciConfEvStructType		mcConfEvH;
    //reset event structure
	memset(&mcConfEvH, 0, sizeof(mciConfEvStructType));
	memset(&mcChanEvH, 0, sizeof(mciChanEvStructType));
	memset(&mcCallEvH, 0, sizeof(mciCallEvStructType));
 
    //set event handlers	
	mcConfEvH.ptpStateChangeHandler = confEvStateChangeHandler;

	mcChanEvH.stateChangeHandler = chanEvStateChangeHandler;

	mcCallEvH.ptpCallStateHandler =	callEvPTPStateChangeHandler;
	mcCallEvH.ptpCapsHandler	  = callCeEvPTPCEStateHandler;
	mcCallEvH.ptpMsdHandler		  =	callMsdEvPTPMsdStateHandler; 

	//set event handlers structures in mcCore
	rv1 = mciConfSetEvHandlers(hMc,&mcConfEvH);
	rv2 = mciChanSetEvHandlers(hMc, &mcChanEvH);
	rv3 = mciCallSetEvHandlers (hMc, &mcCallEvH);

	
	if (rv1 != MCI_RV_OK || rv2 != MCI_RV_OK || rv3 != MCI_RV_OK)
		return MCI_RV_ERROR;

	return MCI_RV_OK;
}

/*------------------------------------------------------------------------------------------*/

