#ifdef __cplusplus
extern "C" {
#endif



/*

NOTICE:
This document contains information that is proprietary to RADVision LTD..
No part of this publication may be reproduced IN  any form whatsoever without 
written prior approval by RADVision LTD..

RADVision LTD. reserves the right to revise this publication and make changes 
without obligation to notify any person of such revisions or changes.

*/

/****************************************************************************
  
  ci.h  --  Configuration Interface Header

  Module Author: Oz Solomonovich
  This Comment:  04-Feb-1997

****************************************************************************/


#ifndef __CI_H
#define __CI_H



#include <rvcommon.h>

DECLARE_OPAQUE(HCFG);

typedef enum
{
     ERR_CI_GENERALERROR     = -1,    /* general error */
     ERR_CI_BUFFERTOOSMALL   = -2,    /* return buffer was too small */
     ERR_CI_NOTFOUND         = -3,    /* the value was not found */
     ERR_CI_INTERFACEUNKNOWN = -4,    /* can't ID input/output source */
     ERR_CI_ALLOCERROR       = -5,    /* couldn't allocate memory */

     ERR_CI_REGOPENKEY       = -10,   /* couldn't open registry key */
     ERR_CI_REGCREATE        = -11,   /* couldn't create registry key */

     ERR_CI_FILEOPEN         = -20,   /* couldn't open file */
     ERR_CI_FILEREAD         = -21,   /* error reading from file */
     ERR_CI_FILEWRITE        = -22    /* error writing to file */
} ci_errors;


RVAPI
HCFG CALLCONV ciConstruct(
        IN  const char * source      /* configuration source identifier */
        );


RVAPI
HCFG CALLCONV ciConstructEx(
        IN  const char * source,     /* configuration source identifier */
        IN  int          extraData,  /* maximum amount of data growth */
        IN  int          extraNodes  /* maximum number of nodes growth */
        );

RVAPI
ci_errors CALLCONV ciDestruct(
        IN  HCFG hCfg
        );  


RVAPI
ci_errors CALLCONV ciSave(
        IN  HCFG hCfg,
        IN  char *target
        );


RVAPI
ci_errors CALLCONV ciGetValue(
        IN   HCFG         hCfg,
        IN   const char * path,      /* full path to node, i.e. "a.b.c" */
        OUT  BOOL *       isString,
        OUT  INT32 *      value      /* data for ints, length for strings */
        );


RVAPI
ci_errors CALLCONV ciGetString(
        IN  HCFG         hCfg,
        IN  const char * path,       /* full path to node, i.e. "a.b.c" */
        OUT char *       str,    
        IN  UINT32       maxStrLen   /* length of output string buffer */
        );


RVAPI      /* returns ERR_CI_NOTFOUND when there is no 'next' value */
ci_errors CALLCONV ciNext(
        IN  HCFG         hCfg,
        IN  const char * path,       /* full path to node, or NULL to start
                                        at first node */
        OUT char *       nextPath,   /* buffer for next path IN cfg */
        IN  UINT32       maxPathLen  /* length of output buffer */
        );


RVAPI
ci_errors CALLCONV ciSetValue(
        IN  HCFG         hCfg,
        IN  const char * path,       /* full path to node */
        IN  BOOL         isString,
        IN  INT32        value,      /* data for ints, length for strings */
        IN  const char * str         /* null for ints, data for strings */
        );


RVAPI
ci_errors CALLCONV ciDeleteValue(
        IN  HCFG hCfg,
        IN  const char * path
        );



#endif /* __CI_H */
#ifdef __cplusplus
}              
#endif



