#ifdef __cplusplus
extern "C" {
#endif



/*

NOTICE:
This document contains information that is proprietary to RADVision LTD..
No part of this publication may be reproduced in any form whatsoever without 
written prior approval by RADVision LTD..

RADVision LTD. reserves the right to revise this publication and make changes 
without obligation to notify any person of such revisions or changes.

*/ 

#ifndef CM_H
#define CM_H



#include <rvcommon.h>

#ifndef RVAPI
#define RVAPI
#endif  

#ifndef CALLCONV
#define CALLCONV
#endif  

DECLARE_OPAQUE(HAPP);
DECLARE_OPAQUE(HAPPAPP);
DECLARE_OPAQUE(HAPPCALL);
DECLARE_OPAQUE(HCALL);
DECLARE_OPAQUE(HPROTCONN);
DECLARE_OPAQUE(HPROTOCOL);
DECLARE_OPAQUE(HAPPCONN);




#include <pvaltree.h>


typedef enum
{
    cmQ931TpktChannel=0,
    cmH245TpktChannel
}cmTpktChanHandleType;

typedef enum
{
    cmRasUdpChannel=0,
    cmRasUdpChannelMCast
}cmUdpChanHandleType;


typedef enum
{
    cmQ931setup =5,
    cmQ931callProceeding=2,	
    cmQ931connect=7,
    cmQ931alerting=1,		
    cmQ931releaseComplete=90,	
    cmQ931status=125,			
    cmQ931facility=98,		
    cmQ931statusEnquiry=117,	
    cmQ931progress=3,		
    cmQ931setupAck=13,
    cmQ931information=123

}cmCallQ931MsgType;

typedef struct
{
	UINT8 cmCodingStandard;
	UINT8 cmSpare;
	UINT8 cmLocation;
	UINT8 cmRecomendation; /*  optional set to -1 if not used */
	UINT8 cmCauseValue;
}
cmCallCauseInfo;

typedef struct 
{
	UINT8 cmCodingStandard;
	UINT8 cmCallStateValue;
}
cmCallStateInfo;

typedef struct
{
	cmCallCauseInfo  callCauseInfo;
	cmCallStateInfo callStateInfo;
	char display[128];
}cmCallStatusMessage;

typedef struct {
  /* Set one of the following choises */

  /* 1. object id */
  char object[128]; /* in object identifier ASN format: "0 1 2" */
  int objectLength; /* in bytes. <=0 if not set */

  /* 2. h.221 id */
  UINT8 t35CountryCode;
  UINT8 t35Extension;
  UINT16 manufacturerCode;
} cmNonStandardIdentifier;

typedef struct
{
    BYTE    t35CountryCode;
    BYTE    t35Extension;
    UINT16  manufacturerCode;
} cmNonStandard;

typedef struct
{
    cmNonStandard   info;
    int             productLen;
    int             versionLen;
    char            productID[256];
    char            versionID[256];
} cmVendor;

typedef struct
{
    cmNonStandardIdentifier   info;
    int			      length;
    char*		      data;
} cmNonStandardParam;


/* --- transport address --- */

typedef enum {
  cmTransportTypeIP, /* No route */
  cmTransportTypeIPStrictRoute,
  cmTransportTypeIPLooseRoute
} cmTransportType;

#define cmRouteSize (7) /* for luck */

typedef enum {
  cmDistributionUnicast,
  cmDistributionMulticast
} cmDistribution;

typedef struct
{
  UINT16          length; /* length in bytes of route */
  UINT32          ip; 
  UINT16          port; 
  cmTransportType type; 
  UINT32          route[cmRouteSize];
  cmDistribution  distribution;
} cmTransportAddress;


typedef enum
{
    cmAliasTypeE164=cmTransportTypeIPLooseRoute+5,
    cmAliasTypeH323ID,
    cmAliasTypeEndpointID,
    cmAliasTypeGatekeeperID,
    cmAliasTypeURLID,
    cmAliasTypeTransportAddress,
	cmAliasTypeEMailID,
	cmAliasTypePartyNumber
} cmAliasType;


typedef enum
{   
		    /* the numbering plan is according to
		       Recommendations E.163 and E.164.*/
    cmPartyNumberPublicUnknown,
		/* if used number digits carry prefix indicating type
		   of number according to national recommendations*/
    cmPartyNumberPublicInternationalNumber	,
    cmPartyNumberPublicNationalNumber	,
    cmPartyNumberPublicNetworkSpecificNumber,
		/* not used, value reserved */
    cmPartyNumberPublicSubscriberNumber	,
    cmPartyNumberPublicAbbreviatedNumber	,
	    /* valid only for called party number at the outgoing
	       access, network substitutes appropriate number. */
	    
    cmPartyNumberDataPartyNumber,
	    /* not used, value reserved */
    cmPartyNumberTelexPartyNumber,
	    /* not used, value reserved*/
    cmPartyNumberPrivateUnknown,			
    cmPartyNumberPrivateLevel2RegionalNumber,	
    cmPartyNumberPrivateLevel1RegionalNumber,	
    cmPartyNumberPrivatePISNSpecificNumber,	
    cmPartyNumberPrivateLocalNumber,		
    cmPartyNumberPrivateAbbreviatedNumber,	
    cmPartyNumberNationalStandardPartyNumber 
} cmPartyNumberType;

typedef struct 
{
	cmAliasType	  type;
	UINT16		  length; /* string in bytes */
	char*		  string;
	cmPartyNumberType pnType;
	cmTransportAddress	  transport;  
} cmAlias;

typedef enum 
{
    cmEndpointTypeTerminal,
    cmEndpointTypeGateway,
    cmEndpointTypeMCU,
    cmEndpointTypeGK,
    cmEndpointTypeUndefined
} cmEndpointType;

typedef enum
{
	cmCallTypeP2P,
	cmCallTypeOne2N,
	cmCallTypeN2One,
	cmCallTypeN2Nw
} cmCallType;


typedef enum
{
	cmCallModelTypeDirect,
	cmCallModelTypeGKRouted
} cmCallModelType;

typedef enum
{
	cmTransportQOSEndpointControlled,
	cmTransportQOSGatekeeperControlled,
	cmTransportQOSNoControl
} cmTransportQOS;

#include <cmras.h>
#include <cmctrl.h>
#include <cmhook.h>
#include <faststart.h>
#ifdef AUTOCHAN
#include <cmautochan.h>
#endif
typedef enum
{
	cmPreGrantedArqDirect=0,
	cmPreGrantedArqRouted
}cmPreGrantedArqUse;

typedef enum 
{
    cmCallStateDialtone,
    cmCallStateProceeding,
    cmCallStateRingBack,
    cmCallStateConnected,       
    cmCallStateDisconnected,            
    cmCallStateIdle,
    cmCallStateOffering,
    cmCallStateTransfering,
    cmCallStateAdmissionConfirm,/*marina*/
    cmCallStateAdmissionReject ,/*marina*/
    cmCallStateIncompleteAddress,
    cmCallStateWaitAddressAck



} cmCallState_e;

typedef enum 
{
    cmCallStateModeDisconnectedBusy,    
    cmCallStateModeDisconnectedNormal,  
    cmCallStateModeDisconnectedReject,  
    cmCallStateModeDisconnectedUnreachable,
    cmCallStateModeDisconnectedUnknown,
    cmCallStateModeDisconnectedLocal,
  
    cmCallStateModeConnectedControl,
    cmCallStateModeConnectedCallSetup,
    cmCallStateModeConnectedCall,
    cmCallStateModeConnectedConference,

    cmCallStateModeControlConnected=cmCallStateModeConnectedControl,
    cmCallStateModeCallSetupConnected,
    cmCallStateModeCallConnected,
     
    cmCallStateModeOfferingCreate=cmCallStateModeConnectedConference+1,
    cmCallStateModeOfferingInvite,
    cmCallStateModeOfferingJoin,
    cmCallStateModeOfferingCapabilityNegotiation,
    cmCallStateModeOfferingCallIndependentSupplementaryService,
    cmCallStateModeDisconnectedIncompleteAddress

} cmCallStateMode_e;

typedef enum
{
    cmReasonTypeNoBandwidth,
    cmReasonTypeGatekeeperResource,
    cmReasonTypeUnreachableDestination,
    cmReasonTypeDestinationRejection,
    cmReasonTypeInvalidRevision,
    cmReasonTypeNoPermision,
    cmReasonTypeUnreachableGatekeeper,
    cmReasonTypeGatewayResource,
    cmReasonTypeBadFormatAddress,
    cmReasonTypeAdaptiveBusy,
    cmReasonTypeInConf,
    cmReasonTypeUndefinedReason,
    cmReasonTypeRouteCallToGatekeeper,
    cmReasonTypeCallForwarded,
    cmReasonTypeRouteCallToMC,
    cmReasonTypeFacilityCallDeflection,
    cmReasonTypeSecurityDenied,					
    cmReasonTypeCalledPartyNotRegistered,        
    cmReasonTypeCallerNotregistered,
	cmReasonTypeConferenceListChoice,
	cmReasonTypeStartH245	
} cmReasonType;



typedef enum
{
    cmCreate,
    cmJoin,
    cmInvite,
    cmCapabilityNegotiation,          
    cmCallIndependentSupplementaryService
} cmConferenceGoalType;


typedef enum
{
    cmITCSpeech=0,
    cmITCUnrestricted=8,
    cmITCRestricted=9,
    cmITCAudio=16,
    cmITCUnrestrictedTones=17,
    cmITCVideo=24
} cmInformationTransferCapability;

typedef enum
{
    cmParamFirst=0x1000,
    cmParamSourceAddress,         /*cmAlias */
    cmParamSourceInfo,            /*cmEndpointType*/
    cmParamDestinationAddress,    /*cmAlias*/
    cmParamDestCallSignalAddress, /*cmTransportAddress*/
    cmParamSrcCallSignalAddress,  /*cmTransportAddress*/
    cmParamDestExtraCallInfo,     /*cmAlias*/
    cmParamActiveMc,              /*BOOL*/  
    cmParamCID,                   /*char*  */
    cmParamConferenceGoal,        /*ENUM */ 
    cmParamCallType,              /*cmCallType*/    
    cmParamSetupH245Address,      /*cmTransportAddress */  
    cmParamH245Address,           /*cmTransportAddress */  
    cmParamDestinationInfo,       /*endpoint type */
    cmParamReleaseCompleteReason, /*cmReasonType */  
    cmParamReleaseCompleteCause,  /*int */  
    cmParamRate,                  /*int */  
    cmParamRequestedRate,         /*int */  
    cmParamInformationTransferCapability,
    cmParamMultiRate,
    cmParamCalledPartyNumber,     /*cmAlias*/
    cmParamCalledPartySubAddress, /*cmAlias*/
    cmParamCallingPartyNumber,     /*cmAlias*/
    cmParamCallingPartySubAddress, /*cmAlias*/
    cmParamExtention,              /*cmAlias*/
    cmParamAlternativeAddress,      /*For future use */
    cmParamAlternativeAliasAddress, /*For future use */
    cmParamFacilityReason,
    cmParamDestinationIpAddress,  /* physical address cmRASTransport- used to detrmine acutal destination of call */
    cmParamRemoteIpAddress,
    cmParamCRV,
    cmParamRASCRV,
    cmParamCallSignalAddress,
    cmParamEstablishH245,
    cmParamDisplay,
    cmParamFacilityCID,          /*char*  */
    cmParamConnectDisplay,
    cmParamUserUser,
    cmParamConnectUserUser,
    cmParamFullSourceInfo,
    cmParamFullDestinationInfo,
    cmParamSetupNonStandardData,	  
    cmParamCallProceedingNonStandardData,
    cmParamAlertingNonStandardData,	  
    cmParamConnectNonStandardData,	  
    cmParamReleaseCompleteNonStandardData,
    cmParamSetupNonStandard,	        
    cmParamCallProceedingNonStandard,   
    cmParamAlertingNonStandard,	        
    cmParamConnectNonStandard,	        
    cmParamReleaseCompleteNonStandard, 
    cmParamAlternativeDestExtraCallInfo,
    cmParamAlternativeExtention,
    cmParamSetupFastStart,  
    cmParamConnectFastStart,
    cmParamEarlyH245,
    cmParamCallID,
    cmParamPreGrantedArq, /*marina */
    cmParamAlertingFastStart,
    cmParamCallProcFastStart,
    cmParamAlertingH245Address,
    cmParamCallProcH245Address,
    cmParamCanOverlapSending,
    cmParamIncompleteAddress,
    cmParamH245Tunneling,
    cmParamFastStartInUse,
    cmParamSetupCanOverlapSending
} cmCallParam;


typedef int cmParam;

typedef int 
(*cmNewCallbackEH)(void);


typedef enum
{
   cmIdle,
   cmDiscovered,
   cmRegistered
} cmRegState;

typedef enum
{
   cmGatekeeperConfirm,
   cmGatekeeperReject,
   cmRegistrationConfirm,
   cmRegistrationReject,
   cmUnregistrationRequest,
   cmUnregistrationConfirm,
   cmUnregistrationReject,
   cmNonStandardMessage,
   cmResourceAvailabilityConfirmation
} cmRegEvent;


typedef int 
    (CALLCONV *cmEvNewCallT)(
		IN      HAPP                hApp,
		IN      HCALL               hsCall,
		OUT     LPHAPPCALL          lphaCall);


typedef int 
    (CALLCONV *cmEvRegEventT)(
		IN      HAPP                hApp,
                IN      cmRegState          regState,
		IN      cmRegEvent          regEvent,
		IN      int                 regEventHandle);

typedef  struct 
{
    cmEvNewCallT cmEvNewCall;
    cmEvRegEventT cmEvRegEvent;
} SCMEVENT,*CMEVENT;

typedef int   
    (CALLCONV *cmEvCallStateChangedT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      UINT32              state,
		IN      UINT32              stateMode);


typedef int 
    (CALLCONV *cmEvCallNewRateT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      UINT32              rate);

typedef int 
    (CALLCONV*cmEvCallInfoT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      char*               display,
		IN      char*               userUser,
                IN      int                 userUserSize);
    
typedef int 
    (CALLCONV*cmCallNonStandardParamT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      char*               data,
		IN      int                 dataSize);   

typedef int 
    (CALLCONV*cmEvCallFacilityT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      int                 handle,
		OUT IN	BOOL		    *proceed);   
typedef int 
    (CALLCONV*cmEvCallFastStartSetupT)(
		IN	HAPPCALL haCall, 
		IN	HCALL hsCall,
		OUT IN	cmFastStartMessage *fsMessage);

typedef int 
    (CALLCONV * cmEvCallStatusT)(
			IN	HAPPCALL haCall, 
			IN	HCALL hsCall,
			OUT IN	cmCallStatusMessage *callStatusMsg);
typedef int 
    (CALLCONV * cmEvCallUserInfoT)(
			IN	HAPPCALL haCall, 
			IN	HCALL hsCall,
			IN	int nodeId);

typedef int 
    (CALLCONV * cmEvCallH450SupplServT)(
			IN	HAPPCALL haCall, 
			IN	HCALL hsCall,
			cmCallQ931MsgType msgType,
			int	nodeId,
			int	size);
typedef int 
    (CALLCONV * cmEvCallIncompleteAddressT)(
			IN	HAPPCALL haCall, 
			IN	HCALL hsCall);
typedef int
    (CALLCONV * cmEvCallAdditionalAddressT)(
			IN	HAPPCALL haCall, 
			IN	HCALL hsCall,
			char	* address,
			BOOL	sendingComplete);

#ifdef AUTOCHAN

typedef int 
    (CALLCONV * cmEvCallAutoChannelsUpdateT)(
			IN	HAPPCALL haCall, 
			IN	HCALL hsCall,
			cmAutoChannelsStruct  *autoChannels);
#endif

typedef int 
    (CALLCONV * cmEvCallTunnNewMessageT)(
					 IN	HAPPCALL haCall, 
					 IN	HCALL hsCall,
					 IN     int vtNodeId,
					 IN     int vtAddNodeId,
					 OUT    BOOL *wait);

typedef  struct 
{
    cmEvCallStateChangedT   cmEvCallStateChanged;
    cmEvCallNewRateT        cmEvCallNewRate;
    cmEvCallInfoT           cmEvCallInfo;  
    cmCallNonStandardParamT cmCallNonStandardParam;
    cmEvCallFacilityT	    cmEvCallFacility;
    cmEvCallFastStartSetupT cmEvCallFastStartSetup;
    cmEvCallStatusT	    cmEvCallStatus;
    cmEvCallUserInfoT	    cmEvCallUserInfo;
    cmEvCallH450SupplServT	cmEvCallH450SupplServ;
    cmEvCallIncompleteAddressT	cmEvCallIncompleteAddress;
    cmEvCallAdditionalAddressT	cmEvCallAdditionalAddress;

  cmEvCallTunnNewMessageT cmEvCallTunnNewMessage;

#ifdef AUTOCHAN
    cmEvCallAutoChannelsUpdateT	cmEvCallAutoChannelsUpdate;
#endif


} SCMCALLEVENT,*CMCALLEVENT;
    

RVAPI
int CALLCONV cmInitialize(char * name,LPHAPP lphApp);

RVAPI
HAPP CALLCONV cmInit(
		IN      char*               appName);

RVAPI
int CALLCONV cmEnd(
		IN      HAPP                hApp);


RVAPI
int CALLCONV cmStart(
		IN	HAPP		hApp);

RVAPI
int CALLCONV cmStop(
		IN	HAPP		hApp);

RVAPI
int CALLCONV cmRegister(
		IN	HAPP		hApp);

RVAPI
int CALLCONV cmUnregister(
		IN	HAPP		hApp);

RVAPI 
int CALLCONV cmSendNonStandardMessage(
               IN      HAPP                hApp,
	       IN      cmNonStandardParam* param);
RVAPI
char* CALLCONV cmGetVersion(void);

RVAPI
int CALLCONV cmGetVersionName(
		IN	char*		buff,
		IN	int		length);


RVAPI
int CALLCONV
cmSetPreCallbackEvent(
		IN      HAPP                hApp,
		IN      cmNewCallbackEH     newCallback
		);

RVAPI
int CALLCONV cmSetGenEventHandler(
		IN      HAPP                hApp,
		IN      CMEVENT             cmEvent,
		IN      int                 size);

RVAPI
int CALLCONV cmGetGenEventHandler(
		IN      HAPP                hApp,
		IN      CMEVENT             cmEvent,
		IN      int                 size);


RVAPI
int CALLCONV cmSetCallEventHandler(
		IN      HAPP                hApp,
		IN      CMCALLEVENT         cmCallEvent,
		IN      int                 size);
RVAPI
int CALLCONV cmGetCallEventHandler(
		IN      HAPP                hApp,
		OUT      CMCALLEVENT         cmCallEvent,
		INOUT      int                 size);


RVAPI 
int CALLCONV cmCallNew(
		IN    HAPP                  hApp,
		IN    HAPPCALL              haCall,
		OUT   LPHCALL               lphsCall);

RVAPI
int CALLCONV cmCallDial(
		IN    HCALL                 hsCall);


RVAPI
int CALLCONV cmCallMake(
		IN      HCALL               hsCall,
		IN      UINT32              maxRate,
		IN      UINT32              minRate,
		IN      char*               destAddress,
		IN      char*               srcAddress,
		IN      char*               display,
		IN      char*               userUser,
                IN      int                 userUserSize);


RVAPI
int CALLCONV cmCallFacility(
	       IN HCALL		hsCall,
	       IN int		handle);

RVAPI
int CALLCONV cmSetCallHandle(
		IN HCALL                    hsCall,
		IN HAPPCALL                 haCall);


RVAPI
int CALLCONV cmCallGetDisplayInfo(
		IN 	HCALL		    hsCall,
                IN      char*               display,
                IN      int                 displaySize);

RVAPI
int CALLCONV cmCallGetUserUserInfo(
		IN 	HCALL		    hsCall,
                IN      char*               userUser,
                IN      int                 userUserSize);

RVAPI
int CALLCONV cmCallAnswerExt(
		IN 	HCALL		    hsCall,
                IN      char*               display,
                IN      char*               userUser,
                IN      int                 userUserSize);

RVAPI
int CALLCONV cmCallAnswerDisplay(
		IN 	HCALL		    hsCall,
                IN      char*               display);

RVAPI
int CALLCONV cmCallAnswer(
		IN      HCALL               hsCall);


RVAPI
int CALLCONV cmCallAccept(
		IN 	HCALL	 	    hsCall);

RVAPI
int CALLCONV cmCallDrop(
		IN      HCALL               hsCall);


RVAPI
int CALLCONV cmCallDropParam(
		IN 	HCALL	 	    hsCall,
                IN      cmReasonType        cmReason);
RVAPI
int CALLCONV cmCallClose(
		IN      HCALL               hsCall);

RVAPI
int CALLCONV cmCallGetCallingPartyId(
		IN      HCALL               hsCall,
		OUT     char*               callerId,
		IN      int                 size);

#define cmCallGetCallerId cmCallGetCallingPartyId

RVAPI
int CALLCONV cmCallGetCalledPartyId(
		IN      HCALL               hsCall,
		OUT     char*               calledPartyId,
		IN      int                 size);

RVAPI
int CALLCONV cmCallGetRate(
		IN      HCALL               hsCall,
		OUT     UINT32*             rate);

RVAPI
int CALLCONV cmCallGetOrigin(
		IN      HCALL               hsCall,
		OUT     BOOL*               origin);

RVAPI
int CALLCONV cmCallSetRate(
		IN      HCALL               hsCall,
		IN      UINT32              rate);


RVAPI
int CALLCONV cmCallForward(
		IN 	HCALL	 	hsCall,
		IN 	char* 		destAddress);


RVAPI
int CALLCONV cmCallGetParam(
		IN      HCALL               hsCall,
		IN      cmCallParam         param,
		IN      int                 index,
		IN      INT32*              value,
		IN      char *              svalue);

RVAPI
int CALLCONV cmCallSetParam(
		IN      HCALL               hsCall,
		IN      cmCallParam         param,
		IN      int                 index,
		IN      INT32               value,
		IN      char *              svalue);

RVAPI 
HPVT CALLCONV cmGetValTree(
                IN	HAPP		     hApp);



RVAPI
int CALLCONV cmCallGetNumOfParams(
		IN      HCALL               hsCall,
		IN      cmCallParam         param);


RVAPI 
HPST CALLCONV cmGetSynTreeByRootName(
                IN	HAPP		    hApp,
                IN      char*               name);

RVAPI 
INT32 CALLCONV cmGetQ931ConfigurationHandle(
                IN	HAPP		    hApp);


RVAPI
int CALLCONV cmCallJoin(
                IN      HCALL               hsCall,
                IN      HCALL               hsSameConferenceCall);

RVAPI
int CALLCONV cmCallInvite(
                IN      HCALL               hsCall,
                IN      HCALL               hsSameConferenceCall);

RVAPI
int CALLCONV cmCallIsSameConference(
		IN 	HCALL		    hsCall,
                IN 	HCALL		    hsAnotherCall);

RVAPI char* CALLCONV 
cmGetProtocolMessageName(
                IN      HAPP                hApp,
                IN      int                 msg);



RVAPI
int CALLCONV cmCallGetRemoteEndpointInfoHandle(
		IN 	HCALL		hsCall);

RVAPI
int CALLCONV cmCallGetLocalEndpointInfoHandle(
		IN 	HCALL		hsCall);

RVAPI
int CALLCONV cmGetEndpointInfoHandle(
		IN 	HAPP		hApp);



RVAPI
int CALLCONV cmGetParam(
		IN HAPP 		hApp,
		IN cmParam		param,
		IN int			index,
		IN INT32*		value,
		IN char *		svalue);


RVAPI int CALLCONV 
cmSetHandle(
	    IN      HAPP                hApp,
	    IN	    HAPPAPP		haApp);

RVAPI int CALLCONV 
cmGetHandle(
	    IN      HAPP                hApp,
	    IN	    HAPPAPP*		haApp);

/* nonStandard handling_______________________________________________________________*/

RVAPI int CALLCONV /* TRUE or ERROR */
cmNonStandardParameterCreate(
			     /* Create NonStandardParameter tree */
			     IN  HPVT hVal,
			     IN  int rootId, /* nonStandardData */

			     IN  cmNonStandardIdentifier *identifier,
			     IN  char *data,
			     IN  int dataLength /* in bytes */
			     );

RVAPI int CALLCONV /* TRUE or ERROR */
cmNonStandardParameterGet(
			  /* Convert NonStandardParameter tree to C structures */
			  IN  HPVT hVal,
			  IN  int rootId, /* nonStandardData */
			  
			  OUT cmNonStandardIdentifier *identifier,
			  OUT char *data,
			  INOUT INT32 *dataLength /* (in bytes) IN: data allocation. OUT: correct size */
			  );
RVAPI
int CALLCONV cmCallConnectControl(
                IN      HCALL               hsCall
                );

RVAPI
int CALLCONV 
cmCallSw2SeparateH245(IN      HCALL               hsCall);

RVAPI
int CALLCONV cmCallSetIndependentSupplementaryService(
                IN      HCALL               hsCall
                );

RVAPI
int CALLCONV cmCallSendSupplementaryService(
                IN      HCALL               hsCall,
		IN      void*		    buffer,
		IN	int		    bufferSize,
		IN      BOOL                force
		);
RVAPI 
int CALLCONV cmEmEncode( 
                IN      HPVT                valH,
	        IN      int	            vNodeId,
	        IN      BYTE*               buffer,
	        IN      int	            length,
	        OUT     int*                encoded);

RVAPI 
int CALLCONV cmEmDecode(
	        IN      HPVT                valH,
        	IN      int	            vNodeId,
	        OUT     BYTE*               buffer,
	        IN      int	            length,
	        OUT     int*                decoded);


RVAPI 
int CALLCONV cmAlias2Vt(
                IN      HPVT                valH,
	        IN      cmAlias*            alias,
	        IN      int                 nodeId);



RVAPI 
INT32 CALLCONV cmGetLocalCallSignalAddress(
                IN	HAPP		     hApp,
                OUT     cmTransportAddress*  tr);

RVAPI 
INT32 CALLCONV cmGetLocalRASAddress(
                IN	HAPP		     hApp,
                OUT     cmTransportAddress*  tr);

RVAPI 
INT32 CALLCONV cmGetGKCallSignalAddress(
                IN	HAPP		     hApp,
                OUT     cmTransportAddress*  tr);

RVAPI 
INT32 CALLCONV cmGetGKRASAddress(
                IN	HAPP		     hApp,
                OUT     cmTransportAddress*  tr);

RVAPI 
HPROTCONN CALLCONV cmGetRASConnectionHandle(
                IN	HAPP		     hApp);

RVAPI int CALLCONV cmVt2Alias(IN  HPVT       hVal,
	        IN      cmAlias*             alias,
	        IN      int                  nodeId);

RVAPI int CALLCONV cmGetProperty( 
                IN      HPROTOCOL	     hsProtocol);

RVAPI HAPP CALLCONV cmGetAHandle(
                IN      HPROTOCOL            hsProtocol);

RVAPI
int CALLCONV cmCallStatusEnquiry(
					IN 	HCALL	 	hsCall,
					UINT8 * display);


/* 
cmGetH225RemoteVersion
return H225 version number of remote side via OUT parameter
version.
version is user allocated string*/
								 
RVAPI
int CALLCONV cmGetH225RemoteVersion(IN HCALL		hsCall,OUT char * version);

/*
cmCallUserInformationCreate
create root of information message .Build all must parameters.
Add display and nonstandard parameter if they are and return root id to user to
do changes if it need
*/
RVAPI
int CALLCONV cmCallUserInformationCreate(IN HCALL hsCall,char * display,cmNonStandardParam* param);

/*
cmCallUserInformationSend
send user information message
*/
RVAPI
int CALLCONV cmCallUserInformationSend(IN HCALL hsCall,int nodeId);

/*
cmCallUserInformationGet
Get rootId of the user information message and retrieve display and nonStandard parameter
Root Id of incoming user information message is passed to callback function of  cmEvCallUserInfoT type 
if it is set by user.   
*/
RVAPI
int CALLCONV cmCallUserInformationGet(IN HAPP hApp,IN int nodeId,
				      OUT char *display,IN int displaySize,
				      OUT cmNonStandardParam* param);
/*
cmCallForwardCallProcMsg
Forward callProceeding message.Get it from call property.So to forward it ,message must be copied 
to property.
*/

RVAPI
int CALLCONV cmCallSendCallProceeding(	IN HCALL hsCall);

/* delete pvt of connect,setup,alerting and callProceeding from call property  to reduce memory
 can be run only in connected state.All information from this messages will be lost after runing
this function.*/

RVAPI
int CALLCONV cmFreeProperty(	IN HCALL hsCall);


RVAPI 
int CALLCONV cmCallOverlapSending(IN HCALL hsCall,char * address);



RVAPI
HPROTCONN CALLCONV cmGetTpktChanHandle(	IN HCALL hsCall,cmTpktChanHandleType  tpktChanHandleType);

RVAPI
HPROTCONN CALLCONV cmGetUdpChanHandle(	IN HCALL hsCall,cmUdpChanHandleType  udpChanHandleType);


RVAPI
int CALLCONV cmSetUdpChanApplHandle(	IN HPROTCONN hCon,HAPPCONN hAppConn);

RVAPI
int CALLCONV cmSetTpktChanApplHandle(	IN HPROTCONN hCon,HAPPCONN hAppConn);

RVAPI
int CALLCONV cmGetUdpChanApplHandle(	IN HPROTCONN hCon,HAPPCONN * hAppConn);

RVAPI
int CALLCONV cmGetTpktChanApplHandle(	IN HPROTCONN hCon,HAPPCONN * hAppConn);

RVAPI
int CALLCONV cmCallIncompleteAddress(IN HCALL hsCall);


RVAPI
int CALLCONV cmCallAddressComplete(IN HCALL hsCall);

RVAPI 
int CALLCONV cmCallOverlapSendingExt(IN HCALL hsCall,char * address,BOOL sendingComplete);

RVAPI int CALLCONV
cmSendRAI(
       IN      HAPP                hApp,
       IN      BOOL 	almostOutOfResources);

#define cmCallSetHandle cmSetCallHandle 
RVAPI
int CALLCONV cmSetCallHandle(IN HCALL hsCall, IN HAPPCALL haCall);

RVAPI
int CALLCONV cmCallGetHandle(IN HCALL hsCall, OUT HAPPCALL* haCall);

#include <ci.h>
/* cmGetConfiguration()

   Gives access to the session's configuration context.
   The returned HCFG should be considered read-only, and should never be
   destroyed. */
RVAPI 
HCFG CALLCONV cmGetConfiguration(IN HAPP hApp);


#endif

#ifdef __cplusplus
}              
#endif





