#ifdef __cplusplus
extern "C" {
#endif



/*

NOTICE:
This document contains information that is proprietary to RADVision LTD..
No part of this publication may be reproduced in any form whatsoever without 
written prior approval by RADVision LTD..

RADVision LTD. reserves the right to revise this publication and make changes 
without obligation to notify any person of such revisions or changes.

*/ 

#ifndef CMCTRL_H
#define CMCTRL_H

#ifndef CM_H
#error cmctrl.h included improperly. Use #include <cm.h> instead.
#else

#ifndef _TRANSPCAP_
#include <transpcap.h>
#endif



DECLARE_OPAQUE(HAPPCHAN);
DECLARE_OPAQUE(HCHAN);


typedef 
int
(*cmNewProtocolEH)(void);

/*
typedef 
int
(*cmNewProtocolEH)(
		IN	HPdlAProtocol		haProtocolParent,
		IN	HPdlSProtocol		hsProtocol,
		OUT	HPdlAProtocol*		lphaProtocol,
		IN	char*			protocolName,
		OUT	newProtocolEH*		newProtocol,
		OUT	newMessageEH*		newMessage,
                OUT     BOOL*                   selfDestructing         
);
------------------------------------------------------------------------- */


typedef enum
{
    cmChannelStateDialtone,
    cmChannelStateRingBack,
    cmChannelStateConnected,
    cmChannelStateDisconnected,
    cmChannelStateIdle,
    cmChannelStateOffering
} cmChannelState_e;

typedef enum
{
    cmChannelStateModeOn,
    cmChannelStateModeOff,

    cmChannelStateModeDisconnectedLocal,
    cmChannelStateModeDisconnectedRemote,
    cmChannelStateModeDisconnectedMasterSlaveConflict,
    cmChannelStateModeDuplex,
/* marina */
    cmChannelStateModeDisconnectedReasonUnknown,
    cmChannelStateModeDisconnectedReasonReopen,
    cmChannelStateModeDisconnectedReasonReservationFailure

} cmChannelStateMode_e;



typedef enum {
  cmCapReceive=1,
  cmCapTransmit=2,
  cmCapReceiveAndTransmit=3
} cmCapDirection;


typedef enum {
  cmCapEmpty=0,
  cmCapAudio=1,
  cmCapVideo=2,
  cmCapData=3,
  cmCapNonStandard=4,
  cmCapUserInput=5,
  cmCapConference=6,
  cmCapH235=7,
  cmCapMaxPendingReplacementFor=8
} cmCapDataType;



typedef struct {
  char *name;
  INT32 capabilityId; /* capabilityTableEntryNumber */
  int capabilityHandle; /* capability item message tree (video/audio/data/nonStandard) */
  cmCapDirection direction;
  cmCapDataType type;
} cmCapStruct;


typedef enum {
  cmCapAccept=1,
  cmCapReject
} cmCapStatus;


typedef enum {
  cmMSSlave=1,
  cmMSMaster,
  cmMSError
} cmMSStatus;



typedef enum {
  cmRequestCloseRequest, /* Received request to close channel */
  cmRequestCloseConfirm, /* Request to close this channel is confirmed */
  cmRequestCloseReject   /* Request to close this channel is rejected */
} cmRequestCloseStatus;


typedef enum {
  cmMediaLoopRequest,    /* Received request for a media loop on a channel */
  cmMediaLoopConfirm,    /* Media loop confirmed */
  cmMediaLoopOff         /* Media loop on channel is canceled */
} cmMediaLoopStatus;



typedef struct {
  char *data; /* general string formatted data (4 bytes/char) */
  int length; /* in bytes */
} cmUserInputData;


typedef enum {
  /* path in paranthesis indicates the nodeId syntax */
  cmReqModeRequest,    /* request mode (requestMode.requestedModes) */
  cmReqModeAccept,     /* request acknoledged (requestModeAck.response) */
  cmReqModeReject,     /* request rejected (requestModeReject.cause) */
  cmReqModeReleaseOut, /* Outgoing request released due to timeout (N/A) */
  cmReqModeReleaseIn   /* Incoming request released due to timeout (N/A) */
} cmReqModeStatus;


typedef enum {
  cmMiscMpOn,  /* multipoint mode command is on (set) */
  cmMiscMpOff  /* multipoint mode command is off (cancel) */
} cmMiscStatus;


typedef struct {
  char name[32];
  INT32 entryId; /* ModeElement node id */
} cmReqModeEntry;

typedef struct {
  UINT8 mcuNumber; /* constraint: 0..192 */
  UINT8 terminalNumber; /* constraint: 0..192 */
} cmTerminalLabel;


typedef enum {
  cmControlStateConnected,
  cmControlStateConference,    
  cmControlStateTransportConnected,
  cmControlStateTransportDisconnected,
/* marina */
  cmControlStateFastStart
} cmControlState;


typedef enum {
  cmControlStateModeNull
} cmControlStateMode;

/* marina */
typedef enum
{
  cmIssueJoin	=0,
  cmIssueQueryAndJoin,
  cmIssueCreate	,
  cmIssueInvite	,
  cmWaitForInvite,
  cmOriginateCall,
  cmWaitForCall,	
  cmIssueQuery	
} cmT120SetupProcedure;

typedef enum
{
  cmCloseReasonUnknown=0,
  cmCloseReasonReopen	,
  cmCloseReasonReservationFailure,
  cmCloseReasonNormal /* for request close only */
}cmCloseLcnReason;

typedef struct
{
  int redundancyEncodingMethodId;
  int secondaryEncodingId;
}cmRedundancyEncoding;

typedef enum
{
	cmVideoFreezePicture=0,
	cmVideoSendSyncEveryGOB,
	cmVideoSendSyncEveryGOBCancel,
	cmSwitchReceiveMediaOff,
	cmSwitchReceiveMediaOn,
	cmVideoFastUpdatePicture
}
cmMiscellaneousCommand;


#ifndef _FAST_START_H
#include <faststart.h>
#endif
#ifndef _REDENCOD_H
#include <redencod.h>
#endif
#ifndef _USER_INPUT_H
#include <userinput.h>
#endif

/* Callback function prototypes____________________________________________________________________________*/






/* --- Control session callback functions --- */

typedef int 
    (CALLCONV *cmEvCallCapabilitiesT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      cmCapStruct*        capabilities[]);

typedef int 
    (CALLCONV *cmEvCallCapabilitiesExtT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      cmCapStruct***      capabilities[]);

typedef int
    (CALLCONV *cmEvCallNewChannelT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      HCHAN               hsChan,
		OUT     LPHAPPCHAN          lphaChan);

typedef int
    (CALLCONV *cmEvCallCapabilitiesResponseT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      UINT32              status);

typedef int
    (CALLCONV *cmEvCallMasterSlaveStatusT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      UINT32              status);

typedef int
    (CALLCONV *cmEvCallRoundTripDelayT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      INT32               delay); /* positive: msec delay, negative: expiration */

typedef int
    (CALLCONV *cmEvCallUserInputT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      INT32               userInputId /* user input node id */
		);

typedef int
    (CALLCONV *cmEvCallRequestModeT)(
	     /* Request to transmit the following modes preferences */
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      cmReqModeStatus     status,
		IN      INT32               nodeId /* pvt nodeId handle according to status */
		); 


typedef int
    (CALLCONV *cmEvCallMiscStatusT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      cmMiscStatus        status);


typedef int
    (CALLCONV *cmEvCallControlStateChangedT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      UINT32              state,
		IN      UINT32              stateMode);


/* marina */

typedef int
    (CALLCONV *cmEvCallMasterSlaveT)(
		IN      HAPPCALL            haCall,
		IN      HCALL               hsCall,
		IN      UINT32              terminalType,
    IN      UINT32              statusDeterminationNumber);







RVAPI int CALLCONV
cmCallMasterSlaveDetermineExt(
			   /*  Start master slave determination request ,
          Acknowledge to master slave determination request 
          set terminalType and determinationNumber .
          */
			   IN 	HCALL	 	hsCall,
			   IN   int             terminalType,
         IN   int     determinationNumber);
    
typedef  struct 
{
  cmEvCallCapabilitiesT           cmEvCallCapabilities;
  cmEvCallCapabilitiesExtT        cmEvCallCapabilitiesExt;
  cmEvCallNewChannelT             cmEvCallNewChannel;
  cmEvCallCapabilitiesResponseT   cmEvCallCapabilitiesResponse;    
  cmEvCallMasterSlaveStatusT      cmEvCallMasterSlaveStatus;
  cmEvCallRoundTripDelayT         cmEvCallRoundTripDelay;
  cmEvCallUserInputT              cmEvCallUserInput;
  cmEvCallRequestModeT            cmEvCallRequestMode;
  cmEvCallMiscStatusT             cmEvCallMiscStatus;
  cmEvCallControlStateChangedT	  cmEvCallControlStateChanged;  
/* marina */
  cmEvCallMasterSlaveT            cmEvCallMasterSlave;
} SCMCONTROLEVENT,*CMCONTROLEVENT;

typedef SCMCONTROLEVENT SCMSESSIONEVENT; /* for backward compatability */
typedef CMCONTROLEVENT CMSESSIONEVENT;




/* --- Control channels callback functions --- */

    
typedef int 
    (CALLCONV *cmEvChannelStateChangedT)(
		IN      HAPPCHAN            haChan,
		IN      HCHAN               hsChan,
		IN      UINT32              state,
		IN      UINT32              stateMode);

typedef int 
    (CALLCONV *cmEvChannelNewRateT)(
		IN      HAPPCHAN            haChan,
		IN      HCHAN               hsChan,
		IN      UINT32              rate);

typedef int 
    (CALLCONV *cmEvChannelMaxSkewT)(
		IN      HAPPCHAN            haChan1,
		IN      HCHAN               hsChan1,
		IN      HAPPCHAN            haChan2,                
		IN      HCHAN               hsChan2,
		IN      UINT32              skew);

typedef int 
    (CALLCONV *cmEvChannelSetAddressT)(
		IN      HAPPCHAN            haChan,
		IN      HCHAN               hsChan,
		IN      UINT32              ip,
		IN      UINT16              port);

typedef int 
    (CALLCONV *cmEvChannelSetRTCPAddressT)(
		IN      HAPPCHAN            haChan,
		IN      HCHAN               hsChan,
		IN      UINT32              ip,
		IN      UINT16              port);

typedef int
    (CALLCONV *cmEvChannelParametersT)(
		IN      HAPPCHAN            haChan,
		IN      HCHAN               hsChan,
		IN      char*               channelName,
		IN      HAPPCHAN            haChanSameSession,
		IN      HCHAN               hsChanSameSession,
		IN      HAPPCHAN            haChanAssociated,
		IN      HCHAN               hsChanAssociated,
		IN      UINT32              rate);

typedef int 
    (CALLCONV *cmEvChannelRTPDynamicPayloadTypeT)(
		IN      HAPPCHAN            haChan,
		IN      HCHAN               hsChan,
		IN      INT8                dynamicPayloadType);

typedef int 
    (CALLCONV *cmEvChannelVideoFastUpdatePictureT)(
		IN      HAPPCHAN            haChan,
		IN      HCHAN               hsChan);

typedef int 
    (CALLCONV *cmEvChannelVideoFastUpdateGOBT)(
		IN      HAPPCHAN            haChan,
		IN      HCHAN               hsChan,
		IN      int                 firstGOB,
		IN      int                 numberOfGOBs);

typedef int 
    (CALLCONV *cmEvChannelVideoFastUpdateMBT)(
		IN      HAPPCHAN            haChan,
		IN      HCHAN               hsChan,
		IN      int                 firstGOB,
		IN      int                 firstMB,
		IN      int                 numberOfMBs);

typedef int 
    (CALLCONV *cmEvChannelHandleT)(
		  /* provide data type tree handle for this channel */
		IN      HAPPCHAN            haChan,
		IN      HCHAN               hsChan,
		IN      int                 dataTypeHandle,
		IN      cmCapDataType       dataType);


typedef int 
    (CALLCONV *cmEvChannelGetRTCPAddressT)(
		IN      HAPPCHAN            haChan,
		IN      HCHAN               hsChan,
		IN      UINT32*             ip,
		IN      UINT16*             port);

typedef int 
    (CALLCONV *cmEvChannelRequestCloseStatusT)(
		IN      HAPPCHAN              haChan,
		IN      HCHAN                 hsChan,
		IN      cmRequestCloseStatus  status);

typedef int 
    (CALLCONV *cmEvChannelTSTOT)(
		IN      HAPPCHAN              haChan,
		IN      HCHAN                 hsChan,
		IN      INT8                  isCommand, /* 0: indication, 1: command */
		IN      INT8                  tradeoffValue); /* 0-31 trade off value */

typedef int 
    (CALLCONV *cmEvChannelMediaLoopStatusT)(
		IN      HAPPCHAN              haChan,
		IN      HCHAN                 hsChan,
		IN      cmMediaLoopStatus     status);


/* marina */
typedef int
		(CALLCONV * cmEvChannelReplaceT)(
					 IN 	HAPPCHAN	 	haChan,
					 IN 	HCHAN	 	    hsChan,
					 IN 	HAPPCHAN 	  haReplacedChannel,
					 IN 	HCHAN	 	    hsReplacedChannel

					 );
typedef		int 
    (CALLCONV * cmEvChannelFlowControlToZeroT)(
					 IN 	   HAPPCHAN	 		        haChan	,
					 IN      HCHAN               hsChan
					 );
typedef int 
    (CALLCONV * cmEvChannelMiscCommandT)(
					 IN 	   HAPPCHAN	 		        haChan	,
					 IN      HCHAN               hsChan,
					 IN		 cmMiscellaneousCommand miscCommand
					 );

typedef int 
    (CALLCONV * cmEvChannelTransportCapIndT)(
					 IN 	   HAPPCHAN	 		        haChan	,
					 IN      HCHAN               hsChan,
					 IN		 int nodeId
					 );


typedef int 
        (CALLCONV *cmEvChannelSetNSAPAddressT)(
		IN      HAPPCHAN    	haChan,
		IN      HCHAN           	hsChan,
		IN      char*               	address,
		IN      int                   	length,
                IN      BOOL                    multicast);



typedef int 
        (CALLCONV *cmEvChannelSetATMVCT)(
		IN      HAPPCHAN   	haChan,
		IN      HCHAN           	hsChan,
		IN      int                   	portNumber);


typedef struct 
{
  cmEvChannelStateChangedT            cmEvChannelStateChanged;
  cmEvChannelNewRateT                 cmEvChannelNewRate;
  cmEvChannelMaxSkewT                 cmEvChannelMaxSkew;
  cmEvChannelSetAddressT              cmEvChannelSetAddress;
  cmEvChannelSetRTCPAddressT          cmEvChannelSetRTCPAddress;
  cmEvChannelParametersT              cmEvChannelParameters;
  cmEvChannelRTPDynamicPayloadTypeT   cmEvChannelRTPDynamicPayloadType;
  cmEvChannelVideoFastUpdatePictureT  cmEvChannelVideoFastUpdatePicture;
  cmEvChannelVideoFastUpdateGOBT      cmEvChannelVideoFastUpdateGOB;
  cmEvChannelVideoFastUpdateMBT       cmEvChannelVideoFastUpdateMB;
  cmEvChannelHandleT                  cmEvChannelHandle;
  cmEvChannelGetRTCPAddressT          cmEvChannelGetRTCPAddress;
  cmEvChannelRequestCloseStatusT      cmEvChannelRequestCloseStatus;
  cmEvChannelTSTOT                    cmEvChannelTSTO;
  cmEvChannelMediaLoopStatusT         cmEvChannelMediaLoopStatus;
/* marina */
  cmEvChannelReplaceT                 cmEvChannelReplace;
  cmEvChannelFlowControlToZeroT       cmEvChannelFlowControlToZero;  
  cmEvChannelMiscCommandT			  cmEvChannelMiscCommand;	
  cmEvChannelTransportCapIndT		  cmEvChannelTransportCapInd;

  cmEvChannelSetNSAPAddressT      cmEvChannelSetNSAPAddress;
  cmEvChannelSetATMVCT            cmEvChannelSetATMVC;      

} SCMCHANEVENT,*CMCHANEVENT;









/* General_______________________________________________________________________________________________*/






RVAPI int CALLCONV /* Real number of channels in configuration or ERROR */
cmGetConfigChannels(
		    /* build array containing the channel names as appear in 
		       the configuration. The strings are copied into array elements */
		    IN	HAPP		    hApp,
		    IN 	int		    arraySize,
		    IN  int                 elementLength, /* sizeof each string in array */
		    OUT	char*		    array[] /* allocated with elements */
		    );

RVAPI
int CALLCONV cmSetControlEventHandler(
		IN      HAPP                hApp,
		IN      CMCONTROLEVENT      cmControlEvent,
		IN      int                 size);

RVAPI
int CALLCONV cmGetControlEventHandler(
		IN      HAPP                hApp,
		OUT      CMCONTROLEVENT      cmControlEvent,
		IN      int                 size);

#define cmSetSessionEventHandler cmSetControlEventHandler /* for backward compatability */


RVAPI
int CALLCONV cmSetChannelEventHandler(
		IN      HAPP                hApp,
		IN      CMCHANEVENT         cmChannelEvent,
		IN      int                 size);


RVAPI
int CALLCONV cmGetChannelEventHandler(
		IN      HAPP                hApp,
		OUT      CMCHANEVENT         cmChannelEvent,
		IN      int                 size);



/* returns the value-tree node id of h245 configuration root node */
RVAPI 
INT32 CALLCONV cmGetH245ConfigurationHandle(
                IN	HAPP		     hApp);



RVAPI int CALLCONV /* TRUE or ERROR */
cmH245SetNewSessionProtocol(
			    /* Set new protocol callback function in case of new protocol
			       generation under h245Session state machine */
			    IN 	HAPP	 	hApp,
			    IN  cmNewProtocolEH newProtocolCallback
			    );



/* Control Session API_______________________________________________________________________________*/





RVAPI 
int CALLCONV cmCallCreateControlSession(
		IN 	 HCALL	 	     hsCall,
                IN OUT   cmTransportAddress* addr);



RVAPI int CALLCONV 
cmCallCloseControlSession(
			  /* Close the H.245 PDL protocol tree */
			  IN     HCALL               hsCall);


RVAPI int CALLCONV 
cmCallHasControlSession(
			  /* Checks for the H.245 PDL protocol tree */
			  IN     HCALL               hsCall);

/* --- master/slave --- */


RVAPI 
int CALLCONV cmCallMasterSlaveDetermine(
					/* Start master slave determination request */
					IN 	HCALL	 	hsCall,
					IN      int             terminalType
					);
#define cmMasterSlaveDetermine cmCallMasterSlaveDetermine


RVAPI int CALLCONV /* master/slave status of this call */
cmCallMasterSlaveStatus(
			IN 	HCALL	 	hsCall
			);








/* --- capabilities --- */

RVAPI int CALLCONV /* remote terminal capability set node id, or ERROR */
cmCallGetRemoteCapabilities(
			    /* Get latest remote terminal capability message */
			    IN 	HCALL	 	hsCall
			    );

RVAPI int CALLCONV /* TRUE or ERROR */
cmCallSendCapability(
		     /* Send terminal capability set tree to remote terminal */
		     /* Note: cap tree is not deleted! */
		     IN	 HCALL hsCall,
		     IN  int termCapSet  /* local terminal capabiliy set Id */
		     );


RVAPI int CALLCONV /* new terminalCapabilitySet node id. */
cmCallCapabilitiesBuild(
			/* Build terminalCapabilitySet value tree including the capability set
			   and capability descriptors. */
			IN 	HCALL		    hsCall,
			IN      cmCapStruct*        capSet[],
			IN      cmCapStruct***      capDesc[]
			);

RVAPI int CALLCONV
cmCallCapabilitiesSend(
		       IN      HCALL               hsCall,
		       IN      cmCapStruct*        capSet[],
		       IN      cmCapStruct***      capDesc[]
		       );

RVAPI int CALLCONV
cmCallGetLocalCapabilities(
			   /* translate local caps (from CESE_OUT machine) to application structures.
			      Usage: 
			       cmCapStruct capA[capSetASize];
			       cmCapStruct *capSetA[capSetASize];
			       void* capDescA[capDescASize];
			       cmCapStruct** capSet;
			       cmCapStruct**** capDesc;
			       int i;

			       for (i=0; i<capSetASize; i++) capSetA[i] = capA+i;
			       cmCallGetLocalCapabilities(hsCall, 
			                                  capSetA, capSetSize, 
			                                  capDescA, capDescASize,
							  &capSet, &capDesc);
							  */
			   
			   IN 	   HCALL		hsCall,
			   IN      cmCapStruct*         capSetA[], /* user allocated */
			   IN      int                  capSetASize,
			   IN      void*                capDescA[], /* user allocated */
			   IN      int                  capDescASize,
			   OUT     cmCapStruct**        capSet[],
			   OUT     cmCapStruct****      capDesc[]
			   );






/* --- alive --- */

RVAPI
int CALLCONV cmCallRoundTripDelay(
				  /* Measure the round trip delay to the remote terminal */
				  /* Note: maxDelay=0 --> delay taken from configuration */
				  IN 	HCALL	 	hsCall,
				  IN    INT32           maxDelay /* maximum waiting delay in seconds */
				  );



/* --- user input --- */


RVAPI int CALLCONV
cmCallSendUserInput(
		    /* send user input msg: userInputId tree is deleted */
		    /* Note: Select one of nonStandard or userData options */
		    IN  HCALL hsCall,
		    IN  INT32 userInputId /* indication.userInput tree node id */
		    );

RVAPI int CALLCONV  /* userInput message node id or ERROR */
cmUserInputBuildNonStandard(
			    /* Build userUser message with non-standard data */
			    IN  HAPP hApp,
			    IN  cmNonStandardIdentifier *identifier,
			    IN  char *data,
			    IN  int dataLength /* in bytes */
			    );

RVAPI int CALLCONV  /* userInput message node id or ERROR */
cmUserInputBuildAlphanumeric(
			     /* Build userUser message with alphanumeric data */
			     IN  HAPP hApp,
			     IN  cmUserInputData *userData
			     );


RVAPI int CALLCONV  /* TRUE or ERROR */
cmUserInputGet(
	       /* Note: One of dataLength or userData->length is NULL (the other exists) */
	       /* Note: pointers are set to addresses in the user input value tree */
	       IN  HAPP hApp,
	       IN  INT32 userInputId,

	       /* 1) nonStandard */
	       IN  cmNonStandardIdentifier *identifier, /* user allocated */
	       IN  char *data, /* user allocated */
	       INOUT INT32 *dataLength,  /* (in bytes) IN: data allocation. OUT: correct size */
	       
	       /* 2) userData */
	       IN  cmUserInputData *userData /*.length: IN: data allocation. OUT: correct size */
	       );


/* --- request mode --- */

RVAPI int CALLCONV
cmCallRequestMode(
		  /* send request mode msg */
		  /* Note: id is deleted when function returns */
		  IN  HCALL hsCall,
		  IN  INT32 modeDescriptorsId /* (requestMode.requestedModes) */
		  );

RVAPI int CALLCONV
cmCallRequestModeAck(
		     /* Acknowledge the request */
		     IN  HCALL hsCall,
		     IN  char* responseName /* (requestModeAck.response) */
		     );

RVAPI int CALLCONV
cmCallRequestModeReject(
			/* Reject the request */
			IN  HCALL hsCall,
			IN  char* causeName /* requestModeReject.cause */
			);



RVAPI int CALLCONV/* request mode node id or ERROR */
cmRequestModeBuild(
		   /* Build request mode msg */
		   /* Note: entryId overrides name */
		   IN  HAPP hApp,
		   IN  cmReqModeEntry **modes[] /* modes matrix in preference order. NULL terminated arrays */
		   );

RVAPI int CALLCONV/* TRUE or ERROR */
cmRequestModeStructBuild(
			 /* Build request mode matrix structure from msg */
			 /* Note: entry name as in h.245 standard */
			 IN  HAPP hApp,
			 IN  INT32 descId, /* requestMode.requestedModes node id */
			 IN  cmReqModeEntry **entries, /* user allocated entries */
			 IN  int entriesSize, /* number of entries */
			 IN  void **ptrs, /* pool of pointers to construct modes */
			 IN  int ptrsSize, /* number of ptrs */

			 /* modes matrix in preference order. NULL terminated arrays */
			 OUT cmReqModeEntry ***modes[]
			 );




/* Channels____________________________________________________________________________________*/





RVAPI int CALLCONV
cmChannelGetCallHandles(
			/* Get the stack and application call handles by the channel handle */
			IN  HCHAN hsChan,
			OUT HCALL *hsCall,
			OUT HAPPCALL *haCall
			);




RVAPI int CALLCONV
cmChannelNew(
	     IN      HCALL               hsCall,
	     IN      HAPPCHAN            haChan,
	     OUT     LPHCHAN             lphsChan);

RVAPI int CALLCONV
cmChannelSetHandle(
		   /* let haChan be the new application handle of this channel */
		   IN HCHAN                    hsChan,
		   IN HAPPCHAN                 haChan);



RVAPI
int CALLCONV cmChannelOpen(
		IN      HCHAN               hsChan,
		IN      char*               channelName,
		IN      HCHAN               hsChanSameSession,
		IN      HCHAN               hsChanAssociated,
		IN      UINT32              rate);

RVAPI
int CALLCONV cmChannelOpenDynamic(
		IN      HCHAN               hsChan,
		IN      int                 dataTypeHandle, /* data type (HPVT) tree handle */
		IN      HCHAN               hsChanSameSession,
		IN      HCHAN               hsChanAssociated,
		IN      BOOL                isDynamicPaloadType);


RVAPI
int CALLCONV cmChannelOn(
		IN      HCHAN               hsChan);

RVAPI
int CALLCONV cmChannelOff(
		IN      HCHAN               hsChan);

RVAPI
int CALLCONV cmChannelAnswer(
		IN      HCHAN               hsChan);

RVAPI
int CALLCONV cmChannelDrop(
		IN      HCHAN               hsChan);

RVAPI
int CALLCONV cmChannelClose(
		 IN     HCHAN               hsChan);


RVAPI int CALLCONV
cmChannelRequestCloseReject(
			    /* Reject the request for closing the outgoing channel. */
			    IN 	HCHAN	 	hsChan
			    );


/* --- Duplex channels --- */
RVAPI
int CALLCONV cmChannelDuplex(
			     /* declare channel as duplex. Meaning full duplex. Composed of 
				two uni-directional channels in opposite directions. */
		IN      HCHAN               hsChan);

RVAPI
int CALLCONV cmChannelIsDuplex(
			       /* Returns TRUE if channel is duplex. FALSE if not. and negative value
				  in case of error */
		 IN     HCHAN               hsChan);



RVAPI
int CALLCONV cmChannelSetDuplexAddress(
				       /* Set address of duplex connection */
		 IN     HCHAN               hsChan,
		 IN     cmTransportAddress  address,
		 IN     int                 externalReferenceLength,
		 IN     char*               externalReference, /* NULL if not exists */
		 IN     BOOL                isAssociated /* TRUE if associated */
		 );

RVAPI
int CALLCONV /* actual size of external reference or ERROR */
cmChannelGetDuplexAddress(
			  /* Get address of duplex connection */
		 IN     HCHAN               hsChan,
		 OUT    cmTransportAddress* address, /* User allocated structure */
		 IN     int                 externalReferenceLength, /* size of extRef buffer */
		 OUT    char*               externalReference, /* User allocated buffer */
		 OUT    BOOL*               isAssociated /* TRUE if associated */
		 );








/* Channel commands_____________________________________________________________________________________*/








RVAPI
int CALLCONV cmChannelMaxSkew(
		IN      HCHAN               hsChan1,
		IN      HCHAN               hsChan2,
		IN      UINT32              skew);

RVAPI
int CALLCONV cmChannelFlowControl(
		IN      HCHAN               hsChan,
		IN      UINT32              rate);

RVAPI
int CALLCONV cmChannelSetAddress(
		IN      HCHAN               hsChan,
		IN      UINT32              ip,
		IN      UINT16              port);

RVAPI
int CALLCONV cmChannelSetRTCPAddress(
		IN      HCHAN               hsChan,
		IN      UINT32              ip,
		IN      UINT16              port);

RVAPI
int CALLCONV cmChannelGetOrigin(
		IN      HCHAN               hsChan,
		OUT     BOOL*               origin);

RVAPI
int CALLCONV cmChannelSetDynamicRTPPayloadType(
		IN      HCHAN           hsChan,
		IN      INT8            dynamicPayloadType);

RVAPI
int CALLCONV cmChannelVideoFastUpdatePicture(
		IN      HCHAN               hsChan);

RVAPI
int CALLCONV cmChannelVideoFastUpdateGOB(
		IN      HCHAN               hsChan,
		IN      int                 firstGOB,
		IN      int                 numberOfGOBs);

RVAPI
int CALLCONV cmChannelVideoFastUpdateMB(
		IN      HCHAN               hsChan,
		IN      int                 firstGOB,
		IN      int                 firstMB,
		IN      int                 numberOfMBs);



RVAPI int CALLCONV
cmChannelTSTOCommand(
		     /* Send temporal spatial trade off command. Request the remote terminal to change
		      the tradeoff. */
		     IN HCHAN hsChan, /* incoming channel */
		     IN int tradeoffValue /* 0-31 */
		     );

RVAPI int CALLCONV
cmChannelTSTOIndication(
		     /* Send temporal spatial trade off indication. Indicates the current tradeoff value
		      of the local terminal. */
		     IN HCHAN hsChan, /* outgoing channel */
		     IN int tradeoffValue /* 0-31 */
		     );



/* Media loop____________________________________________________________________________*/

RVAPI int CALLCONV
cmChannelMediaLoopRequest(
			  /* Request media loop on this channel */
			  IN HCHAN hsChan /* outgoing channel */
			  );

RVAPI int CALLCONV
cmChannelMediaLoopConfirm(
			  /* Confirm media loop request on this channel */
			  IN HCHAN hsChan /* incoming channel */
			  );

RVAPI int CALLCONV
cmChannelMediaLoopReject(
			 /* Reject media loop request on this channel */
			 IN HCHAN hsChan /* incoming channel */
			 );

RVAPI int CALLCONV
cmCallMediaLoopOff(
		   /* Release all media loops in this call */
		   IN HCALL hsCall
		   );




RVAPI 
int CALLCONV cmChannelSameSession(
		     /* get the same session opposite channel of hsChan */
		     IN 	HCHAN	 	    hsChan,
		     OUT 	HAPPCHAN* 	    haSameSession,
		     OUT 	HCHAN*	 	    hsSameSession
		     );

RVAPI int CALLCONV /* returns the session id for this channel */
cmChannelSessionId(
		   /* get the session id of channel */
		   IN 	HCHAN	 	    hsChan
		   );



/* replace exist channel with new */
RVAPI int CALLCONV
cmChannelReplace(
					 IN      HCHAN          hsChan, /* openning outgoing channel,that proposed to replaced exist channel*/  
					 IN      HCHAN          hsChanReplace);/* exist outgoing channel,that must be replaced */



/* check if channel is replacing  incoming exist channel
return TRUE if  it is and return in haReplaceChannel and hsReplaceChannel
handles of replaced  channel.
*/

RVAPI int CALLCONV
cmIsChannelReplace(
		     IN 	HCHAN	 	    hsChan,
		     OUT 	HAPPCHAN* 	    haReplaceChannel,
		     OUT 	HCHAN*	 	    hsReplaceChannel
		     );


/* set establishing outgoing channel hsChan dependent on base channel with handle hsChanBase*/
RVAPI int CALLCONV
cmChannelSetDependency(
		     IN      HCHAN               hsChan,
         IN      HCHAN               hsChanBase     );


/* check if channel is dependent on another channel
return TRUE if  it is and return in haBaseChannel and hsBaseChannel
handles of base channel.
*/
RVAPI int CALLCONV
cmChannelGetDependency(
					 IN 	HCHAN	 	    hsChan,
					 OUT 	HAPPCHAN* 	    haBaseChannel,
					 OUT 	HCHAN*	 	    hsBaseChannel);



RVAPI int CALLCONV
cmChannelSetT120Setup(
		     IN      HCHAN               hsChan,
         IN      cmT120SetupProcedure t120SetupProc);

RVAPI int CALLCONV
cmChannelGetT120Setup(
		     IN      HCHAN               hsChan,
         OUT      cmT120SetupProcedure *t120SetupProc);


/* set FlowControlToZero on incoming channel during establising process
  used before answering */

RVAPI int CALLCONV
cmChannelSetFlowControlToZero(
		     IN      HCHAN               hsChan,
         IN      BOOL   flowControl);



RVAPI int CALLCONV
cmChannelSetTransportCapability(
		     IN      HCHAN               hsChan,
         int transportCapId    );

RVAPI int CALLCONV
cmChannelGetTransportCapabilityId(
		     IN      HCHAN               hsChan);

RVAPI int CALLCONV
cmChannelSetRedundancyEncoding(
		     IN      HCHAN               hsChan,
         IN     cmRedundancyEncoding * redundancyEncoding);

RVAPI int CALLCONV
cmChannelGetRedundancyEncoding(
		     IN      HCHAN               hsChan,
         OUT     cmRedundancyEncoding * redundancyEncoding);


RVAPI int CALLCONV
cmChannelSetSource(
		     IN      HCHAN               hsChan,
         IN     cmTerminalLabel *terminalLabel);

RVAPI int CALLCONV
cmChannelGetSource(
		     IN      HCHAN               hsChan,
         OUT     cmTerminalLabel *terminalLabel);

/* -------------------- close logical channels ----------------- */

RVAPI int CALLCONV
cmChannelDropReason(
	      /*
		- Close outgoing channel 
		- Reject incoming channel open request
		- New: Request to close incoming channel
		- On the other side the close lcn reason is passed via rejectStateMode to the
		  cmEvChannelStateChanged callback function.		
	      */
			IN 	HCHAN	 	hsChan,
			IN  cmCloseLcnReason reason
	      );


RVAPI int CALLCONV
cmChannelRequestClose(
	      /*
		- New: Request to close incoming channel
	      */
	      IN 	HCHAN	 	hsChan,
        IN  cmCloseLcnReason reason,
        cmQosCapability * cmQOSCapability
	      );

/* get on the outgoing side request to close channel ,reason and Qos capability of remoteendpoint */
RVAPI int CALLCONV
cmChannelGetRequestCloseParam(	IN 	HCHAN	 	hsChan,
                                OUT  cmCloseLcnReason *reason,
                                OUT cmQosCapability * cmQOSCapability
                               );

/* Multipoint operations_______________________________________________________________________________*/




RVAPI int CALLCONV
cmCallStartConference(
		      /* enter call conference mode */
		      IN      HCALL               hsCall
		      );

RVAPI int CALLCONV
cmCallCancelConference(
		       /* cancel the call conference mode */
		       IN      HCALL               hsCall
		       );



RVAPI int CALLCONV
cmCallSetTerminalLabel(
		       /* Set the remote endpoint terminal label. */
		       IN      HCALL               hsCall,
		       IN      cmTerminalLabel*    terminalLabel
		       );

RVAPI int CALLCONV /* ERROR if terminal label is not defined for this terminal */
cmCallGetTerminalLabel(
		       /* Get the local endpoint terminal label. */
		       IN      HCALL               hsCall,
		       OUT     cmTerminalLabel*    terminalLabel /* user allocated */
		       );


/* When the conference becomes active, the master (MC) terminal shall call cmCallDeclareMC() to
   indicate the new conference status. */
RVAPI 
int CALLCONV cmCallDeclareMC(
			     /* declare this terminal to be the MC of the call */
		IN      HCALL               hsCall
		);




/* When the conference becomes active, this function provides the address of the Active MC. */
RVAPI 
int CALLCONV cmCallGetMCAddress(
				/* get active MC address */
		IN      HCALL               hsCall,
		IN      UINT32*             ip,
		IN      UINT16*             port
		);





typedef struct {
  HCHAN hsChan; /* outgoing: new reverse channel. incoming: keep the existing channel 
		   null: new not associated reverse channel. */

  /* ! set the following fields only for new outgoing channels */

  HCHAN hsChanAssociate; /* associate channel. FFS */

  char *channelName; /* channel name from configuration or NULL for handle setting. */
  INT32 channelDataTypeHandle; /* data type (HPVT) tree handle. channelName should be NULL. */
  
  cmTerminalLabel terminalLabel; /* Use 0xffff if label unavailable */

  char *sessionDescription;
  int sessionDescriptionLength; /* number of bytes in sessionDescription */

  cmTransportAddress rtpAddress;
  cmTransportAddress rtcpAddress;
/* marina used only in cmCallSetChannelsExt function */
  int uid; /* uniq id may be index in array.*/
  int dependency; /* uniq id of element,which describe the channel,that this channel is dependent on */
  /* if its value is equal -1 ,parameter is not used. */
  int redEncodingId; /* node id of redundancyEncoding pvt,built by user. */

} cmChannelStruct;


/* The master (MC) terminal shall use cmCallSetChannels() to force the remote terminal to open
   the indicated channels for transmit. */
RVAPI 
int CALLCONV cmCallSetChannels(
			     /* As an Active MC, set transfer channels for remote terminal */
		IN      HCALL               hsCall,
		IN      int                 channelSetSize, /* number of elements in channelSet */
		IN      cmChannelStruct     channelSet[]
		);

/*marina*/
/* The master (MC) terminal shall use cmCallSetChannels() to force the remote terminal to open
   the indicated channels for transmit.Add few parameters specified in version 3,such as redEncoding
   and sessionDependancy */

RVAPI 
int CALLCONV cmCallSetChannelsExt(
			     /* As an Active MC, set transfer channels for remote terminal */
		IN      HCALL               hsCall,
		IN      int                 channelSetSize, /* number of elements in channelSet */
		IN      cmChannelStruct     channelSet[]
		);



/*   multipoint mode set/cancel operations */

RVAPI int CALLCONV
cmCallMultipointCommand(
		  /* Send multipoint command (on or off) message */
		  IN  HCALL hsCall,
		  IN  BOOL isModeOn /* TRUE: mp mode (on), FALSE: cancel mp mode (off) */
		  );


RVAPI int CALLCONV
cmCallMultipointStatus(
		  /* Get the multipoint mode status (on or off) */
		  IN  HCALL hsCall,
		  IN  BOOL *isModeOn /* TRUE: mp mode (on), FALSE: cancel mp mode (off) */
		  );


RVAPI int CALLCONV
cmChannelSendTransportCapInd(
	     IN 	HCHAN	 	hsChan,
		 IN		int			nodeId);  /* node Id of type TransportCapability */

RVAPI int CALLCONV
cmChannelSendMiscCommand(
	     IN 	HCHAN	 	hsChan,
		 IN		cmMiscellaneousCommand miscCommand);


/* delete pvt of remote capability to reduce memory*/
RVAPI 
int CALLCONV cmFreeCapability(	IN HCALL hsCall);


RVAPI 
int CALLCONV cmChannelSetNSAPAddress(
		IN      HCHAN           	hsChan,
		IN      char*               	address,
		IN      int                   	length,
                IN      BOOL                    multicast);





RVAPI 
int CALLCONV cmChannelSetATMVC(
		IN      HCHAN           	hsChan,
		IN      int                   	portNumber);


RVAPI
INT32 CALLCONV cmChannelGetNumber(IN HCHAN hsChan);





#endif
#endif
#ifdef __cplusplus
}              
#endif



