#ifdef __cplusplus
extern "C" {
#endif

/*
***********************************************************************************

NOTICE:
This document contains information that is proprietary to RADVision LTD..
No part of this publication may be reproduced in any form whatsoever without 
written prior approval by RADVision LTD..

RADVision LTD. reserves the right to revise this publication and make changes 
without obligation to notify any person of such revisions or changes.

***********************************************************************************
*/


typedef enum
{
    cmChannelStateDialtone,
    cmChannelStateRingBack,
    cmChannelStateConnected,
    cmChannelStateDisconnected,
    cmChannelStateIdle,
    cmChannelStateOffering,
    cmChannelStateRequestClose /* Remote terminal requests closing of an outgoing channel */
} cmChannelState_e;

RVAPI
int CALLCONV cmChannelOpenMulticast(
				    /* open a channel with multicast address */
		IN      HCHAN               hsChan,
		IN      char*               channelName,
		IN      HCHAN               hsChanSameSession,
		IN      HCHAN               hsChanAssociated,
		IN      UINT32              rate);


/* for incoming channel: it requests the remote terminal to close its outgoing channel */
RVAPI
int CALLCONV cmChannelDrop(
		IN      HCHAN               hsChan);

/* upon cmChannelStateRequestClose state notification: The user may reject the close 
   request by calling cmChannelDropReject(). Calling cmChannelDrop() causes the channel
   to be closed.
   */
RVAPI
int CALLCONV cmChannelDropReject(
				 /* reject close request on outgoing channel */
		IN      HCHAN               hsChan);


/* -- Multipoint commands */

/* When the conference becomes active, the master (MC) terminal shall call cmCallDeclareMC() to
   indicate the new conference status. */
RVAPI 
int CALLCONV cmCallDeclareMC(
			     /* declare this terminal to be the MC of the call */
		IN      HCALL               hsCall
		);


typedef struct {
  UINT8 mcuNumber;
  UINT8 terminalNumber;
} cmTerminalLabel;

typedef struct {
  UINT32 ip;
  UINT16 port;
  BOOL isMulticast; /* TRUE if multicast ip */
} cmIpAddress;


typedef struct {
  int sessionId; /* use 0 for stack decision */
  int associatedSessionId;
  cmTerminalLabel terminalLabel;
  char *sessionDescription;
  char *channelName;
  INT32 channelDataTypeHandle; /* HVT node id of channel data type */
  cmCapDataType type;
  cmIpAddress rtpAddress;
  cmIpAddress rtcpAddress;
} cmChannelStruct;

/* The master (MC) terminal shall use cmCallSetChannels() to force the remote terminal to open
   the indicated channels for transmit. */
RVAPI 
int CALLCONV cmCallSetChannels(
			     /* As an Active MC, set transfer channels for remote terminal */
		IN      HCALL               hsCall,
		IN      cmChannelStruct     channelSet[]
		);

#ifdef __cplusplus
}              
#endif



